/*---------------------------------------------------------------------------*
  Project:  TwlSDK - library - camera
  File:     camera_sp.c

  Copyright 2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Log: $
  $NoKeywords: $
 *---------------------------------------------------------------------------*/
#include <twl.h>
#include <twl/camera.h>

/*---------------------------------------------------------------------------*
    萔`
 *---------------------------------------------------------------------------*/
#define CAMERA_PXI_SIZE_CHECK(nums)                                                     \
    if (cameraWork.total != (nums)) {                                                   \
        CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_INVALID_PARAMETER);    \
        break;                                                                          \
    }

// ACgăRs[
#define CAMERA_UNPACK_U16(d, s)    \
    (*(d) = (u16)((((u8*)s)[0] << 0) | (((u8*)s)[1] << 8)))
#define CAMERA_UNPACK_U32(d, s)    \
    (*(d) = (u32)((((u8*)s)[0] << 0) | (((u8*)s)[1] << 8) | (((u8*)s)[2] << 16) | (((u8*)s)[3] << 24)))

#define CAMERA_SET_GPIO(r)      ((r) |= CAMERA_STBYN_MASK)
#define CAMERA_CLEAR_GPIO(r)    ((r) &= ~CAMERA_STBYN_MASK)

/*---------------------------------------------------------------------------*
    ^`
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
    ÓIϐ`
 *---------------------------------------------------------------------------*/
static BOOL cameraInitialized;          // mFtO
static CAMERAWork cameraWork;           // [Nϐ܂Ƃ߂\

/*---------------------------------------------------------------------------*
    ֐`
 *---------------------------------------------------------------------------*/
static void CameraPxiCallback(PXIFifoTag tag, u32 data, BOOL err);
static void CameraReturnResult(CAMERAPxiCommand command, CAMERAPxiResult result);
static void CameraReturnResultEx(CAMERAPxiCommand command, CAMERAPxiResult result, u8 size, u8* data);
static void CameraThread(void *arg);

/*---------------------------------------------------------------------------*
  Name:         CAMERA_Init

  Description:  CAMERACuB

  Arguments:    priority

  Returns:      None.
 *---------------------------------------------------------------------------*/
void CAMERA_Init(u32 priority)
{
    // ς݂mF
    if (cameraInitialized)
    {
        return;
    }
    cameraInitialized = 1;

    // [N
    cameraWork.camera = CAMERA_SELECT_NONE;

    // PXI֘A
    PXI_Init();
    PXI_SetFifoRecvCallback(PXI_FIFO_TAG_CAMERA, CameraPxiCallback);

    // sXbh쐬
    OS_InitMessageQueue(&cameraWork.msgQ, cameraWork.msgArray, CAMERA_MESSAGE_ARRAY_MAX);
    OS_CreateThread(&cameraWork.thread,
                    CameraThread,
                    0,
                    (void *)(cameraWork.stack + (CAMERA_THREAD_STACK_SIZE / sizeof(u64))),
                    CAMERA_THREAD_STACK_SIZE, priority);
    OS_WakeupThreadDirect(&cameraWork.thread);
}

/*---------------------------------------------------------------------------*
  Name:         CameraPxiCallback

  Description:  PXIoRŎMf[^͂B

  Arguments:    tag -  PXIʂ^OB
                data - Mf[^B26bitLB
                err -  PXIʐMɂG[tOB
                       ARM9ɂēʂPXIĂȂƂB

  Returns:      None.
 *---------------------------------------------------------------------------*/
static void CameraPxiCallback(PXIFifoTag tag, u32 data, BOOL err)
{
#pragma unused( tag )

    // PXIʐMG[`FbN
    if (err)
    {
        return;
    }
    // 擪f[^
    if (data & CAMERA_PXI_START_BIT)
    {
        cameraWork.total = (u8)((data & CAMERA_PXI_DATA_NUMS_MASK) >> CAMERA_PXI_DATA_NUMS_SHIFT);
        cameraWork.current = 0;
        cameraWork.command = (CAMERAPxiCommand)((data & CAMERA_PXI_COMMAND_MASK) >> CAMERA_PXI_COMMAND_SHIFT);
        cameraWork.data[cameraWork.current++] = (u8)((data & CAMERA_PXI_1ST_DATA_MASK) >> CAMERA_PXI_1ST_DATA_SHIFT);
//OS_TPrintf("START_BIT (total=%d, command=%X).\n", cameraWork.total, cameraWork.command);
    }
    // 㑱f[^
    else
    {
        cameraWork.data[cameraWork.current++] = (u8)((data & 0xFF0000) >> 16);
        cameraWork.data[cameraWork.current++] = (u8)((data & 0x00FF00) >> 8);
        cameraWork.data[cameraWork.current++] = (u8)((data & 0x0000FF) >> 0);
    }

    // pPbg
    if (cameraWork.current >= cameraWork.total)   // ő2]Ɏ擾
    {
        // MR}h
        switch (cameraWork.command)
        {
            // m̃R}hQ
        case CAMERA_PXI_COMMAND_INIT:
        case CAMERA_PXI_COMMAND_ACTIVATE:
        case CAMERA_PXI_COMMAND_OUTPUT_WITH_DUAL_ACTIVATION:
        case CAMERA_PXI_COMMAND_RESIZE:
        case CAMERA_PXI_COMMAND_FRAME_RATE:
        case CAMERA_PXI_COMMAND_EFFECT:
        case CAMERA_PXI_COMMAND_FLIP:
            // XbhĊJ
            if (!OS_SendMessage(&cameraWork.msgQ, NULL, OS_MESSAGE_NOBLOCK))
            {
                CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_FATAL_ERROR);
            }
            break;

            // m̃R}h
        default:
            CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_INVALID_COMMAND);
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:         CameraReturnResult

  Description:  PXIoRŏʂARM9ɑMB

  Arguments:    command     - ΏۃR}h
                result      - CAMERAPxiResult̂ЂƂ

  Returns:      None.
 *---------------------------------------------------------------------------*/
static void CameraReturnResult(CAMERAPxiCommand command, CAMERAPxiResult result)
{
    u32 pxiData = (u32)(CAMERA_PXI_START_BIT | CAMERA_PXI_RESULT_BIT |
            ((command << CAMERA_PXI_COMMAND_SHIFT) & CAMERA_PXI_COMMAND_MASK) |
            ((1 << CAMERA_PXI_DATA_NUMS_SHIFT) & CAMERA_PXI_DATA_NUMS_MASK) |
            ((result << CAMERA_PXI_1ST_DATA_SHIFT) & CAMERA_PXI_1ST_DATA_MASK));
    while (0 > PXI_SendWordByFifo(PXI_FIFO_TAG_CAMERA, pxiData, 0))
    {
    }
}

/*---------------------------------------------------------------------------*
  Name:         CameraReturnResultEx

  Description:  w㑱f[^PXIoRARM7ɑMB

  Arguments:    command     - ΏۃR}h
                result      - CAMERAPxiResult̂ЂƂ
                size        - tf[^TCY
                data        - tf[^

  Returns:      None.
 *---------------------------------------------------------------------------*/
static void CameraReturnResultEx(CAMERAPxiCommand command, CAMERAPxiResult result, u8 size, u8* data)
{
    u32 pxiData = (u32)(CAMERA_PXI_START_BIT | CAMERA_PXI_RESULT_BIT |
            ((command << CAMERA_PXI_COMMAND_SHIFT) & CAMERA_PXI_COMMAND_MASK) |
            (((size+1) << CAMERA_PXI_DATA_NUMS_SHIFT) & CAMERA_PXI_DATA_NUMS_MASK) |
            ((result << CAMERA_PXI_1ST_DATA_SHIFT) & CAMERA_PXI_1ST_DATA_MASK));
    int i;
    while (0 > PXI_SendWordByFifo(PXI_FIFO_TAG_CAMERA, pxiData, 0))
    {
    }
    for (i = 0; i < size; i += 3)
    {
        pxiData = (u32)((data[i] << 16) | (data[i+1] << 8) | data[i+2]);
        while (0 > PXI_SendWordByFifo(PXI_FIFO_TAG_CAMERA, pxiData, 0))
        {
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:         CameraThread

  Description:  CAMERA̎sXbhB

  Arguments:    arg - gpȂB

  Returns:      None.
 *---------------------------------------------------------------------------*/
static void CameraThread(void *arg)
{
#pragma unused( arg )

    OSMessage msg;
    BOOL result;
    u16 data16a;
    u16 data16b;

    while (TRUE)
    {
        // bZ[Ws܂ŐQ
        (void)OS_ReceiveMessage(&(cameraWork.msgQ), &msg, OS_MESSAGE_BLOCK);

        // R}hɏ]Ċe폈s
        switch (cameraWork.command)
        {
        case CAMERA_PXI_COMMAND_INIT:
            CAMERA_PXI_SIZE_CHECK(CAMERA_PXI_SIZE_INIT);
            // StandbyłȂStandbyԂɂ (TODO: o͂Ȃ̂܂܏ł悤ɂȂ΂ȂȂI)
            if (cameraWork.camera != CAMERA_SELECT_NONE)
            {
                if (FALSE == CAMERA_I2CStandby(cameraWork.camera))
                {
                    CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_ILLEGAL_STATUS);    // ARM9ɏ̎sʒB
                    break;
                }
            }
            result = CAMERA_I2CInit((CameraSelect)cameraWork.data[0]);
            cameraWork.camera = CAMERA_SELECT_NONE; // Ԃ͗Standby
            CameraReturnResult(cameraWork.command, result ? CAMERA_PXI_RESULT_SUCCESS : CAMERA_PXI_RESULT_ILLEGAL_STATUS);  // ARM9ɏ̐ʒB
            break;

        case CAMERA_PXI_COMMAND_ACTIVATE:
            CAMERA_PXI_SIZE_CHECK(CAMERA_PXI_SIZE_ACTIVATE);
            // ł́ABOTH͐ݒs
            if (CAMERA_SELECT_BOTH == cameraWork.data[0])
            {
                CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_INVALID_PARAMETER);    // ARM9ɏ̎sʒB
                break;
            }
            // ύXȂ؂ւ
            if (cameraWork.camera != cameraWork.data[0])
            {
                // StandbyłȂStandbyԂɂ
                if (cameraWork.camera != CAMERA_SELECT_NONE)
                {
                    if (FALSE == CAMERA_I2CStandby(cameraWork.camera))
                    {
                        CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_ILLEGAL_STATUS);    // ARM9ɏ̎sʒB
                        break;
                    }
                }
                cameraWork.camera = (CameraSelect)cameraWork.data[0];   // ANeBuȃJ̋L^
                // Standbyɂ̂ł͂Ȃ΁Aw肳ꂽANeBuɂ
                if (cameraWork.camera != CAMERA_SELECT_NONE)
                {
                    if (FALSE == CAMERA_I2CResume(cameraWork.camera))   // Standby
                    {
                        CAMERA_I2CStandby(CAMERA_SELECT_BOTH);  // r[ȏԑ΍
                        cameraWork.camera = CAMERA_SELECT_NONE; // NONEɖ߂Ă
                        CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_ILLEGAL_STATUS);    // ARM9ɏ̎sʒB
                        break;
                    }
                }
            }
            CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_SUCCESS);     // ARM9ɏ̐ʒB
            break;

        case CAMERA_PXI_COMMAND_OUTPUT_WITH_DUAL_ACTIVATION:
            CAMERA_PXI_SIZE_CHECK(CAMERA_PXI_SIZE_OUTPUT_WITH_DUAL_ACTIVATION);
            // ł́AIN/OUT̂ݐݒ
            if (CAMERA_SELECT_IN != cameraWork.data[0] && CAMERA_SELECT_OUT != cameraWork.data[0])
            {
                CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_INVALID_PARAMETER);    // ARM9ɏ̎sʒB
                break;
            }
            // (Ԃۑ) ɐݒ肷
            cameraWork.camera = CAMERA_SELECT_BOTH; // ANeBuȃJ̋L^
            if (FALSE == CAMERA_I2CResumeBoth((CameraSelect)cameraWork.data[0]))
            {
                CAMERA_I2CStandby(CAMERA_SELECT_BOTH);  // r[ȏԑ΍
                cameraWork.camera = CAMERA_SELECT_NONE; // NONEɖ߂Ă
                CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_ILLEGAL_STATUS);    // ARM9ɏ̎sʒB
                break;
            }
            CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_SUCCESS);     // ARM9ɏ̐ʒB
            break;

        case CAMERA_PXI_COMMAND_RESIZE:
            CAMERA_PXI_SIZE_CHECK(CAMERA_PXI_SIZE_RESIZE);
            CAMERA_UNPACK_U16(&data16a, &cameraWork.data[1]);
            CAMERA_UNPACK_U16(&data16b, &cameraWork.data[3]);
            result = CAMERA_I2CResize((CameraSelect)cameraWork.data[0], data16a, data16b);
            CameraReturnResult(cameraWork.command, result ? CAMERA_PXI_RESULT_SUCCESS : CAMERA_PXI_RESULT_ILLEGAL_STATUS);  // ARM9ɏ̐ۂʒB
            break;

        case CAMERA_PXI_COMMAND_FRAME_RATE:
            CAMERA_PXI_SIZE_CHECK(CAMERA_PXI_SIZE_FRAME_RATE);
            result = CAMERA_I2CFrameRate((CameraSelect)cameraWork.data[0], cameraWork.data[1]);
            CameraReturnResult(cameraWork.command, result ? CAMERA_PXI_RESULT_SUCCESS : CAMERA_PXI_RESULT_ILLEGAL_STATUS);  // ARM9ɏ̐ۂʒB
            break;

        case CAMERA_PXI_COMMAND_EFFECT:
            CAMERA_PXI_SIZE_CHECK(CAMERA_PXI_SIZE_EFFECT);
            result = CAMERA_I2CEffect((CameraSelect)cameraWork.data[0], (CameraEffect)cameraWork.data[1]);
            CameraReturnResult(cameraWork.command, result ? CAMERA_PXI_RESULT_SUCCESS : CAMERA_PXI_RESULT_ILLEGAL_STATUS);  // ARM9ɏ̐ۂʒB
            break;

        case CAMERA_PXI_COMMAND_FLIP:
            CAMERA_PXI_SIZE_CHECK(CAMERA_PXI_SIZE_FLIP);
            result = CAMERA_I2CFlip((CameraSelect)cameraWork.data[0], (CameraFlip)cameraWork.data[1]);
            CameraReturnResult(cameraWork.command, result ? CAMERA_PXI_RESULT_SUCCESS : CAMERA_PXI_RESULT_ILLEGAL_STATUS);  // ARM9ɏ̐ۂʒB
            break;

            // T|[gȂR}h
        default:
            CameraReturnResult(cameraWork.command, CAMERA_PXI_RESULT_INVALID_COMMAND);
        }
    }
}
