/*---------------------------------------------------------------------------*
  Project:  TWL - SD low-level driver
  File:     sdif.c

  Copyright 2006,2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include <twl.h>
#include "sdmc_config.h"
#include "sdif_reg.h"       /*  IP ΉWX^` */
#include <twl/sdmc.h>
#include "sdif_ip.h"        /*  IP ΉtO` */

//#if (SD_DEBUG_PRINT_ON == 1)
//    #define PRINTDEBUG    OS_TPrintf
//#else
    #define PRINTDEBUG( ...) ((void)0)
//#endif


#define ADD_CHECK 1



/***********************************************************************
    O[o
***********************************************************************/
u16    SD_CID[8];        /* CIDۑp (Card IDentification register) : ID*/
u16    SD_CSD[8];        /* CSDۑp (Card Specific Data register) : spec*/
u16    SD_OCR[2];        /* OCRۑp (Operation Condition Register) : voltage and status*/
u16    SD_SCR[4];        /* SCRۑp (Sd card Configulation Register) : bus-width, card-ver, etc*/
u16    SD_RCA;           /* RCAۑp (Relative Card Address register) : address*/

s16    SDCARD_MMCFlag;   /* MMCJ[htO */
s16    SDCARD_SDHCFlag;  /* SDHCJ[htO(łPhysicalLayer2.0̈) */
u16    SD_port_number;   /* ݂̃|[gԍ */

/***********************************************************************
    OQƕϐ
***********************************************************************/
extern volatile u16    SDCARD_ErrStatus;    /* G[Xe[^X */
extern volatile u32    SDCARD_Status;       /* J[hXe[^X */
//extern volatile s16    SDCARD_OutFlag;    /* J[hrotO */
//extern void (*func_SDCARD_Out)(void);     /* J[hroCxgpR[obNۑp */





void SD_DisableInfo( void);



/*---------------------------------------------------------------------------*
  Name:         SD_Init

  Description:  reset and initialize SD card interface
                SDJ[hIP̃ZbgƏݒ

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_Init(void)
{
    SD_AndFPGA( SOFT_RST,(~(SOFT_RST_SDIF_RST))); /* SD I/F W[Zbg */
    SD_OrFPGA( SOFT_RST,((SOFT_RST_SDIF_RST)));   /* SD I/F W[ZbgA */
    
    SD_AndFPGA( SD_STOP,(~SD_STOP_STP));          /* f[^]INA */
}

/*---------------------------------------------------------------------------*
  Name:         SD_EnableInfo

  Description:  enable SD card insert and remove interrupts.
                SDJ[h̑}/ 荞݂

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_EnableInfo( void)
{
    if(SD_port_number == SDCARD_PORT0)
    {
        SD_AndFPGA(SD_INFO1,(~(SD_INFO1_INSERT | SD_INFO1_REMOVE))); /* SD_INFO1WX^ card inserted removedNA */
        SD_AndFPGA(SD_INFO1_MASK,(~(SD_INFO1_MASK_INSERT | SD_INFO1_MASK_REMOVE))); /* }/ 荞݋ */
    }
    else if(SD_port_number == SDCARD_PORT1)
    {    /* |[g1CD[qqĂȂ̂Ŏ */
        SD_AndFPGA(EXT_CD,(~(EXT_CD_PORT1_INSERT | EXT_CD_PORT1_REMOVE)));                /* EXT_CD WX^ card inserted removedNA */
        SD_AndFPGA(EXT_CD_MASK,(~(EXT_CD_MASK_PORT1INSERT | EXT_CD_MASK_PORT1REMOVE)));    /* } 荞݋ */
    }
}
/*---------------------------------------------------------------------------*
  Name:         SD_DisableInfo

  Description:  disable SD card insert and remove interrupts.
                SDJ[h̑}/ 荞݂֎~

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_DisableInfo( void)
{
    if(SD_port_number == SDCARD_PORT0)
    {
        SD_AndFPGA(SD_INFO1,(~(SD_INFO1_INSERT | SD_INFO1_REMOVE))); /* SD_INFO1WX^ card inserted removedNA */
        SD_OrFPGA(SD_INFO1_MASK,(SD_INFO1_MASK_INSERT | SD_INFO1_MASK_REMOVE)); /* }/ 荞݋֎~ */
    }
    else if(SD_port_number == SDCARD_PORT1)
    {    /* |[g1CD[qqĂȂ̂Ŏ */
        SD_AndFPGA(EXT_CD,(~(EXT_CD_PORT1_INSERT | EXT_CD_PORT1_REMOVE))); /* EXT_CD WX^ card inserted removedNA */
        SD_OrFPGA(EXT_CD_MASK,(EXT_CD_MASK_PORT1INSERT | EXT_CD_MASK_PORT1REMOVE)); /* } 荞݋֎~ */
    }
}

/*---------------------------------------------------------------------------*
  Name:         SD_Command

  Description:  send command that the card will response only.
                R}h𑗏oiX|XԂĂ邾̃R}hpj

  Arguments:    ucCommand : command number

  Returns:      0 : success
                >0 : error
 *---------------------------------------------------------------------------*/
u16 SD_Command(u16 ucCommand)
{
    SD_AndFPGA(SD_INFO2,(~SD_INFO2_ERR_ALLCLR));           /* SD Card I/F  SG[NA */
    SD_AndFPGA(SD_INFO1,(~SD_INFO1_RES_END));              /* SD_INFO1WX^ Response end NA */
    SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_ALLERRMASK)); /* SD Card I/F  SG[݋ */

    SD_SetFPGA(SD_CMD,(ucCommand));                        /* R}hs */

    while(!SD_CheckFPGAReg(SD_INFO1,SD_INFO1_RES_END)){    /* Response end ҂ */
        if( SDCARD_ErrStatus != SDMC_NORMAL){      /* ^CAEg`FbN */
            break;
        }
    }

    SD_OrFPGA(SD_INFO2_MASK,(SD_INFO2_MASK_ALLERRMASK));   /* SD Card I/F ̑SG[݋֎~ */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_AppCommand

  Description:  send apprication command.
                AvP[VR}h(CMD55)𑗏o

  Arguments:    None

  Returns:      0 : success
                >0 : error
 *---------------------------------------------------------------------------*/
u16 SD_AppCommand(void)
{
    PRINTDEBUG( " CMD55 (APP_CMD)\n");
    
    /* argumentZbg */
    SD_SetFPGA(SD_ARG0,(0x0000));            /* Argument(15:0) = stuff bits */
    SD_SetFPGA(SD_ARG1,(SD_RCA));            /* Argument(31:16) = RCA */

    SD_Command(SD_CMD_CMD | APP_CMD);        /* CMD55sAX|X(R1)҂ */

#if ADD_CHECK
    /*SD_CheckStatussƁAOCMD8𔭍sĂꍇASDHCȊOCardStatus
      IllegalCommandG[tOĂ܂̂ŁAň邱ƂɂȂ
     ĩtONÂ1R}hԂx邽߁j
      SD Physical Layer dl Card StatusQ*/
//    SD_CheckStatus(FALSE);                 /* R}hX|X(R1)  Card Status `FbN */
#endif

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_AppOpCond

  Description:  get operating condition register data of SD card.
                SDJ[hACMD41𔭍sOCR擾

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_AppOpCond(void)
{
    SD_ClrErr((u16)(~SDMC_ERR_FPGA_TIMEOUT));      /* ^CAEgȊÕG[NA */
    
    while( !SDCARD_ErrStatus){                     /* G[ȂԂ͌JԂ */
        /* Argument(31:0) = OCR without busy (0x00100000 = 3.2-3.3V) */
        /*zXgœdIłꍇȂǂ́ASD_ARG10ɂ邱Ƃɂ
          J[hΉĂd₢킹邱ƂłBCTR͓d3.3Vߑł
          Ȃ̂Ŗ₢킹AȂ3.3VΉJ[hɗvBΉłȂ
          J[h Inactive Mode ɈڍsB*/
        SD_SetFPGA(SD_ARG0,(0x0000));
        if( SDCARD_SDHCFlag) {
            SD_SetFPGA(SD_ARG1,(0x4010));
        }else{
            SD_SetFPGA(SD_ARG1,(0x0010));
        }
        
        PRINTDEBUG( " ACMD41 (SD_SEND_OP_COND)\n");
        SD_Command(SD_CMD_ACMD | SD_APP_OP_COND);  /* ACMD41sAX|X(R3)҂ */
        
        if(!SDCARD_ErrStatus){                     /* G[Xe[^X̊mFiG[Hj*/
            SD_GetFPGA(SD_OCR[0],SD_RSP0);         /* X|X(R3)OCR擾 */
            SD_GetFPGA(SD_OCR[1],SD_RSP1);         /* X|X(R3)OCR擾 */
            if(SD_RSP1 & RSP_R3_OCR31){            /* OCR(WX^)31bitڃ`FbN(busy?) */
                break;
            }
        }
        SD_AppCommand();                           /* ς݂RCAݒ肵CMD55s */
    }
    
    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_SendOpCond

  Description:  send CMD1 without busy for MMC
                MMC  CMD1𔭍siSDACMD41ɈʒuR}hj

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SendOpCond(void)
{
    PRINTDEBUG( " CMD1  (SEND_OP_COND)\n");
    
    SD_ClrErr((u16)(~SDMC_ERR_FPGA_TIMEOUT));  /* ^CAEgG[NA */
    
    while( !SDCARD_ErrStatus){                 /* G[ȂԂ͌JԂ */
        SD_SetFPGA(SD_ARG0,(0x0000));          /* Argument(15:0) for MMC (None for SD) */
        SD_SetFPGA(SD_ARG1,(0x0010));          /* Argument(31:16) for MMC (None for SD) */
        SD_Command(SD_CMD_CMD | SEND_OP_COND); /* CMD1sAX|X(R1)҂ */
        if(!SDCARD_ErrStatus){                 /* G[Xe[^X̊mFiG[Hj */
            SD_GetFPGA(SD_OCR[0],SD_RSP0);
            SD_GetFPGA(SD_OCR[1],SD_RSP1);
            if(SD_RSP1 & RSP_R3_OCR31){        /* OCR(WX^)31bitڃ`FbN(busy?) */
                break;
            }
        }
    }
    return SDCARD_ErrStatus;
}


/*---------------------------------------------------------------------------*
  Name:         SD_SendIfCond

  Description:  send CMD8 
                CMD8𔭍siPhysical Layer 2.00ŒǉꂽR}hj

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SendIfCond(void)
{
    PRINTDEBUG( " CMD8  (SEND_IF_COND)\n");
    
    SD_ClrErr((u16)(~SDMC_ERR_FPGA_TIMEOUT));  /* ^CAEgG[NA */
    
    /* (31:12) Reserved bits, (11:8) supply voltage(VHS), (7:0) check pattern */
    SD_SetFPGA(SD_ARG0,(0x01AA));              /* Argument */
    SD_SetFPGA(SD_ARG1,(0x0000));              /* Argument */
        
    SD_Command(SD_CMD_CMD | SEND_IF_COND_EXT); /* CMD8sAX|X(R7)҂ */
        
    if(!SDCARD_ErrStatus){                     /* G[Xe[^X̊mFiG[Hj */
        SDCARD_SDHCFlag = TRUE;                /* SDHC */
//        SD_GetFPGA(SD_R7[0],SD_RSP0);
//        SD_GetFPGA(SD_R7[1],SD_RSP1);
    }else{
        SDCARD_SDHCFlag = FALSE;
    }
    
    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_SendRelativeAddr

  Description:  send relative addr
                CMD3𔭍sASDJ[h̏ꍇ̓X|XRCA擾
                iRCA͑̃R}hsArgumentƂĕKvj

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SendRelativeAddr(void)
{
    PRINTDEBUG( " CMD3  (SEND_RELATIVE_ADDR)\n");
    
    if(SDCARD_MMCFlag){                          /* MMCJ[htO ON H */
        SD_SetFPGA(SD_ARG0,(0x0000));            /* Argument(15:0) */
        SD_SetFPGA(SD_ARG1,(0x0001));            /* Argument(31:16) = 0x0001 (drv͂Ȃ0x0100ɂĂ) */
    }                            /* SDJ[ĥƂArgument(31:0) = stuff bits */
    
    SD_Command(SD_CMD_CMD | SEND_RELATIVE_ADDR); /* CMD3sAX|X(R6)҂ */

    if(!SDCARD_ErrStatus){                       /* G[Xe[^X̊mFiG[Hj */
        if(SDCARD_MMCFlag){                      /* MMCJ[htO ON H */
            SD_RCA = 0x0001;                     /* RCA <- 1 (drv͂Ȃ0x0100ɂĂ) */
        }else{
            SD_GetFPGA(SD_RCA,SD_RSP1);          /* X|XWX^RCA擾 */
        }
    }
    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_SelectCard

  Description:  toggle card between the stand-by and transfer states.
                CMD7𔭍săX^oC[hƓ][h؂ւ

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SelectCard(void)
{
    PRINTDEBUG( " CMD7  (SELECT#/DESELECT_CARD)\n");
    
    SD_SetFPGA(SD_ARG0,(0x0000));         /* Argument(15:0) = stuff bits*/
    SD_SetFPGA(SD_ARG1,(SD_RCA));         /* Argument(31:16) = RCA */

    SD_Command(SD_CMD_CMD | SELECT_CARD); /* CMD7sAX|X(R1b)҂ */

#if ADD_CHECK
    SD_CheckStatus(FALSE);                /* R}hX|X(R1)  Card Status `FbN */
#endif

    return SDCARD_ErrStatus;

}

/*---------------------------------------------------------------------------*
  Name:         SD_SetBlockLength

  Description:  set of block length.
                SDJ[h̃ubNOXݒ肷B
                ӁFmultiple read ̂Ƃ SD_SIZEWX^512ȊO̒l
                ݒ肷Ɛ삵Ȃ(IP reg spec)B

  Arguments:    ulBlockLength : bytes of a block ( must be multiplier of 2)
                (memo : the default block length is as specified in the CSD.)
                1ubN̒F2̏搔w肷邱

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SetBlockLength(u32 ulBlockLength)
{
    u16 usValue;
    
    PRINTDEBUG( " CMD16 (SET_BLOCKLEN)\n");

    /*------- IP transfer length ݒ -------*/
    if(ulBlockLength == 512){
        SD_SetFPGA(SD_SIZE,(SD_SIZE_DATA_LENGTH_512B)); /* SDJ[hf[^]TCY 512Bytes ݒ */
    }else{
        usValue = (u16)ulBlockLength;                   /* 16bitɕϊ */
        SD_SetFPGA(SD_SIZE,(usValue));                  /* IPSDJ[hf[^]TCYݒ */
    }/*------------------------------------------*/

    /*------- J[h̐ݒ -------*/
    /* argument(31:0) = block length */
    SD_SetFPGA(SD_ARG0,(((LELONG *)&ulBlockLength)->dt2word.low));
    SD_SetFPGA(SD_ARG1,(((LELONG *)&ulBlockLength)->dt2word.high));
    
    SD_Command(SD_CMD_CMD | SET_BLOCKLEN);              /* CMD16sAX|X(R1)҂ */
#if ADD_CHECK
    SD_CheckStatus(FALSE);                              /* R}hX|X(R1)  Card Status `FbN */
#endif
    /*----------------------------*/
    return SDCARD_ErrStatus;
}


/*---------------------------------------------------------------------------*
  Name:         SD_SendCID

  Description:  get card identification data (128bits).
                SDJ[hCIDl擾

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SendCID(void)
{
    PRINTDEBUG( " CMD2  (ALL_SEND_CID)\n");
                                            /* Argument(31:0) = stuff bits */
    SD_Command(SD_CMD_CMD | ALL_SEND_CID);  /* CMD2sAX|X(R2)҂ */

    if(!SDCARD_ErrStatus){                  /* G[Xe[^X̊mFiG[Hj */
        SD_GetFPGA(SD_CID[0],SD_RSP0);
        SD_GetFPGA(SD_CID[1],SD_RSP1);
        SD_GetFPGA(SD_CID[2],SD_RSP2);
        SD_GetFPGA(SD_CID[3],SD_RSP3);
        SD_GetFPGA(SD_CID[4],SD_RSP4);
        SD_GetFPGA(SD_CID[5],SD_RSP5);
        SD_GetFPGA(SD_CID[6],SD_RSP6);
        SD_GetFPGA(SD_CID[7],SD_RSP7);
    }

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_SendCSD

  Description:  get card specific data
                SDJ[hCSDl擾

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SendCSD(void)
{
    PRINTDEBUG( " CMD9  (SEND_CSD)\n");
    
    SD_SetFPGA(SD_ARG0,(0x0000));       /* Argument(15:0) = stuff bits */
    SD_SetFPGA(SD_ARG1,(SD_RCA));       /* Argument(31:16) = RCA */

    SD_Command(SD_CMD_CMD | SEND_CSD);  /* CMD9sAX|X(R2)҂ */

    if(!SDCARD_ErrStatus){              /* G[Xe[^X̊mFiG[Hj */
        SD_GetFPGA(SD_CSD[0],SD_RSP0);
        SD_GetFPGA(SD_CSD[1],SD_RSP1);
        SD_GetFPGA(SD_CSD[2],SD_RSP2);
        SD_GetFPGA(SD_CSD[3],SD_RSP3);
        SD_GetFPGA(SD_CSD[4],SD_RSP4);
        SD_GetFPGA(SD_CSD[5],SD_RSP5);
        SD_GetFPGA(SD_CSD[6],SD_RSP6);
        SD_GetFPGA(SD_CSD[7],SD_RSP7);
    }

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_SendStatus

  Description:  send status read command (compatible to the MMC protocol).
                }`fBAJ[h݊łstatus擾R}hs

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SendStatus(void)
{
    PRINTDEBUG( " CMD13 (Send STATUS)\n");

    SD_SetFPGA(SD_ARG0,(0x0000));                          /* Argument(15:0) = stuff bits */
    SD_SetFPGA(SD_ARG1,(SD_RCA));                          /* Argument(31:16) = RCA */

    /* J[hrȍdh */
    SD_AndFPGA(SD_INFO2,(~SD_INFO2_ERR_ALLCLR));           /* SD Card I/F  SẴG[NA */
    SD_AndFPGA(SD_INFO1,(~SD_INFO1_RES_END));              /* SD_INFO1WX^ Response end NA */
    SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_ALLERRMASK)); /* SD Card I/F  SG[݋ */

    SD_SetFPGA(SD_CMD,(SD_CMD_CMD | SD_SEND_STATUS));      /* CMD13s */

    while(!SD_CheckFPGAReg(SD_INFO1,SD_INFO1_RES_END)){    /* Response end (R1)҂ */
        if( SDCARD_ErrStatus != SDMC_NORMAL) {
            break;
        }
    }

    SD_OrFPGA(SD_INFO2_MASK,(SD_INFO2_MASK_ALLERRMASK));   /* SD Card I/F  SG[݋֎~ */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_SendSCR

  Description:  get condition data.
                SDJ[hSCRWX^l擾R}hsB̌J[h
                DATCoR1ubN(8Bytesɐݒ肵Ă)MĂB
                MultiBlock R/W ƈقȂADATCoRœ]ĂSDJ[h
                WX^́AMSBɑĂ邱ƂɒӁB
                (Physical Layer Specification 2.00 p12-13Q)

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SendSCR(void)
{
    PRINTDEBUG( " ACMD51 (Send SCR)\n");
                                                    /* Argument(31:0) = stuff bits */
#if ADD_CHECK
    SD_TransReadyFPGA();                            /* ֘AWX^ */
    SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_BRE)); /* SDJ[h̃f[^Ǐov݋*/
    
    SD_TransCommand((SD_CMD_ACMD | SEND_SCR));      /* SCR擾R}hsAX|X(R1)҂ */
#endif

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_SDStatus

  Description:  send status read command (for SD card only).
                SDJ[hpłstatus擾R}hsB̌J[h
                DATCoR1ubN(64Bytesɐݒ肵Ă)MĂB
                MultiBlock R/W ƈقȂADATCoRœ]ĂSDJ[h
                WX^́AMSBɑĂ邱ƂɒӁB
                (Physical Layer Specification 2.00 p12-13Q)

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SDStatus(void)
{
    PRINTDEBUG( " ACMD13 (SD_SD STATUS)\n");
                                                    /* Argument(31:0) = stuff bits */
    SD_TransReadyFPGA();                            /* ֘AWX^ */
    SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_BRE)); /* SDJ[h̃f[^Ǐov݋*/
    
    SD_TransCommand((SD_CMD_ACMD | SD_STATUS));     /* ACMD13 Send the SD_CARD status R}hsAX|X(R1)҂ */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_MultiReadBlock

  Description:  send multiple block read command.
                }`ubN[hR}hsB̌J[h
                DATCoRŃf[^𑗐MĂB

  Arguments:    ulOffset : offset address to read(BYTE).

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_MultiReadBlock(u32 ulOffset)
{
    PRINTDEBUG( " CMD18 (READ_MULTIPLE_BLOCK)\n");
    
    SD_TransReadyFPGA();                                /* INFOWX^ */
    if( !SDCARD_UseFifoFlag) {                          /* FIFOgȂƂ */
        SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_BRE)); /* SDJ[h̃f[^Ǐov݋*/
    }
  
    /* ǂݍ݊JnAhXiItZbgjݒ */
    SD_SetFPGA(SD_ARG0,(((LELONG *)&ulOffset)->dt2word.low));
    SD_SetFPGA(SD_ARG1,(((LELONG *)&ulOffset)->dt2word.high));

    SD_TransCommand((READ_MULTIPLE_BLOCK));             /* CMD18(}`ZN^[hR}h)sAX|X(R1)҂ */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_ClockDivSet

  Description:  set clock speed into the SD card.
                (notice : clock into IP is 33.51MHz.)
                (memo : default clock is 262KHz.)
                SDJ[hւ̃NbNxݒ肷BCTRIMCLK[qIPɋ
                {NbN33.51MHzB
                ÑftHgł128(262KHz)ݒ肳ĂB

  Arguments:    CSD responce include "TRAN_SPEED".
                \CSDSDJ[hǂݏoĂATRAN_SPEEDĂ
                CSD(CSD[5])BSDJ[hőΉ\ȎgQƂ邽߁B

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
u16 SD_ClockDivSet(u16 usTranSpeed)
{
    u16 usTranTime;

    //CTRł́AIMCLKɓ͂NbN 33.51Mhz
    //SD_CLK_CTRL  defaultl 0x0020( 128 = 262kHz)
    /*------*/
    usTranTime = (u16)((usTranSpeed >> 11) & 0x000F); /* CSD[103:96]  time value(=4bit) 擾 */
    
    usTranSpeed &= CSD_TRANSFER_RATE;                 /* CSD[103:96]  transfer rate unit 擾 */
    usTranSpeed = (u16)(usTranSpeed >> 8);            /* transfer rate unit 8rbgɐݒ肷 */
    /*------*/


    switch( usTranSpeed) {
      /*--- 100kbit/s(one dat line) = 100KHz, ̔{̂Ƃ ---*/
      case CSD_TRAN_SPEED_100K:
        if(usTranTime > 0x000C){                               /* time value  5.5 傫? */
            SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_64));          /* 523KHz */
        }
        else{
            if(usTranTime > 0x0006){                           /* time value  2.5 傫? */
                SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_128));     /* 262KHz */
            }else{
                if( usTranTime == 1) {                         /* time value  1.0 ? */
                    SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_512)); /* 65KHz */
                }else{                                         /* time value  1.2`2.5 ̂Ƃ */
                    SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_256)); /* 131KHz */
                }
            }
        }
        break;
      /*--- 1Mbit/s(one dat line) = 1MHz, ̔{̂Ƃ ---*/
      case CSD_TRAN_SPEED_1M:
        if(usTranTime == 0x0001){                              /* time value  1.0 ? */
            SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_64));          /* 523KHz */
        }
        else{
            if(usTranTime <= 0x0005){                          /* time value  2.0 ȉ? */
                SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_32));      /* 1.05MHz */
            }
            else{
                if(usTranTime <= 0x0009){                      /* time value  4.0 ȉ? */
                    SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_16));  /* 2.095MHz */
                }else{                                         /* time value  4.5 ȏ̂Ƃ */
                    SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_8));   /* 4.18MHz */
                }
            }
        }
        break;
      /*--- 10Mbit/s(one dat line) = 10MHz, ̔{̂Ƃ ---*/
      case CSD_TRAN_SPEED_10M:
        if(usTranTime > 0x0004){                        /* time value  1.5 傫? */
            SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_2));    /* 16.76MHz */
        }else{                                          /* time value  1.5 ȉ̂Ƃ */
            SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_4));    /* 8.38MHz */
        }
        break;
      /*--- 100Mbit/s(one dat line) = 100MHz, ̔{̂Ƃ ---*/
      case CSD_TRAN_SPEED_100M:                         /* time value ǂȒlłĂ */
           SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_2));     /* 16.76MHz */
        break;
      default:
        if( usTranSpeed != 7) {                         /* reservedl(=7)ȊO? */
               SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_2)); /* 16.76MHz */
        }                                               /* (drvł́A4ȏȂ288KHzɂĂ) */
        break;                                          /* transfer rate unit  7 ̂Ƃ͉Ȃ */
    }

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_EnableClock

  Description:  enable clock

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_EnableClock( void)
{
    SD_OrFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_SDCLKEN));
}

/*---------------------------------------------------------------------------*
  Name:         SD_DisableClock

  Description:  disable clock (for power save).

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_DisableClock( void)
{
    SD_AndFPGA(SD_CLK_CTRL,(~SD_CLK_CTRL_SDCLKEN));
}


/*---------------------------------------------------------------------------*
  Name:         SD_SelectBitWidth

  Description:  set bus width for SD Card ( Not for MMC Card).
                (The allowed data bus widths are given in SCR register.)
                SDJ[hDATCoX1bit܂4bitɐݒ肷B
                SDJ[h4bitɑΉĂȂAMMCJ[h̏ꍇ̓oX4bit
                ݒ肷邱Ƃ͂łȂBSDJ[h̋eoX́A\ߎ擾ς݂
                SCRWX^ɂĎQƂĂB

  Arguments:    b4bit : TRUE = 4bit, FALSE = 1bit

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
u16 SD_SelectBitWidth(s16 b4bit)
{
    /*--- J[h̎dl(SCR)Q ---*/
#if ADD_CHECK
    if( b4bit){                                         /* SCRWX^[hςł邱! */
        if( !(SD_SCR[0] & SCR_DAT_BUS_WIDTH_4BIT)) {    /* J[h4bitɑΉĂȂ? */
            b4bit = FALSE;                              /* ΉĂȂꍇ1bitw肵Ă */
        }
    }/*------------------------------*/
#endif
    /*--- 4bit(TRUE) ŵƂ ---*/
    if(b4bit){
        if(!SDCARD_MMCFlag){                            /* MMCJ[htO OFFH(ONȂ牽Ȃ) */
            if(SD_AppCommand()){                        /* RCAݒ CMD55s */
                return SDCARD_ErrStatus;                /* CMD55IȂG[I */
            }
            SD_SetFPGA(SD_ARG0,(0x0002));                /* Argument(1:0) = (10) to 4bit,(00) to 1bit */
            SD_SetFPGA(SD_ARG1,(0x0000));                /* Argument(31:2) = stuff bits */
            PRINTDEBUG( " ACMD6 (SET_BUS_WIDTH:4bit)\n");
            SD_Command(SD_CMD_ACMD | SET_BUS_WIDTH);     /* ACMD6irbgIR}hjsAX|X(R1)҂ */
#if ADD_CHECK
    SD_CheckStatus(FALSE);                               /* R}hX|X(R1) Card Status `FbN */
#endif
            if(!SDCARD_ErrStatus){                       /* G[Ȃ? */
                SD_AndFPGA(SD_OPTION,(~SD_OPTION_WIDTH_1BIT)); /* IPɃrbg̐ݒ(4bit) */
            }
        }
    }/*--- 1bit(FALSE) ŵƂ ---*/
    else{
        if(!SDCARD_MMCFlag){                            /* MMCJ[htO OFFH(ONȂ牽Ȃ) */
            if(SD_AppCommand()){                        /* RCAݒ CMD55s */
                return SDCARD_ErrStatus;                /* CMD55IȂG[I */
            }
            SD_SetFPGA(SD_ARG0,(0x0000));                /* Argument(1:0) = bus width : 1bit */
            SD_SetFPGA(SD_ARG1,(0x0000));                /* Argument(31:2) = stuff bits */
            PRINTDEBUG( " ACMD6 (SET_BUS_WIDTH:1bit)\n");
            SD_Command(SD_CMD_ACMD | SET_BUS_WIDTH);     /* ACMD6irbgIR}hjsAX|X(R1)҂ */
#if ADD_CHECK
    SD_CheckStatus(FALSE);                               /* R}hX|X(R1) Card Status `FbN */
#endif
        }
        if(!SDCARD_ErrStatus){                           /* G[Xe[^X̊mFiG[Hj */
            SD_OrFPGA(SD_OPTION,(SD_OPTION_WIDTH_1BIT)); /* IPɃrbg̐ݒ(1bit) */
        }
    }/*------------------------------*/

    return SDCARD_ErrStatus;
}


/*---------------------------------------------------------------------------*
  Name:         MMCP_WriteBusWidth

  Description:  set bus width for SD Card ( Not for MMC Card).
                (The allowed data bus widths are given in SCR register.)
                MMCplusJ[hDATCoX1bit܂4bitɐݒ肷B
                SD܂MMCJ[h̏ꍇ̓oX4bitɐݒ肷邱Ƃ͂łȂB
    
  Arguments:    b4bit : TRUE = 4bit, FALSE = 1bit

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
u16 MMCP_WriteBusWidth( BOOL b4bit)
{
    if( !SDCARD_MMCFlag) {
        return 1;
    }
    
    /* ݊JnItZbgݒ */
    if( b4bit) {
        SD_SetFPGA(SD_ARG0,(0x0100));    /* Argument(15:8)=(0x1) to 4bit,(0x0) to 1bit */
    }else{
        SD_SetFPGA(SD_ARG0,(0x0000));    /* Argument(15:8)=(0x1) to 4bit,(0x0) to 1bit */
    }
    SD_SetFPGA(SD_ARG1,(0x03B7));        /* Argument(25:24)=(0x3)Write, (23:16)=(183)Index */

    /* CMD6irbgIR}hjsAX|X(R1)҂ */
    SD_Command(SD_CMD_CMD | EXT_CSD_ACCESS);     

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         MMCP_BusTest

  Description:  oX̃eXgsB
                ̌J[hDATCoRŃf[^𑗐MKvB
    
  Arguments:    readflag : [hTRUE, CgFALSE

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
u16 MMCP_BusTest( BOOL readflag)
{
    /**/
    SD_TransReadyFPGA();                            /* INFOWX^ */
    SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_BWE)); /* SDJ[h̃f[^ݗv݋ */

    /**/
    SD_SetFPGA( SD_ARG0, 0);
    SD_SetFPGA( SD_ARG1, 0);

    if( readflag) {
        SD_TransCommand( 14);
    }else{
        SD_TransCommand( 19);
    }

    return( SDCARD_ErrStatus);
}


/*---------------------------------------------------------------------------*
  Name:         SD_FPGA_irq

  Description:  handler of data transfer request interrupt from card,
                check and clear interrupt register of IP.
                J[h̓]viBRE܂BWEj荞ݔ
                ǂ炩𒲂ׂĕԂAINFO2̓Y荞ݗvtONAB

  Arguments:    None

  Returns:      TRUE : BRE荞ݔ
                FALSE : BWE荞ݔ
 *---------------------------------------------------------------------------*/
s16 SD_FPGA_irq(void)
{
    /*--- FIFOgƂ ---*/
    if( SDCARD_UseFifoFlag) {
        if( SD_CheckFPGAReg( *SDIF_CNT_L, (u16)SDIF_CNT_FFIE)) {   /* FULL荞݋̂Ƃ */
            return TRUE;
        }else{
#if (SD_FIFO_EMPTY_FLAG_NEW == 1) //FIFO EmptytOdlύX
            return FALSE;
#else                             //FIFO EmptytOdlύXO
            if(!SD_CheckFPGAReg(SD_INFO2_MASK,SD_INFO2_MASK_BWE)){ /* SDJ[h̃f[^ݗvH */
                SD_AndFPGA(SD_INFO2,(~SD_INFO2_BWE));              /* SDpobt@ Write Enable Zbg*/
            }
            return FALSE;
#endif
        }
    }else{    /*--- FIFOgȂƂ ---*/
        if(!SD_CheckFPGAReg(SD_INFO2_MASK,SD_INFO2_MASK_BRE)){ /* SDJ[h̃f[^ǏovH */
            SD_AndFPGA(SD_INFO2,(~SD_INFO2_BRE));              /* SDpobt@ Read Enable Zbg*/
            return TRUE;
        }
        if(!SD_CheckFPGAReg(SD_INFO2_MASK,SD_INFO2_MASK_BWE)){ /* SDJ[h̃f[^ݗvH */
            SD_AndFPGA(SD_INFO2,(~SD_INFO2_BWE));              /* SDpobt@ Write Enable Zbg*/
            return FALSE;
        }
    }
    return FALSE;
}

/*---------------------------------------------------------------------------*
  Name:         SD_StopTransmission

  Description:  manual send CMD12 to terminate translate.
                蓮CMD12𔭍sA]IFPGAɒʒm

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_StopTransmission(void)
{
    SD_OrFPGA(SD_STOP,(SD_STOP_STP));        /* f[^]Iݒ */
}

/*---------------------------------------------------------------------------*
  Name:         SD_TransEndFPGA

  Description:  clear the transfer interrupt.
                J[h]Iɓ]֘Å荞݂֎~ɖ߂Ă

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_TransEndFPGA(void)
{
    SD_OrFPGA(SD_INFO2_MASK,(SD_INFO2_MASK_ALLERRMASK));   /* SD Card I/F  SG[݋֎~ */
    SD_OrFPGA(SD_INFO1_MASK,(SD_INFO1_MASK_ALL_END));      /* R/W ANZXI݋֎~ */

    if(!SD_CheckFPGAReg(SD_INFO2_MASK,SD_INFO2_MASK_BRE)){ /* SDJ[h̃f[^Ǎݗv݋? */
        SD_OrFPGA(SD_INFO2_MASK,(SD_INFO2_MASK_BRE));      /* SDJ[h̃f[^Ǎݗv݋֎~ */
    }
    if(!SD_CheckFPGAReg(SD_INFO2_MASK,SD_INFO2_MASK_BWE)){ /* SDJ[h̃f[^ݗv݋? */
        SD_OrFPGA(SD_INFO2_MASK,(SD_INFO2_MASK_BWE));      /* SDJ[h̃f[^ݗv݋֎~ */
    }
}

/*---------------------------------------------------------------------------*
  Name:         SD_CheckStatus

  Description:  check the card status in the R1.
                R1X|X̃J[hXe[^X`FbN

  Arguments:    bRead : ǂݍݏ = TRUE

  Returns:      0 : success
                >0 : error
 *---------------------------------------------------------------------------*/
u16 SD_CheckStatus(BOOL bRead)
{
    /* R}hX|X(R1)[39:8]  Card status 擾 */
    SD_GetFPGA((((LELONG *)&SDCARD_Status)->dt2word.low),SD_RSP0);    
    SD_GetFPGA((((LELONG *)&SDCARD_Status)->dt2word.high),SD_RSP1);

    /*--- TvhCołĂ鏈 ---*/
    if(bRead){                                            /* [h? */
        if(!(SDCARD_ErrStatus & SDMC_ERR_TIMEOUT)){       /* ^CAEgĂȂ? */
            SDCARD_Status &= ~SDCARD_STATUS_OUT_OF_RANGE; /* OUT_OF_RANGEtO𗎂Ƃ */
        }
    }/*-------------------------------------------*/
    
    PRINTDEBUG( " SD_CheckStatus ======== 0x%x\n", SDCARD_Status);

    if(SDCARD_Status & RSP_R1_STATUS_ERR){ /* R}hX|X(R1)̃J[hXe[^XG[mF */
        SD_SetErr(SDMC_ERR_R1_STATUS);     /* R}hX|X(R1)̃J[hXe[^X G[ */
    }
    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_SwapByte

  Description:  swap bytes in a 16bit data.
                16bitf[^̏ʂƉʂꊷ

  Arguments:    data : 16bit data

  Returns:      swapped data
 *---------------------------------------------------------------------------*/
u16    SD_SwapByte(u16 *data)
{
    u16    usDATA;
    
    usDATA = *data;
    usDATA = (u16)(((usDATA & 0x00FF) << 8) | ((usDATA & 0xFF00) >> 8));
    
    return usDATA;
}

/*---------------------------------------------------------------------------*
  Name:         SD_EnableSeccnt

  Description:  enable SD_SECCNT register.
                SD_SECCNT LĒlZbgB

  Arguments:    ulSDCARD_SectorCount : ZN^JEgli1ZN^=512bytej

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_EnableSeccnt( u32 ulSDCARD_SectorCount)
{
    u16    usSector;
    
    usSector = (u16)ulSDCARD_SectorCount;
    SD_OrFPGA(SD_STOP,SD_STOP_SEC_ENABLE); /* SD_SECCNTWX^Lɂ */
    SD_SetFPGA(SD_SECCNT,usSector);        /* SD_SECCNTWX^ɓ]ZN^JEgݒ */
}

/*---------------------------------------------------------------------------*
  Name:         SD_DisableSeccnt݂͖gp֐

  Description:  disable SD_SECCNT register.
                SD_SECCNT 𖳌B

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_DisableSeccnt( void)
{
    u16    i;
    
    /*--- hCołĂ邱 ---*/
    for ( i=0; i<50000; i++)                   /* JEgl50000ł͑ȂƂ */
    {    /* SD_INFO1WX^ R/W access all end ҂ */
        if(SD_CheckFPGAReg(SD_INFO1,SD_INFO1_ALL_END)) {
            break;
        }
    }/*-----------------------------------*/
    SD_AndFPGA(SD_STOP,(~SD_STOP_SEC_ENABLE)); /* SD_SECCNTWX^𖳌ɂ */
}

/*---------------------------------------------------------------------------*
  Name:         SD_SetErr

  Description:  set error flag
                w̃G[tOZbg

  Arguments:    Error : error flag to set

  Returns:      None
 *---------------------------------------------------------------------------*/
void    SD_SetErr(u16 Error)
{
    OSIntrMode irq_core_flag;
    
    irq_core_flag = OS_DisableInterrupts();
        SDCARD_ErrStatus |= Error;         /* G[Xe[^Xݒ */
    OS_RestoreInterrupts( irq_core_flag);
} 

/*---------------------------------------------------------------------------*
  Name:         SD_ClrErr

  Description:  clear error flag
                w̃G[tONA

  Arguments:    Error : error flag to clear

  Returns:      None
 *---------------------------------------------------------------------------*/
void    SD_ClrErr(u16 Error)
{
    OSIntrMode irq_core_flag;

    irq_core_flag = OS_DisableInterrupts(); /* ݋֎~ */
        SDCARD_ErrStatus &= ~(Error);       /* G[Xe[^XNA */
    OS_RestoreInterrupts( irq_core_flag);   /* 荞ݐݒɖ߂ */
} 


/*---------------------------------------------------------------------------*
  Name:         SD_TransReadyFPGA

  Description:  setup for the command that the card will response and request data transfer.
                DATCł̃f[^]R}hsO̓]
                iBRE, BWE荞݂̋͕ʓrsƁj

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SD_TransReadyFPGA(void)
{
    /* ֘AWX^ */
    SD_AndFPGA(SD_INFO2,(~SD_INFO2_ERR_ALLCLR));                    /* SD Card I/F  SẴG[NA */
    SD_AndFPGA(SD_INFO1,(~(SD_INFO1_RES_END | SD_INFO1_ALL_END)));    /* SD_INFO1WX^ Response end  access all end NA */
    SD_AndFPGA(SD_INFO2,(~SD_INFO2_BRE));                            /* SDpobt@ Read Enable NA*/
    SD_AndFPGA(SD_INFO2,(~SD_INFO2_BWE));                            /* SDpobt@ Write Enable NA*/
    SD_AndFPGA(SD_STOP,(~SD_STOP_STP));                                /* f[^]INA */

    /* 荞݊֘A */
    SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_ALLERRMASK));            /* SD Card I/F  SG[݋ */
    SD_AndFPGA(SD_INFO1_MASK,(~SD_INFO1_MASK_ALL_END));                /* R/W access all end ݋(Response̓|[OŌ邽ߋȂ) */
}


/*---------------------------------------------------------------------------*
  Name:         SD_TransCommand

  Description:  send command that the card will response and request data transfer.
                R}h𑗏oiDATCł̃f[^]R}hpj

  Arguments:    ucCommand : command number

  Returns:      0 : success
                >0 : error
 *---------------------------------------------------------------------------*/
u16    SD_TransCommand(u16 ucCommand)
{
    SD_SetFPGA(SD_CMD,(ucCommand));                     /* R}hs */

    while(!SD_CheckFPGAReg(SD_INFO1,SD_INFO1_RES_END)){ /* Response end ҂ */
        if( SDCARD_ErrStatus != SDMC_NORMAL) {
            break;
        }
    }
    SD_CheckStatus(FALSE);                              /* R1X|Xcard status`FbN*/
    
    if(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS){
        SD_StopTransmission();                          /* J[h]IFPGAɒʒm */
    }

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_CheckFPGAReg

  Description:  check register bits.
                WX^̏Ԃ`FbN

  Arguments:    reg : 16bit register data
                value : mask data

  Returns:      TRUE : some of mask data bits are 1 on the register.
                FALSE : no mask data bits is 1 on the register.
 *---------------------------------------------------------------------------*/
BOOL    SD_CheckFPGAReg(u16 reg,u16 value)
{
    if( reg & value) {
        return TRUE;
    }else{
        return FALSE;
    }
}

/*---------------------------------------------------------------------------*
  Name:         SD_SendNumWRSectors

  Description:  send "number of well written blocks" command (for SD card only).
                SDJ[hpł̃Cgς݃ZN^擾R}hsB̌J[h
                DATCoR1ubN(4Bytesɐݒ肵Ă)MĂB
                MultiBlock R/W ƈقȂADATCoRœ]ĂSDJ[h
                WX^́AMSBɑĂ邱ƂɒӁB
                (Physical Layer Specification 2.00 p12-13Q)

  Arguments:    None

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_SendNumWRSectors(void)
{
    PRINTDEBUG( " ACMD22 (SEND_NUM_WR_SECTORS)\n");
                                                          /* Argument(31:0) = stuff bits */
    SD_TransReadyFPGA();                                  /* ֘AWX^ */
    SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_BRE));       /* SDJ[h̃f[^Ǎݗv݋ */

    SD_TransCommand((SD_CMD_ACMD | SEND_NUM_WR_SECTORS)); /* ACMD22i݊ZN^擾R}hjsAX|X(R1)҂ */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SD_MultiWriteBlock

  Description:  send multiple block write command.
                }`ubNCgR}hsB̌J[h
                DATCoRŃf[^𑗐MKvB

  Arguments:    ulOffset : offset address to write(BYTE).

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
u16 SD_MultiWriteBlock(u32 ulOffset)
{
    PRINTDEBUG( " CMD25 (WRITE_MULTIPLE_BLOCK)\n");
    
    SD_TransReadyFPGA();                                /* INFOWX^ */

#if (SD_FIFO_EMPTY_FLAG_NEW == 1)  // FIFO EmptytOdlύX
    if( !(SDCARD_UseFifoFlag)) {                           /* FIFOgȂƂ */
        SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_BWE)); /* SDJ[h̃f[^ݗv݋ */
    }
#else                              // FIFO EmptytOdlύXO
    SD_AndFPGA(SD_INFO2_MASK,(~SD_INFO2_MASK_BWE)); /* SDJ[h̃f[^ݗv݋ */
#endif

    /* ݊JnItZbgݒ */
    SD_SetFPGA(SD_ARG0,(((LELONG *)&ulOffset)->dt2word.low));
    SD_SetFPGA(SD_ARG1,(((LELONG *)&ulOffset)->dt2word.high));

    SD_TransCommand(WRITE_MULTIPLE_BLOCK);              /* CMD25i}`ZN^CgR}hjs */

    return SDCARD_ErrStatus;
}


