/*---------------------------------------------------------------------------*
  Project:  TWL - SD port driver
  File:     sdmc.c

  Copyright 2006,2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include <twl.h>
#include "sdmc_config.h"
#include "sdif_reg.h"           /*  IP ΉWX^` */
#include <twl/sdmc.h>
#include "sdif_ip.h"            /*  IP ΉtO` */

//        #define PRINTDEBUG    OS_TPrintf
    #define PRINTDEBUG( ...) ((void)0)


/***********************************************************************
 萔
***********************************************************************/
#define SD_STACK_SIZE        (4096*2)
#define SD_THREAD_PRIO       (10)
#define SD_INTR_THREAD_PRIO  (SD_THREAD_PRIO - 1)

#define SD_OPERATION_INIT               (0)
#define SD_OPERATION_READ               (1)
#define SD_OPERATION_READ_WITH_FIFO     (2)
#define SD_OPERATION_WRITE              (3)
#define SD_OPERATION_WRITE_WITH_FIFO    (4)

/*
TODO:
SD_INTR_THREAD_PRIOSleepȂΓB
Sleepgɂ́AWakeup̑OSleep邽SD_INTR_THREAD_PRIOႭ
Kv邪ASleep̑OSD荞݂ȂꍇȂ
荞݂ȂȂ蓮ȂȂBSleep̑OɊ荞݂ΓB
*/


/***********************************************************************
 externϐ
***********************************************************************/
extern u16 BgBak[32*32];
//u16 sdcard_request_flag;            //J[h̃f[^]v̗LtO
static BOOL    thread_flag;


/***********************************************************************
 globalϐ
***********************************************************************/
BOOL    sdmc_tsk_created = FALSE;
#if (TARGET_OS_CTR == 1)
ER_ID   sdmc_tsk_id;        //SD^XNID
ER_ID   sdmc_dtq_id;
ER_ID   sdmc_result_dtq_id;
ER_ID   sdmc_alm_id;
ER_ID   sdmc_intr_tsk_id;
#else
OSThread       sdmc_tsk;
OSMessageQueue sdmc_dtq;
OSMessage      sdmc_dtq_array[1];
OSMessageQueue sdmc_result_dtq;
OSMessage      sdmc_result_dtq_array[1];
OSAlarm        sdmc_alm;
OSThread       sdmc_intr_tsk;
#endif

u16 sdmc_wakeup_count = 0; //OS_WakeupThread̃L[CO
u16 sdmc_intr_wakeup_count = 0;

#if (TARGET_OS_CTR == 1)
#else
u64            sd_stack[SD_STACK_SIZE / sizeof(u64)];
u64            sd_intr_stack[SD_STACK_SIZE / sizeof(u64)];
#endif

u16         sdmc_dma_no = SDMC_NOUSE_DMA;
/* drsdmc.cłQ */
SdmcSpec    sdmc_current_spec;    //TODO:|[g؂ւAPort0Port1ɕۑ悤

typedef struct {                  //OSMessage
    void*           buf;
    u32             bufsize;
    u32             offset;
    void            (*func)(void);
    SdmcResultInfo* info;
    u32             operation;
    void            (*func2)(void);
} SDCARDMsg;


/***********************************************************************
 static֐̐錾
***********************************************************************/
void i_sdmcCalcSize( void);
static void SDCARD_Backup_port0(void);
static void SDCARD_Backup_port1(void);
static void SDCARD_Restore_port0(void);
static void SDCARD_Restore_port1(void);

static SDMC_ERR_CODE SDCARDi_ReadFifo(void* buf,u32 bufsize,u32 offset,void(*func)(void),SdmcResultInfo *info);
static SDMC_ERR_CODE SDCARDi_WriteFifo( const void* buf,u32 bufsize,u32 offset,void(*func)(),SdmcResultInfo *info);

static SDMC_ERR_CODE SDCARDi_Read(void* buf,u32 bufsize,u32 offset,void(*func)(void),SdmcResultInfo *info);
static SDMC_ERR_CODE SDCARDi_Write( const void* buf,u32 bufsize,u32 offset,void(*func)(),SdmcResultInfo *info);

int     MMCP_SetBusWidth( BOOL b4bit);       /* rbg̑I(MMCplus, eMMC, moviNAND) */

static void SDCARD_Thread( void* arg);        //SDXbh
static void SDCARD_Intr_Thread( void* arg);        //SD荞ݏXbh

SDMC_ERR_CODE sdmcGoIdle( void (*func1)(),void (*func2)());
static SDMC_ERR_CODE i_sdmcInit( void);
static SDMC_ERR_CODE SDCARD_Layer_Init(void);
static SDMC_ERR_CODE i_sdmcMPInit( void);     /* J[hhCo(}`|[gΉ) */

static u16 i_sdmcErrProcess(void);            /* G[̏ */
static u16 i_sdmcGetResid(u32 *pResid);       /* ݊ZN^̎擾 */
static u16 i_sdmcCheckWP(void);               /* SDCARD CgveNg`FbN */

static void i_sdmcEnable( void);              /* SDgp̂߂OS */
static void i_sdmcDisable( void);

static u16 i_sdmcSendSCR(void);               /* SCR̓ǂݏo */
static u16 SDCARD_SD_Status(void);            /* SD_STATUS̎擾 */
static u32 SDCARD_GetR1Status(void);          /* Normal response command J[hXe[^X擾 */

static void SDCARD_Dmy_Handler( void);        /* Ȃ */
static void SDCARD_Timer_irq(void* arg);      /* ^CAEg荞݃nh */
static void SDCARD_irq_Handler( void);        /* SD-IP̊荞݃nh */
static void SDCARD_FPGA_irq(void);            /* J[h[hCg荞ݏ */
static void SYSFPGA_irq(void);                /* SYSFPGAG[荞ݏ */
static void SDCARD_ReadyToEnd( void);

/*|[g1͖ŒȂ̂Ń|[gI֐͌JȂ*/
static u16 i_sdmcSelectedNo(void);            /* J[h|[g̑I */
static u16 i_sdmcSelect(u16 select);          /* ݂̃J[h|[gԍ̃`FbN */




/***********************************************************************
    OQƕϐ
***********************************************************************/
static void SDCARD_TimerStart(u32 tim);    /* ^CAEgvX^[g */
static void SDCARD_TimerStop(void);        /* ^CAEgv~ */

extern u16  SD_CID[8];                /* CIDlۑp */
extern u16  SD_CSD[8];                /* CSDlۑp */
extern u16  SD_OCR[2];                /* OCRlۑp */
extern u16  SD_SCR[4];                /* SCRlۑp */
extern u16  SD_RCA;                   /* RCAlۑp */

extern s16  SDCARD_MMCFlag;           /* MMCJ[htO */
extern s16  SDCARD_SDHCFlag;          /* SDHCJ[htO */
extern u16  SD_port_number;           /* ݃|[gԍ  */

//static BOOL init_io_exist;            /* SDIO݃tO */
//static BOOL init_mem_exist;           /* ݃tO */


/***********************************************************************
    O[o
***********************************************************************/
u16  SD_SDSTATUS[32];            /* SD_STATUSWX^ۑp */
u16  SDCARD_WP_FLAG0;            /* J[hCgveNgtOB0=ȂA1=L    */
u16  SDCARD_WP_FLAG1;            /* J[hCgveNgtOB0=ȂA1=L    */
u16  SDCARD_WP_PERMANENT;        /* J[hCgveNgivtOB0=ȂA1=L    */
u16  SDCARD_WP_TEMPORARY;        /* J[hCgveNgꎞtOB0=ȂA1=L    */

u16* pSDCARD_BufferAddr;         /* ۑpf[^i[obt@AhX */

u32  ulSDCARD_SectorCount;       /* ]ZN^ */
u32  ulSDCARD_RestSectorCount;   /* c]ZN^ */
u32  SDCARD_SectorSize;          /* ZN^TCY ftHg 512bytes */

u16  SD_INFO1_VALUE;             /* SD_INFO1WX^擾pϐ */
u16  SD_INFO1_MASK_VALUE;        /* SD_INFO1݃}XNpϐ(0ŋ, 1ŋ֎~) */
u16  SD_INFO2_VALUE;             /* SD_INFO2WX^擾pϐ */
u16  SD_INFO2_MASK_VALUE;        /* SD_INFO2荞݃}XNpϐ(0ŋ, 1ŋ֎~) */
u16  SD_INFO_ERROR_VALUE;        /* SD_INFO2, SD_INFO1̃G[rbgmFpϐ */

u16  SD_port_en_numbers;         /* T|[g|[g */



/*|[gԕۑ*/
SDPortContext SDPort0Context;
SDPortContext SDPort1Context;
//SDPortContext *SDPortCurrentContext = &SDPort0Context; /*TODO*/



u32    ulSDCARD_Size;                    /* J[hSZN^ */

volatile s16    SDCARD_EndFlag;          /* ]tO */

SDMC_ERR_CODE   SDCARD_ErrStatus;        /* G[Xe[^X */
vu32            SDCARD_Status;           /* J[hXe[^X */
s16             SDCARD_SDFlag;           /* SDJ[htO */

volatile s16    SDCARD_OutFlag;          /* J[hrotO */
SdmcResultInfo  *pSDCARD_info;           /* ۑpsʍ\̃|C^ */

u16             SDCARD_IO_Port;          /* J[h}/ro荞ݔ̃|[gԍ */

void (*func_SDCARD_In)(void);            /* J[h}CxgpR[obNۑp */
void (*func_SDCARD_Out)(void);           /* J[hroCxgpR[obNۑp */
/* void (*func_SDCARD_CallBack)(SdmcResultInfo *info);     ʒʒmpR[obNۑp */





/*---------------------------------------------------------------------------*
  Name:         MyCpuRecv*

  Description:  CPUœAhXf[^󂯎܂

  Arguments:    src : ]AhXAdest : ]AhXAsize : ]TCY(Bytes)

  Returns:      None
 *---------------------------------------------------------------------------*/
static void MyCpuRecv32( const u32* src, u32* dest, u32 size)
{
    u32 i;
    for( i=0; i<(size/sizeof(u32)); i++) {
        *(u32*)dest++ = *(u32*)src;
    }
}
static void MyCpuRecv16( const u16* src, u16* dest, u32 size)
{
    u32 i;
    for( i=0; i<(size/sizeof(u16)); i++) {
        *(u16*)dest++ = *(u16*)src;
    }
}


/*---------------------------------------------------------------------------*
  Name:         SDCARD_Backup_port0

  Description:  backup registers and variables of port0.
                |[g0̃WX^ϐobNAbv

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_Backup_port0(void)
{
    /* registers */
    SD_GetFPGA(SDPort0Context.SD_CLK_CTRL_VALUE,SD_CLK_CTRL);
    SD_GetFPGA(SDPort0Context.SD_OPTION_VALUE,  SD_OPTION);

    /* variables */
    SDPort0Context.SD_RCA    = SD_RCA;
    SDPort0Context.ErrStatus = SDCARD_ErrStatus;
    SDPort0Context.Status    = SDCARD_Status;
    SDPort0Context.MMCFlag   = SDCARD_MMCFlag;
    SDPort0Context.SDHCFlag  = SDCARD_SDHCFlag;
    SDPort0Context.SDFlag    = SDCARD_SDFlag;

    SDPort0Context.OutFlag   = SDCARD_OutFlag;

    /*media registers*/
    MI_CpuCopy8( SD_CID, SDPort0Context.SD_CID, 16);
    MI_CpuCopy8( SD_CSD, SDPort0Context.SD_CSD, 16);
    MI_CpuCopy8( SD_OCR, SDPort0Context.SD_OCR,  4);
    MI_CpuCopy8( SD_SCR, SDPort0Context.SD_SCR,  8);
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_Backup_port1

  Description:  backup registers and variables of port1.
                |[g1̃WX^ϐobNAbv

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_Backup_port1(void)
{
    /* registers */
    SD_GetFPGA(SDPort1Context.SD_CLK_CTRL_VALUE,SD_CLK_CTRL);
    SD_GetFPGA(SDPort1Context.SD_OPTION_VALUE,  SD_OPTION);

    /* variables */
    SDPort1Context.SD_RCA       = SD_RCA;
    SDPort1Context.ErrStatus = SDCARD_ErrStatus;
    SDPort1Context.Status    = SDCARD_Status;
    SDPort1Context.MMCFlag   = SDCARD_MMCFlag;
    SDPort1Context.SDHCFlag  = SDCARD_SDHCFlag;
    SDPort1Context.SDFlag    = SDCARD_SDFlag;

    SDPort1Context.OutFlag   = SDCARD_OutFlag;

    /*media registers*/
    MI_CpuCopy8( SD_CID, SDPort1Context.SD_CID, 16);
    MI_CpuCopy8( SD_CSD, SDPort1Context.SD_CSD, 16);
    MI_CpuCopy8( SD_OCR, SDPort1Context.SD_OCR,  4);
    MI_CpuCopy8( SD_SCR, SDPort1Context.SD_SCR,  8);
}


/*---------------------------------------------------------------------------*
  Name:         i_sdmcEnable

  Description:  assign OS resouce for using SD memory card.
                SDJ[hg߂̏

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void i_sdmcEnable( void)
{
    /*SD荞݂IF*/
    *(vu16*)CTR_INT_IF = CTR_IE_SD_MASK;
    OS_SetIrqFunction( OS_IE_SD, SDCARD_irq_Handler);
    OS_EnableIrqMask( OS_IE_SD);
}

/*---------------------------------------------------------------------------*
  Name:         i_sdmcDisable

  Description:  under construction
                H

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void i_sdmcDisable( void)
{
}


/*---------------------------------------------------------------------------*
  Name:         SDCARD_irq_Handler

  Description:  SD interrupt handler
                SD荞݃nh

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void SDCARD_irq_Handler( void)
{
//    OS_TPrintf( "SD irq!\n");
    OS_SetIrqCheckFlag( OS_IE_SD);
//  sdmc_intr_wakeup_count++;
//    OS_WakeupThreadDirect( &sdmc_intr_tsk);
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_Dmy_Handler

  Description:  dmy handler for timer interrupt.
                ^C}[荞ݗp_~[nh

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_Dmy_Handler( void)
{
}

/*---------------------------------------------------------------------------*
  Name:         sdmcInit

  Description:  Initialize SD interface and SD card.
                

  Arguments:    dma_no : DMAԍ
                func1 : J[h}R[obN֐
                func2 : J[hroR[obN֐

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
SDMC_ERR_CODE sdmcInit( SDMC_DMA_NO dma_no, void (*func1)(),void (*func2)())
{
    SDMC_ERR_CODE    api_result;

    /*--- DMA / CPU I ---*/
    if( (dma_no < 4) || (dma_no == SDMC_NOUSE_DMA)) {
        sdmc_dma_no = dma_no;
    }else{
        return( SDMC_ERR_PARAM);
    }
    /*----------------------*/
  
    if( sdmc_tsk_created == FALSE) {
        /*---------- OS ----------*/
        if( !OS_IsAlarmAvailable()) {	/* A[`FbN(OS_InitAlarmς݂?) */
            SDCARD_ErrStatus |= SDMC_ERR_END;
        }else{
            OS_CreateAlarm( &sdmc_alm);	//gp\łΏ
        }
        
        /* bZ[W */
        OS_InitMessageQueue( &sdmc_dtq,        &sdmc_dtq_array[0],        1);
        OS_InitMessageQueue( &sdmc_result_dtq, &sdmc_result_dtq_array[0], 1);

        /* SDXbh̗グ */
        OS_CreateThread( &sdmc_tsk, SDCARD_Thread, NULL,
                         (sd_stack+SD_STACK_SIZE / sizeof(u64)), SD_STACK_SIZE, SD_THREAD_PRIO);
        OS_WakeupThreadDirect( &sdmc_tsk);
        PRINTDEBUG( "sdmc_tsk:0x%x\n", &sdmc_tsk);
        

        /* SD荞ݏXbh̗グ */
        (void)OS_ClearIrqCheckFlag( OS_IE_SD);
        OS_CreateThread( &sdmc_intr_tsk, SDCARD_Intr_Thread, NULL,
                         (sd_intr_stack+SD_STACK_SIZE / sizeof(u64)), SD_STACK_SIZE, SD_INTR_THREAD_PRIO);
        OS_WakeupThreadDirect( &sdmc_intr_tsk);
        PRINTDEBUG( "sdmc_intr_tsk:0x%x\n", &sdmc_intr_tsk);
        /*----------------------------*/

        /**/
        sdmc_tsk_created = TRUE;
    }
    
    /**/
    func_SDCARD_In = func1;         /* J[h}Cxgp֐̃AhXݒ */
    func_SDCARD_Out = func2;        /* J[hroCxgp֐̃AhXݒ */
    api_result = sdmcGoIdle( func1, func2);
//    api_result = SDMC_NORMAL;
    
    return api_result;
}

/* J[hꊷƂȂǂɏiRTFSpj*/
SDMC_ERR_CODE sdmcGoIdle( void (*func1)(),void (*func2)())
{
    SDCARDMsg        SdMsg;
    OSMessage        init_msg;
    SDMC_ERR_CODE    api_result;
    
    func_SDCARD_In  = func1;        /* J[h}Cxgp֐̃AhXݒ */
    func_SDCARD_Out = func2;        /* J[hroCxgp֐̃AhXݒ */
    
    /*----- SDXbhƒʐM -----*/
    SdMsg.operation = SD_OPERATION_INIT;
//    SdMsg.func  = func1;
//    SdMsg.func2 = func2;

    init_msg = (OSMessage)&SdMsg;
    OS_SendMessage( &sdmc_dtq, init_msg, OS_MESSAGE_BLOCK);

    /* Ԃl҂ */
    OS_ReceiveMessage( &sdmc_result_dtq, &init_msg, OS_MESSAGE_BLOCK);
    api_result = (SDMC_ERR_CODE)init_msg;
    /*----------------------------------*/

    return api_result;
}


/*---------------------------------------------------------------------------*
  Name:         i_sdmcInit

  Description:  Initialize SD interface and SD card.
                

  Arguments:    

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
static SDMC_ERR_CODE i_sdmcInit( void)
{
    i_sdmcEnable();
    
    /* SD */
    SDCARD_ErrStatus = sdmcReset();

    if(!SDCARD_ErrStatus) {
        SDCARD_ErrStatus = i_sdmcMPInit();
    }

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         sdmcReset

  Description:  reset SD card.
                Zbg

  Arguments:    

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
SDMC_ERR_CODE sdmcReset( void)
{
    OSIntrMode irq_core_flag;
    ulSDCARD_Size   = 0;               /* J[hSZN^NA */
    SDCARD_MMCFlag  = FALSE;           /* MMCJ[htONA */
    SDCARD_SDHCFlag = FALSE;
    SDCARD_SDFlag   = FALSE;           /* SDJ[htONA */

    /*** J[hXe[^XNA ***/
    SDCARD_ErrStatus = SDMC_NORMAL;
    SDCARD_Status    = SDMC_NORMAL;
    
    /*** J[hCSD WPrbgNA ***/
    SDCARD_WP_FLAG0 = 0;
    SDCARD_WP_FLAG1 = 0;
    SDCARD_WP_PERMANENT = 0;
    SDCARD_WP_TEMPORARY = 0;
    
    pSDCARD_info   = NULL;
    SDCARD_OutFlag = FALSE;            /* J[hrotONA */
    
    irq_core_flag = OS_DisableInterrupts();    /* ݋֎~ */

        *SDIF_CNT_L = (SDIF_CNT_FCLR | SDIF_CNT_USEFIFO); //bp[WX^
        *SDIF_CNT_L = 0x0000;
        *SDIF_FDS_L = 0;
        *SDIF_FSC_L = 1;
        SD_Init();                                 /* SD Card I/F  */
        SD_AndFPGA( SD_OPTION, SD_CD_DETECT_TIME); /* CD o^C[NA */

        SD_port_en_numbers = SDCARD_PORT_NO_MAX; /*** T|[g|[gftHgɐݒ ***/
        SD_port_number = 0;                      /*** ݂̃|[gԍftHgɐݒ      ***/

        SDCARD_Backup_port0();                   /* port0 backup    */
        SDCARD_Backup_port1();                   /* port1 backup    */
    
    OS_RestoreInterrupts( irq_core_flag);      /* 荞ݐݒɖ߂ */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_Layer_Init

  Description:  initialize sequence for SD card.
                SDJ[hK̏菇

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static SDMC_ERR_CODE SDCARD_Layer_Init(void)
{
//    SYSTIM  wait_tim, limit_tim;

//    u16 memory_exist, function_number;
    SDCARD_Status = SDMC_NORMAL;    /* J[hXe[^XNA */
/*    func_SDCARD_CallBack = NULL;    */
    pSDCARD_info = NULL;
    SDCARD_EndFlag = TRUE;          /* ]tOZbg */
    SDCARD_MMCFlag = FALSE;         /* MMCJ[htONA */
    SDCARD_SDHCFlag = FALSE;
    SDCARD_SDFlag = FALSE;          /* SDJ[htONA */
    SDCARD_OutFlag = FALSE;         /* J[hrotONA */
    ulSDCARD_Size = 0;              /* J[hSZN^NA */
//    init_io_exist = 0;
//    init_mem_exist = 0;

    SD_SetFPGA(SD_CLK_CTRL,(SD_CLK_CTRL_128)); /* SDNbN̎g 261KHz(100`400khz) */
    SD_EnableClock();                          /* SDJ[h̃NbNCl[uɂ */

    /* SD I/F_~[80NbN(1mSec)]҂ */
#if (TARGET_OS_CTR == 1)
    dly_tsk( 1);
#else
//    OS_Sleep( 1);
    SVC_WaitByLoop( 17900); //179*4TCN=716TCN=10024ns=10us
#endif
    
    SDCARD_ErrStatus = SDMC_NORMAL;            /* G[Xe[^XNA */

    SDCARD_TimerStop();                        /* ^CAEgp^C}Xgbv */

#if TIMEOUT
    SDCARD_TimerStart(SDCARD_RESET_TIMEOUT);   /* ^CAEgp^C}X^[g */
#endif

    /***********************SDIOeXg*************************/
//    SDIO_RWDirectCommand();
/*    while( 1) {
        SDIO_ReadOCR();
        if(SDCARD_ErrStatus){                       // G[Xe[^X̊mFiG[LHj
            init_io_exist = 0;
            break;                                  //
        }
        memory_exist = SD_OCR[1] & SD_OCR_MEMORY_PRESENT;
        function_number = (SD_OCR[1] & SD_OCR_NUM_OF_FUNC)>>12;
        if( function_number > 0) {                  // function number  0łȂ?
            if( (SD_OCR[0] == 0x0000)&&((SD_OCR[1]&SD_OCR_OCR_23_16)==0x0000)) {    //OCR invalid?
                if( memory_exist) {                 //݂邩?
                    init_mem_exist = 1;
                    break;                          //J[h
                }else{
                    init_mem_exist = 0;
                    return SDCARD_ErrStatus;        //G[(OCR invalid & Ȃ)
                }
            }
            SDIO_WriteOCR();
            if(SDCARD_ErrStatus){                   // G[Xe[^X̊mFiG[LHj
                init_io_exist = 0;
                return SDCARD_ErrStatus;
            }
            init_io_exist = 1;
            memory_exist = SD_OCR[1] & SD_OCR_MEMORY_PRESENT;
            if( memory_exist) {
                break;                //
            }else{
                //IȎ(CMD3)
            }
        }
    }
*/
    /******************************************************************/

    SDCARD_TimerStop();                         /* ^CAEgp^C}Xgbv */

#if TIMEOUT
    SDCARD_TimerStart(SDCARD_RESET_TIMEOUT);    /* ^CAEgp^C}X^[g */
#endif

    PRINTDEBUG( " CMD0(GO_IDLE_STATE)\n");
    SD_ClrErr((u16)(~SDMC_ERR_FPGA_TIMEOUT));   /* ^CAEgȊÕG[NA */
    {
/*PRINTDEBUG( "SD_INFO1      : 0x%x\n", SD_INFO1);
PRINTDEBUG( "SD_INFO2      : 0x%x\n", SD_INFO2);
PRINTDEBUG( "SD_INFO1_MASK : 0x%x\n", SD_INFO1_MASK);
PRINTDEBUG( "SD_INFO2_MASK : 0x%x\n", SD_INFO2_MASK);
PRINTDEBUG( "SD_CLK_CTRL   : 0x%x\n", SD_CLK_CTRL);
PRINTDEBUG( "SD_SIZE       : 0x%x\n", SD_SIZE);

PRINTDEBUG( "SD_INFO1_MASK : 0x%x\n", (*(vu32 *)(SD_IP_BASE + 0x20)));*/
    }
    SD_Command(SD_CMD_CMD | GO_IDLE_STATE);     /* CMD0sAX|XmF */
    if(SDCARD_ErrStatus){                       /* G[Xe[^X̊mFiG[LHj */
        return SDCARD_ErrStatus;
    }

    /*------- idle state -------*/
#if (TARGET_OS_CTR == 1)
    dly_tsk( 1);            /* 1ms҂ */
#else
//    OS_Sleep( 1);
    SVC_WaitByLoop( 17900); //179*4TCN=716TCN=10024ns=10us
#endif
    SD_SendIfCond();                            /* CMD8sAX|XmF */
    if( !SDCARD_SDHCFlag) {                     /* SDHCȊO͎sĂ͂Ȃ̂ */
        SDCARD_ErrStatus = SDMC_NORMAL;         /* G[tONAĂ */
    } //:CMD8ɂ肱Ŋ荞݂I

    while(!(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT)){ /* ^CAEgɂȂ甲 */
        SD_ClrErr((u16)(~SDMC_ERR_FPGA_TIMEOUT));

        SD_RCA = 0;                             /* RCA = 0Zbg */
        if(!SDCARD_MMCFlag){                    /* MMCJ[htO 0(OFF) ? */
            if(!SD_AppCommand()){               /* CMD55 s킩? */
                SDCARD_MMCFlag = FALSE;         /* MMCJ[htONA */
                if(!SD_AppOpCond()){            /* ACMD41s킩?(OCR31bit = L ̎ No Response) */
                    SDCARD_SDFlag = TRUE;       /* SDJ[htOZbg */
                    break;
                }
            }else{                                        /* CMD55 IȂ */
                if(SDCARD_ErrStatus == SDMC_ERR_TIMEOUT){ /* ^CAEg(==No Response)? */
                    SDCARD_MMCFlag = TRUE;                /* MMCJ[htOZbg */
                }else{
//                    break;                //RgAEgȂSDJ[h̏Ɏs
                }
            }
        }
        if(SDCARD_MMCFlag){                     /* MMCJ[htO 1(ON) ̂Ƃ */
            SD_RCA = 1;                         /* RCA = 1Zbg */
            if(!SD_SendOpCond()){               /* CMD1s킩? */
                break;
            }
        }
    }
/*
    if( SDCARD_SDHCFlag) {
        SD_ReadOCR();
    }
*/    
    SDCARD_TimerStop();                               /* ^CAEgp^C}Xgbv */
    if(SDCARD_ErrStatus){                             /* G[Xe[^X̊mFiG[LHj*/
        if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[`FbN */
            SD_ClrErr(SDMC_ERR_FPGA_TIMEOUT);         /* ^CAEgG[̐ݒNA */
            SD_SetErr(SDMC_ERR_RESET);                /* J[hZbgR}h1.5b^CAEgG[̐ݒ */
        }
        SDCARD_MMCFlag = FALSE;                       /* MMCJ[htONA */
        return SDCARD_ErrStatus;
    }

#if TIMEOUT
    SDCARD_TimerStart(SDCARD_INITIAL_TIMEOUT); /* ^CAEgp^C}X^[g */
#endif
    SD_SendCID();                            /* CMD2s X|XmF */
    if(SDCARD_ErrStatus){                    /* G[Xe[^X̊mFiG[LHj */
        return SDCARD_ErrStatus;
    }
    while(1){
        SD_SendRelativeAddr();               /* CMD3s X|XmF I RCA<-ResRCA */
        if(SDCARD_ErrStatus){                /* G[Xe[^X̊mFiG[LHj */
            return SDCARD_ErrStatus;
        }
        if(SD_RCA != 0){
            break;
        }
    }
    /*------- standby state -------*/
    SD_SendCSD();                            /* CMD9s X|XmF */
    if(SDCARD_ErrStatus){                    /* G[Xe[^X̊mFiG[LHj */
        return SDCARD_ErrStatus;
    }
    SDCARD_WP_PERMANENT = (u16)(SD_CSD[0] & (u16)(SDCARD_WP_PERMANENT_BIT));
    SDCARD_WP_TEMPORARY = (u16)(SD_CSD[0] & (u16)(SDCARD_WP_TEMPORARY_BIT));

    /* ]xݒ */
    SD_ClockDivSet(SD_RSP5);                 /* SDJ[h̓NbNݒ (CSD[5]) */
    if(SDCARD_ErrStatus){                    /* G[Xe[^X̊mFiG[LHj */
        return SDCARD_ErrStatus;
    }
    /* Command toggles acard between the Stand-by and Transfer states */
    SD_SelectCard();                         /* CMD7s X|XmF */
    if(SDCARD_ErrStatus){                    /* G[Xe[^X̊mFiG[LHj */
        return SDCARD_ErrStatus;
    }

    /*------- translate state -------*/

    SDCARD_SectorSize = SECTOR_SIZE;         /* ZN^TCY ftHg 512bytes */
    SD_SetBlockLength(SDCARD_SectorSize);    /* CMD16 ubNTCY̐ݒ */
    if(SDCARD_ErrStatus){                    /* G[Xe[^X̊mFiG[LHj */
        return SDCARD_ErrStatus;
    }

#if (SCR_ON == 1)
    SD_SelectBitWidth(FALSE);                /* CMD55->ACMD6 rbg̑I 1bit */

    /* ACMD51 s SD configuration register (SCR) */
    if(SDCARD_SDFlag){                       /* SDJ[htO ON `FbN */
        SDCARD_TimerStop();                  /* ^CAEgp^C}Xgbv */
        i_sdmcSendSCR();
        if(SDCARD_ErrStatus){                /* G[Xe[^X̊mFiG[LHj */
            return SDCARD_ErrStatus;
        }
#if TIMEOUT
        SDCARD_TimerStart(SDCARD_CLOCK_WAIT);/* ^CAEgp^C}X^[g */
#endif
    }
#endif
    SD_EnableClock();                        /* SD-CLK Enable */

    if(SDCARD_MMCFlag){                      /* MMCJ[h ON `FbN */
        if( ((SD_CSD[7] & 0x3C)>>2) >= 4) {
            MMCP_SetBusWidth( TRUE);
        }
//        SD_SelectBitWidth(FALSE);            /* CMD55->ACMD6 rbg̑I 1bit */
    }else{
        SD_SelectBitWidth(TRUE);             /* CMD55->ACMD6 rbg̑I 4bit */
    }

    SDCARD_TimerStop();                      /* ^CAEgp^C}Xgbv */

    if(SDCARD_SDFlag){                       /* SDJ[htO ON `FbN */
        if (SDCARD_SD_Status())              /* CMD55->ACMD13 J[hXe[^X擾 */
            return SDCARD_ErrStatus;
        if(SD_SDSTATUS[1] & SD_MEMORY_CARD){
            SDCARD_SDFlag = FALSE;           /* SDJ[htONA */
        }
    }

    i_sdmcCalcSize();

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         i_sdmcCalcSize

  Description:  

  Arguments:    

  Returns:      None
 *---------------------------------------------------------------------------*/
void i_sdmcCalcSize( void)
{
    u32     ulCSize;
    u16     read_block_len_val, mult_val;
  
    /*--------------J[hTCY̎Zo---------------*/
    if( ((SD_CSD[7] & CSD_STRUCT_BIT_127_126) >> 6) == 0x1) {    //SDHĈƂ
        sdmc_current_spec.csd_ver2_flag = 1;
        ulCSize = (u32)((((u32)(SD_CSD[3]) & CSD_C_SIZE_BIT_69_56) << 8) +
                    ((SD_CSD[2] & CSD_C_SIZE_BIT_55_48) >> 8) + 1);
        ulCSize = ulCSize * 1024;    //Ƃ512KBytePʂȂ̂512BytePʂɂ
        /* f[^̈TCYZo */
        sdmc_current_spec.memory_capacity = ulCSize;
        ulSDCARD_Size = ulCSize;
        /* veNg̈TCYZo */
        sdmc_current_spec.protected_capacity = (u32)((((SD_SwapByte( &SD_SDSTATUS[2])) << 16) +
                                            (SD_SwapByte( &SD_SDSTATUS[3]))) / 0x200);
        /*g[^TCYZo */
        sdmc_current_spec.card_capacity = sdmc_current_spec.memory_capacity +
                                            sdmc_current_spec.protected_capacity;
        
    }else{                                                      //]SDJ[ĥƂ
        sdmc_current_spec.csd_ver2_flag = 0;
        ulCSize = (u32)(((SD_CSD[3] & CSD_C_SIZE_BIT_71_62) >> 6) + 
                    ((SD_CSD[4] & CSD_C_SIZE_BIT_73_72) << 10) + 1);
        mult_val = (u16)(((SD_CSD[2] & CSD_C_SIZE_MULT) >> 7) + 2);    //2̏搔
        ulCSize = ulCSize << mult_val;
        if(SDCARD_MMCFlag){                        /* MMCJ[htOON `FbN */
            read_block_len_val = (u16)((SD_CSD[4] & CSD_READ_BL_LEN) >> 8);
            ulCSize = (ulCSize << read_block_len_val);
        }else{                                    /* SDJ[htO(SDCARD_SDFlag)ON ̂͂ */
            read_block_len_val = (u16)(((SD_CSD[1] & CSD_WRITE_BL_LEN_BIT_25_24) << 2) |
                        ((SD_CSD[0] & CSD_WRITE_BL_LEN_BIT_23_22) >> 14));
            ulCSize = (ulCSize << read_block_len_val);
        }
        /* f[^̈TCYZo */
        ulCSize /= SDCARD_SectorSize;            /* SZN^̎Zo */
        sdmc_current_spec.memory_capacity = ulCSize;
        ulSDCARD_Size += ulCSize;                /* SZN^̃Zbg */
        /* veNg̈TCYZo */
        sdmc_current_spec.protected_capacity = (u32)((SD_SwapByte( &SD_SDSTATUS[2])) << 16) +
                                                     (SD_SwapByte( &SD_SDSTATUS[3]));
        sdmc_current_spec.protected_capacity <<= mult_val;
        sdmc_current_spec.protected_capacity <<= read_block_len_val;
        sdmc_current_spec.protected_capacity /= SDCARD_SectorSize;    //TODO:\̂ɂ܂Ƃ߂邱
        /*g[^TCYZo */
        sdmc_current_spec.card_capacity = sdmc_current_spec.memory_capacity +
                                            sdmc_current_spec.protected_capacity;
    }
    sdmc_current_spec.SS = SDCARD_SectorSize;

    PRINTDEBUG( "SD memory capacity : 0x%x\n", sdmc_current_spec.memory_capacity);
    PRINTDEBUG( "SD protected capacity : 0x%x\n", sdmc_current_spec.protected_capacity);
    PRINTDEBUG( "SD total capacity : 0x%x\n", sdmc_current_spec.card_capacity);
}

/*---------------------------------------------------------------------------*
  Name:         i_sdmcMPInit

  Description:  initialize SD card in multi ports.
                }`|[gSDJ[h

  Arguments:    

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
static SDMC_ERR_CODE i_sdmcMPInit( void)
{
    /*|[gIԁA}
    SD_port_number, SD_CheckFPGAReg(SD_INFO1,SD_INFO1_DETECT),
    SD_CheckFPGAReg(EXT_CD,EXT_CD_PORT1_DETECT)
    */
  
    if( SD_CheckFPGAReg( SD_INFO1, SD_INFO1_DETECT)) {
        sdmcSelect( (u16)SDMC_PORT_CARD);
        SDCARD_ErrStatus = SDCARD_Layer_Init();
    }
    if( SD_CheckFPGAReg( EXT_CD, EXT_CD_PORT1_DETECT)) {
        sdmcSelect( (u16)SDMC_PORT_NAND);
        SDCARD_ErrStatus = SDCARD_Layer_Init();
    }
    SDCARD_OutFlag = FALSE;     /* rotOZbg */

    SDCARD_TimerStop();         /* ^CAEgp^C}Xgbv */
    SD_DisableClock();          /* SD-CLK Disable */
    SD_EnableInfo();            /* SD Card  } 荞݋ */
    
    return SDCARD_ErrStatus;
}


/*---------------------------------------------------------------------------*
  Name:         sdmcGetStatus

  Description:  get card status
                J[h̏Ԃ擾
                bit15 SDJ[hʃrbg(o1)
                bit14 MMCJ[hʃrbg(o1)
                bit10 IO3 card detect(o1)     CTRł̓vAbv̂ߎgȂ
                bit9 IO3 card inserted(}1) CTRł̓vAbv̂ߎgȂ
                bit8 IO3 card removed(E1)     CTRł̓vAbv̂ߎgȂ
                bit7 write protect(݋֎~̏ꍇ1)
                bit5 card detect(o1)
                bit4 card inserted(}1)
                bit3 card removed(E1)
                bit2 R/W access all end
                bit0 Response end

  Arguments:    *status : J[h̏Ԃi[ϐւ̃|C^

  Returns:      0 : success
                > 0 : error code
 *---------------------------------------------------------------------------*/
/*-----------------------------------------------
SD_INFO1WX^
bit[10,9,8,7] = DAT3CD, DAT3IN, DAT3OUT, WP
bit[5, 4, 3, 2] = CD, INS, REM, ALLEND
bit0 = RESEND
-------------------------------------------------
EXT_CDWX^
bit[2,1,0] = P1CD, P1INS, P1REM
-------------------------------------------------
EXT_CD_DAT3WX^
bit[2, 1, 0] = P1DCD, P1DINS, P1DREM
-------------------------------------------------
EXT_WPWX^
bit0 = P1WP
-----------------------------------------------*/
SDMC_ERR_CODE sdmcGetStatus(u16 *status)
{
    u16 SD_INFO1_STATUS;
    
    SD_INFO1_STATUS = SD_INFO1;         /* SD_INFO1WX^ǂݏo */
    *status = SD_INFO1_STATUS;          /* _] */

    /*--- |[g0̂Ƃ ---*/
    if(SD_port_number == SDCARD_PORT0)
    {
        *status &= SDCARD_FLAG_CLR;     /* SD/MMCtONA */
    }
    /*--- |[g1̂Ƃ ---*/
    else if (SD_port_number == SDCARD_PORT1)
    {
        *status &= SDCARD_PORT1_CLR;    /* port1Ɋ֌WȂNA */
        SD_INFO1_STATUS = (u16)((EXT_CD & 0x0007) << 3);
        SD_INFO1_STATUS |= ((EXT_CD_DAT3 & 0x0007) << 8);
        SD_INFO1_STATUS |= ((EXT_WP & 0x0001) << 7);
        *status |= SD_INFO1_STATUS;     /* J[hport1tOݒ   */
    }
    /*--- SD/MMCtOZbg ---*/
    if( SDCARD_MMCFlag) {               /* oJ[hMMCJ[h̎ */
        *status |= SDCARD_FLAG_MMC;     /* J[h蕔MMCJ[h */
    }
    if( SDCARD_SDFlag) {                /* oJ[hSDJ[h̎ */
        *status |= SDCARD_FLAG_SD;      /* J[h蕔SDJ[h */
    }

    return SDMC_NORMAL;
}


/*---------------------------------------------------------------------------*
  Name:         SDCARD_GetR1Status

  Description:  get the card status of R1 response.
                R1X|X̃J[hXe[^X擾B

  Arguments:    None

  Returns:      SDCARD_Status : R1[39:8]
 *---------------------------------------------------------------------------*/
static u32 SDCARD_GetR1Status(void)
{
    /* SD_CheckStatusSDCARD_Statusɒl */
    return SDCARD_Status;
}


/*---------------------------------------------------------------------------*
  Name:         sdmcReadFifo

  Description:  read from card.
                bp[FIFOgpăJ[h̓ǂݏoB

  Arguments:    buf : ǂݏof[^i[邽߂̃obt@̃AhX
                bufsize : ǂݏoTCYiZN^j
                offset : ǂݏoJnItZbgiZN^ԍj
                info : sʂi[邽߂̍\̂ւ̃AhX

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
SDMC_ERR_CODE sdmcReadFifo(void* buf,u32 bufsize,u32 offset,void(*func)(void),SdmcResultInfo *info)
{
    SDCARDMsg     SdMsg;
    OSMessage     recv_dat;
    SDMC_ERR_CODE api_result;                    //SDCARD֐̕Ԃl

    if( bufsize == 0) {
       return( SDMC_ERR_PARAM);
    }
  
    SdMsg.buf       = buf;
    SdMsg.bufsize   = bufsize;
    SdMsg.offset    = offset;
    SdMsg.func      = func;
    SdMsg.info      = info;
    SdMsg.operation = SD_OPERATION_READ_WITH_FIFO;

    recv_dat = (OSMessage)&SdMsg; //SdMsg̃AhX`
    OS_SendMessage( &sdmc_dtq, recv_dat, OS_MESSAGE_BLOCK);
    
    /* Ԃl҂ */
    OS_ReceiveMessage( &sdmc_result_dtq, &recv_dat, OS_MESSAGE_BLOCK);
    api_result = (SDMC_ERR_CODE)recv_dat;

    return api_result;
}

/*---------------------------------------------------------------------------*
  Name:         SDCARDi_ReadFifo

  Description:  read from card.
                bp[FIFOgpăJ[h̓ǂݏoB

  Arguments:    buf : ǂݏof[^i[邽߂̃obt@̃AhX
                bufsize : ǂݏoTCYiZN^j
                offset : ǂݏoJnItZbgiZN^ԍj
                info : sʂi[邽߂̍\̂ւ̃AhX

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
static SDMC_ERR_CODE SDCARDi_ReadFifo(void* buf,u32 bufsize,u32 offset,void(*func)(void),SdmcResultInfo *info)
{
    u32* DmaReg;
    SDMC_ERR_CODE result;
    
    if( sdmc_dma_no == SDMC_NOUSE_DMA) {
        /* FIFO Empty荞ݖAFIFO Full荞ݗL */
        *(SDIF_CNT) = (*(SDIF_CNT) & (~SDIF_CNT_FEIE)) | SDIF_CNT_FFIE;
    }else{
        *(SDIF_CNT) = (*(SDIF_CNT) & (~(SDIF_CNT_FFIE | SDIF_CNT_FEIE)));
    }
    *(SDIF_FDS) = (u16)SDCARD_SectorSize;   /* FIFÕf[^TCY */
    *(SDIF_FSC) = bufsize;
    *(SDIF_CNT) |= (SDIF_CNT_FCLR | SDIF_CNT_USEFIFO);        /* FIFOgptOON */
    CC_EXT_MODE = CC_EXT_MODE_DMA;          /* DMA[hON */

    if( sdmc_dma_no != SDMC_NOUSE_DMA) {
        DmaReg = (u32*)(REG_DMA4SAD_ADDR + (0x1C * sdmc_dma_no));
        *(u32*)(DmaReg+6) =  0;            //reg_MI_DMA4CNT
        *(u32*)(DmaReg++) =  (u32)SDIF_FI; //reg_MI_DMA4SAD
        *(u32*)(DmaReg++) =  (u32)buf;     //reg_MI_DMA4DAD
        *(u32*)(DmaReg++) = ((SDCARD_SectorSize / sizeof(u32)) * bufsize); //][h(reg_MI_DMA4TCNT)
        *(u32*)(DmaReg++) = (SDCARD_SectorSize / sizeof(u32));             //][h(reg_MI_DMA4WCNT)
        *(u32*)(DmaReg++) = 0;                                             //C^[o(reg_MI_DMA4BCNT)
        *(u32*)(DmaReg+1) = REG_MI_DMA4CNT_E_MASK | 0x08000000 | 0x00070000 | 0x4000; //SDNA128WC, SRC-FIX(reg_MI_DMA4CNT)
    }

    result = SDCARDi_Read( buf, bufsize, offset, func, info);

    /* FIFO */
    *(SDIF_CNT) &= (~SDIF_CNT_USEFIFO);     /* FIFOgptOOFF */
    CC_EXT_MODE = CC_EXT_MODE_PIO;          /* PIO[h(DMA[hOFF) */
    
    return result;
}

/*---------------------------------------------------------------------------*
  Name:         sdmcRead

  Description:  read from card.
                J[h̓ǂݏoB

  Arguments:    buf : ǂݏof[^i[邽߂̃obt@̃AhX
                bufsize : ǂݏoTCYiZN^j
                offset : ǂݏoJnItZbgiZN^ԍj
                info : sʂi[邽߂̍\̂ւ̃AhX

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
SDMC_ERR_CODE sdmcRead(void* buf,u32 bufsize,u32 offset,void(*func)(void),SdmcResultInfo *info)
{
    SDCARDMsg     SdMsg;
    OSMessage     recv_dat;
    SDMC_ERR_CODE api_result;

    if( bufsize == 0) {
       return( SDMC_ERR_PARAM);
    }
  
    SdMsg.buf       = buf;
    SdMsg.bufsize   = bufsize;
    SdMsg.offset    = offset;
    SdMsg.func      = func;
    SdMsg.info      = info;
    SdMsg.operation = SD_OPERATION_READ;

    recv_dat = (OSMessage)&SdMsg;
    OS_SendMessage( &sdmc_dtq, recv_dat, OS_MESSAGE_BLOCK);
    
    /* Ԃl҂ */
    OS_ReceiveMessage( &sdmc_result_dtq, &recv_dat, OS_MESSAGE_BLOCK);
    api_result = (SDMC_ERR_CODE)recv_dat;

    return api_result;
}

/*---------------------------------------------------------------------------*
  Name:         SDCARDi_Read

  Description:  read from card.
                J[h̓ǂݏoB

  Arguments:    buf : ǂݏof[^i[邽߂̃obt@̃AhX
                bufsize : ǂݏoTCYiZN^j
                offset : ǂݏoJnItZbgiZN^ԍj
                info : sʂi[邽߂̍\̂ւ̃AhX

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
static SDMC_ERR_CODE SDCARDi_Read(void* buf,u32 bufsize,u32 offset,void(*func)(void),SdmcResultInfo *info)
{
    s16           nRetryCount;                  /* gC񐔃JEg */
    SDMC_ERR_CODE SaveErrStatus;                /* G[Xe[^Xۑp */
    u32           SaveStatus;                   /* J[hXe[^Xۑp */
#pragma unused( func)

    for( nRetryCount=0; nRetryCount<SDCARD_RETRY_COUNT; nRetryCount++) {

        SD_EnableClock();                       /* NbN */

    /*    func_SDCARD_CallBack = func;    */
        pSDCARD_info = info;                    /* ۑpsʍ\̂̃AhXݒ */
        ulSDCARD_SectorCount = bufsize;         /* ]ZN^̐ݒ */
        ulSDCARD_RestSectorCount = bufsize;     /* c]ZN^̐ݒ */
        pSDCARD_BufferAddr = buf;               /* f[^i[obt@̃AhXݒ */

        SDCARD_EndFlag = FALSE;                 /* ]tONA */
        SDCARD_ErrStatus = SDMC_NORMAL;         /* G[Xe[^X̃NA */

#if TIMEOUT
        SDCARD_TimerStart(SDCARD_RW_TIMEOUT);   /* ^CAEgp^C}X^[g(4000msec) */
#endif
        /* IPSD_SECCNTWX^LA]ZN^ݒ(CMD12ŝ) */
        SD_EnableSeccnt( ulSDCARD_RestSectorCount);

        thread_flag = TRUE;
        /*--- [hR}hs ---*/
        if( SDCARD_SDHCFlag) {
            SD_MultiReadBlock( offset);         /* Foffset(SDHC̓ZN^TCY512ByteŒ) */
        }else{
            SD_MultiReadBlock( offset * SDCARD_SectorSize); /* Foffset ~ ZN^TCY */
        }
        /*--------------------------*/

        /**/
        PRINTDEBUG( "==Slp Tsk==\n");
#if (TARGET_OS_CTR == 1)
        //can_wup( 0);
        slp_tsk();
#else
        /*--------------------*/
//        OS_TPrintf( "sleep %d\n", __LINE__);
//        OS_SleepThread( NULL);
        /*--------------------*/
#endif
        PRINTDEBUG( "waked\n");

        thread_flag = FALSE;
        while( !SDCARD_EndFlag) {                          /* SDJ[h R/W ANZXItO ONiIj҂ */
            if( SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[? */
                return SDCARD_ErrStatus;                   /* G[I */
            }
        }
        /* G[ĂȂA^CAEgȊÕG[̏ꍇ */
        if(!(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS)){      /* R}hX|X(R1)̃J[hXe[^XG[łȂ? */
            SD_CheckStatus(TRUE);                          /* R}hX|X(R1) Card Status `FbN */
            if(!(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS)){  /* R}hX|X(R1)̃J[hXe[^XG[łȂ? */
                SD_SendStatus();                           /* J[hXe[^X̎擾R}hs */
                SD_CheckStatus(TRUE);                      /* R}hX|X(R1) Card Status `FbN */
            }
        }
        SaveStatus = SDCARD_Status;                   /* J[hXe[^X̕ۑ */
        SaveErrStatus = SDCARD_ErrStatus;             /* G[Xe[^X̕ۑ */

        if( SDCARD_ErrStatus) {                       /* G[Xe[^X̊mF(G[L?) */
            i_sdmcErrProcess();                       /* G[̏(status擾A~) */
        }
        if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[mF */
            return SDCARD_ErrStatus;                  /* G[I */
        }
        SDCARD_Status = SaveStatus;                   /* J[hXe[^X̕A */
        SDCARD_ErrStatus = SaveErrStatus;             /* G[Xe[^X̕A */


        if(SDCARD_ErrStatus == SDMC_NORMAL){          /* G[Xe[^X̊mFiG[Hj*/
            break;                                    /* gC[v甲 */
        }
    }

    SD_DisableClock();                                /* NbN~ */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_FPGA_irq

  Description:  read/write between Card, when BRE or BWE interrupt.
                J[hBRE([hv)܂BWE(Cgv)荞݂
                ƂAۂɃ[h܂̓CgsB

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_FPGA_irq(void)
{
    BOOL    bRead;

    /* FIFOgƂFFIFO荞݋ԂāAReadWriteʂ*/
    /* FIFOgȂƂFBREBWE𒲂ׁAINFO2̓Y荞ݗvtONA */
    bRead = SD_FPGA_irq();

        /*--- SDJ[h̃[h ---*/
        if(bRead){
            if( ulSDCARD_RestSectorCount > 0) {
//              if( ulSDCARD_RestSectorCount == 1) { TTT();}
                if( SDCARD_UseFifoFlag) {                            /*--- FIFOgƂ ---*/
                    MyCpuRecv32( (u32*)SDIF_FI, (u32*)pSDCARD_BufferAddr, SDCARD_SectorSize);
                }else{                                               /*--- FIFOgȂƂ ---*/
                    MyCpuRecv16( (u16*)SD_BUF0, pSDCARD_BufferAddr, SDCARD_SectorSize);
                }
                ulSDCARD_RestSectorCount--;                 /* cZN^fNg */
            }
            if( ulSDCARD_RestSectorCount <= 0) {                 /* vZN^[h? */
                if(SD_CheckFPGAReg(SD_STOP,SD_STOP_SEC_ENABLE)){ /* SD_SECCNTWX^Enable? */
//                    SD_DisableSeccnt();                        /* SD_SECCNTWX^ݒ */
                }else{                                           /* SD_SECCNTWX^DisablêƂ */
                    SD_StopTransmission();                       /* J[h]IFPGAɒʒmiCMD12sj */
                }
            }
        }else{    /*--- SDJ[hւ̃Cg ---*/
            if( ulSDCARD_RestSectorCount > 0) {
              if( ulSDCARD_RestSectorCount == 1) { SDCARD_ReadyToEnd();}
                if( SDCARD_UseFifoFlag) {                            /*--- FIFOgƂ ---*/
                    MI_CpuSend32( pSDCARD_BufferAddr, SDIF_FI, SDCARD_SectorSize);
                }else{                                               /*--- FIFOgȂƂ ---*/
                    MI_CpuSend16( pSDCARD_BufferAddr, SD_BUF0, SDCARD_SectorSize);
                }
                ulSDCARD_RestSectorCount--;                 /* cZN^fNg */
             }
            if( ulSDCARD_RestSectorCount <= 0){                  /* vZN^Cg? */
                if( SDCARD_UseFifoFlag) {                        /* FIFOgpƂ */
                    while( (*SDIF_CNT & SDIF_CNT_NEMP)) {};      /* FIFOɃf[^cĂ邤͏IȂ */
                }
                if(SD_CheckFPGAReg(SD_STOP,SD_STOP_SEC_ENABLE)){ /* SD_SECCNTWX^Enable? */
//                    SD_DisableSeccnt();                        /* SD_SECCNTWX^ݒ */
                }else{                                           /* SD_SECCNTWX^DisablêƂ */
                    SD_StopTransmission();                       /* J[h]IFPGAɒʒmiCMD12sj */
                }
            }
        }
        /* [h/Cgʏ */
        SDCARD_TimerStart(SDCARD_RW_TIMEOUT);   /* ^CAEgp^C}X^[g(4000msec) */
        pSDCARD_BufferAddr+=(512/2);            /* ݃f[^̃obt@AhXCNg */
}


/*---------------------------------------------------------------------------*
  Name:         SDCARD_ReadyToEnd

  Description:  
                J[hƂ̍ŏIZN^]OɍsB

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_ReadyToEnd( void)
{
        *(SDIF_CNT) = (*(SDIF_CNT) & (~(SDIF_CNT_FFIE | SDIF_CNT_FEIE)));
        SD_OrFPGA( SD_INFO2_MASK, SD_INFO2_MASK_BWE);   /* SDJ[h̃f[^ݗv݋֎~ */
        SD_OrFPGA( SD_INFO2_MASK, SD_INFO2_MASK_BRE);   /* SDJ[h̃f[^ݗv݋֎~ */
        SDCARD_TimerStop();             /* ^CAEgp^C}Xgbv */
}

/*---------------------------------------------------------------------------*
  Name:         SYSFPGA_irq

  Description:  insert/remove/error/access end interrupt handler.
                BREBWE荞݂荞݂̃nhB}AG[A
                ANZXI̊荞ݔɂꂼ̏sB

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SYSFPGA_irq(void)
{
    /*--- |[g0̑}荞݃`FbNƃR[obNN ---*/
    if(!SD_CheckFPGAReg( SD_INFO1_MASK, SD_INFO1_MASK_REMOVE)){ /* SD Card  ݋Ԃ? */
        if( SD_CheckFPGAReg( SD_INFO1, SD_INFO1_REMOVE)){       /* SD Card  ? */
            SD_AndFPGA( SD_INFO1,(~SD_INFO1_REMOVE));           /* INFO1̔tO𗎂Ƃ */
            SDCARD_OutFlag = TRUE;                              /* rotOZbg */
            if(func_SDCARD_Out){                                /* R[obN֐Null`FbN */
                SDCARD_IO_Port = SDCARD_PORT0;                  /* J[h|[gԍݒ */
                func_SDCARD_Out();                              /* J[hR[obN֐Ăяo */
            }
        }
    }
    if(!SD_CheckFPGAReg( SD_INFO1_MASK, SD_INFO1_MASK_INSERT)){ /* SD Card } ݋Ԃ? */
        if( SD_CheckFPGAReg( SD_INFO1, SD_INFO1_INSERT)){       /* SD Card } ? */
            SD_AndFPGA( SD_INFO1, (~SD_INFO1_INSERT));          /* INFO1̑}tO𗎂Ƃ */
            SDCARD_OutFlag = FALSE;                             /* rotOZbg */
            if(func_SDCARD_In){                                 /* R[obN֐Null`FbN */
                SDCARD_IO_Port = SDCARD_PORT0;                  /* J[h}|[gԍݒ */
                func_SDCARD_In();                               /* J[h}R[obN֐Ăяo */
            }
        }
    }
    /* (CTR̓|[g1CD[qڑ(ɑ})Ȃ̂ŁA|[g1̑}`FbN͍sȂ) */

    /*--- 荞ݗvƊ荞݃}XNۑ ---*/
    SD_GetFPGA( SD_INFO1_VALUE,         SD_INFO1);
    SD_GetFPGA( SD_INFO1_MASK_VALUE,    SD_INFO1_MASK);
    SD_GetFPGA( SD_INFO2_VALUE,         SD_INFO2);
    SD_GetFPGA( SD_INFO2_MASK_VALUE,    SD_INFO2_MASK);
    /*------------------------------------------*/

    /*--- SD_INFO2̃G[tO쐬 ---*/
    SD_INFO_ERROR_VALUE = (u16)(SD_INFO2_VALUE & (~SD_INFO2_MASK_VALUE));
    /*--- G[Xe[^X쐬 (RESTIMEOUTILAG[̃tO͔fȂ) ---*/
    SDCARD_ErrStatus |= SD_INFO_ERROR_VALUE & (~(SD_INFO2_ERR_RESTIMEOUT)) &
                        (~(SD_INFO2_ERR_ILA)) & SD_INFO2_MASK_ERRSET;

    /*--- RESTIMEOUTILAG[̓tÖʒu炵Ĕf ---*/
    if( SD_INFO_ERROR_VALUE & SD_INFO2_ERR_ILA) {
        SDCARD_ErrStatus |= SDMC_ERR_ILA;                       /* C[KANZXG[ */
    }
    if( SD_INFO_ERROR_VALUE & SD_INFO2_ERR_RESTIMEOUT) {
        SDCARD_ErrStatus |= SDMC_ERR_TIMEOUT;                   /* Response Time out G[ */
    }/*------------------------------------------------------------*/
    
    SD_AndFPGA( SD_INFO2,(~(SD_INFO2_ERROR_SET)));              /* SD_INFO2̃G[tOSėƂ */
    if ( SDCARD_ErrStatus) {                                    /* 炩̃G[Ă邩? */
        SD_OrFPGA( SD_INFO2_MASK, SD_INFO2_MASK_ERRSET);        /* SẴG[荞݂֎~ */
    }
    /*--- SD_INFO2̃G[`FbNI ---*/
    
    
    /*--- SD_INFO1̊荞ݔ󋵃tO쐬 ---*/
    SD_INFO_ERROR_VALUE = (u16)(SD_INFO1_VALUE & (~SD_INFO1_MASK_VALUE));

    if( SD_INFO_ERROR_VALUE & SD_INFO1_MASK_ALL_END) {          /* R/W access all end ݔ? */
        SD_OrFPGA( SD_INFO1_MASK, SD_INFO1_MASK_ALL_END);       /* INFO1 access all end ݋֎~ */

//        OS_TPrintf( "SYS\n");
            SDCARD_TimerStop();                                 /* ^CAEgp^C}Xgbv */
            SD_TransEndFPGA();                                  /* ]I(荞݃}XN֎~ɖ߂) */
            if( SDCARD_EndFlag == FALSE) {                      /* ]IĂȂ? */
                SDCARD_EndFlag = TRUE;                          /* ]tOZbg */
                if( pSDCARD_info) {                             /* Null`FbN */
                    pSDCARD_info->result = SDCARD_ErrStatus;    /* SdmcResultInfo ɏݒ */
                    pSDCARD_info->resid = (ulSDCARD_SectorCount - ulSDCARD_RestSectorCount) *
                                            SDCARD_SectorSize;  /* SdmcResultInfo ɏZN^ݒ */
                }
            }    /* ]Iς݂̂Ƃ */

            return;

        if( SDCARD_ErrStatus != SDMC_NORMAL) {                  /* G[Ăꍇ */
            SDCARD_TimerStop();                                 /* ^CAEgp^C}Xgbv */
            if( SDCARD_EndFlag == FALSE) {                      /* ]IĂȂ? */
                SD_TransEndFPGA();                              /* J[h]̏I */
                SDCARD_EndFlag = TRUE;                          /* ]tOZbg */
            /*    SD_Init();    */                              /* SD CardC^[tF[XZbg&ݒ */
                if( pSDCARD_info) {                             /* Null`FbN */
                    pSDCARD_info->result = SDCARD_ErrStatus;    /* SdmcResultInfo ɏݒ */
                    pSDCARD_info->resid = (ulSDCARD_SectorCount - ulSDCARD_RestSectorCount) *
                                            SDCARD_SectorSize;  /* SdmcResultInfo ɏZN^ݒ */
                }
            }
        }
    }                                                           /* R/W access all end 荞ݖ */
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_TimerStart

  Description:  start timer for measure timeout.
                ^CAEgvJn

  Arguments:    tim : msPʂ̃^CAEg
                (50ms𒴂l̏ꍇ50msPʂɂȂ)

  Returns:      None
 *---------------------------------------------------------------------------*/
static    void    SDCARD_TimerStart(u32 tim)
{
#if (TARGET_OS_CTR == 1)
    sta_alm( sdmc_alm_id, tim);
    PRINTDEBUG( "Timer Started.\n");
#else
    OSTick tim_tick;
    
    tim_tick = OS_MilliSecondsToTicks( tim);        //usPʂTickPʂ
    
    OS_CancelAlarm( &sdmc_alm);                     //A[j
    OS_SetAlarm( &sdmc_alm, tim_tick, SDCARD_Timer_irq, NULL);   //A[Zbg
#endif
} 

/*---------------------------------------------------------------------------*
  Name:         SDCARD_TimerStop

  Description:  stop timer
                ^CAEgv~

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static    void    SDCARD_TimerStop(void)
{
#if (TARGET_OS_CTR == 1)
    stp_alm( sdmc_alm_id);
    PRINTDEBUG( "Timer Stopped.\n");

    //memo:荞݋֎~Ԃɂstp_almĂł~Ȃ̂Œ
#else
    OS_DisableIrq();
    OS_CancelAlarm( &sdmc_alm);
    OS_EnableIrq();
#endif
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_Timer_irq

  Description:  timer interrupt handler.
                ^C}[荞݃nh

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void    SDCARD_Timer_irq(void* arg)
{
#pragma unused( arg)

#if (SD_DEBUG_PRINT_ON == 1)
    u16 tmp;
    
    PRINTDEBUG( ">>>Timer intr(Timeout)\n");
    tmp = SD_INFO1;
    PRINTDEBUG( "SD_INFO1      : 0x%x\n", tmp);
    tmp = SD_INFO1_MASK;
    PRINTDEBUG( "SD_INFO1_MASK : 0x%x\n", tmp);
    tmp = SD_INFO2;
    PRINTDEBUG( "SD_INFO2      : 0x%x\n", tmp);
    tmp = SD_INFO2_MASK;
    PRINTDEBUG( "SD_INFO2_MASK : 0x%x\n", tmp);
    tmp = SD_ERR_STS1;
    PRINTDEBUG( "SD_ERR_STS1   : 0x%x\n", tmp);
    tmp = SD_ERR_STS2;
    PRINTDEBUG( "SD_ERR_STS2   : 0x%x\n", tmp);
    tmp = *(vu16 *)(SD_IF_BASE+0x00);
    PRINTDEBUG( "SD_CNT        : 0x%x\n", tmp);
    tmp = SD_SECCNT;
    PRINTDEBUG( "SD_SECCNT     : 0x%x\n", tmp);
#endif
  
    SDCARD_ErrStatus |= SDMC_ERR_FPGA_TIMEOUT;    /* ^CAEgG[rbg̐ݒ */

    if(SDCARD_EndFlag == FALSE){                    /* ]tO̊mFiNAHj*/
        SDCARD_EndFlag = TRUE;                      /* ]tOZbg */
    /*    SD_TransEndFPGA();    */                  /* J[h]̏I */
    /*    SD_StopTransmission();    */              /* J[h]Iݒ */
    /*    SD_Init();    */                          /* SD CardC^[tF[XZbg&ݒ */
        if(pSDCARD_info){                           /* Null`FbN */
            pSDCARD_info->result = SDCARD_ErrStatus;/* SdmcResultInfo ɏݒ */
            pSDCARD_info->resid = (ulSDCARD_SectorCount - ulSDCARD_RestSectorCount) *
                SDCARD_SectorSize;                  /* SdmcResultInfo ɏZN^ݒ */
        }
    }

    /**/
    PRINTDEBUG( "--Wup sdTsk(Time)--\n");
#if (TARGET_OS_CTR == 1)
    iwup_tsk( sdmc_tsk_id);
    PRINTDEBUG( "id : 0x%x\n", sdmc_tsk_id);
#else
    sdmc_wakeup_count++;
    PRINTDEBUG( "wakeup\n");
    OS_WakeupThreadDirect( &sdmc_tsk);
#endif
}


/*---------------------------------------------------------------------------*
  Name:         i_sdmcSendSCR

  Description:  get SCR register.
                SCR擾iDATCoR8oCgĂjB
                MultiBlock R/W ƈقȂADATCoRœ]ĂSDJ[h
                WX^́AMSBɑĂ邱ƂɒӁB
                (Physical Layer Specification 2.00 p12-13Q)

  Arguments:    None

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
static u16 i_sdmcSendSCR(void)
{
    SDMC_ERR_CODE SaveErrStatus;          /* G[Xe[^Xۑp */
    u32           SaveStatus;             /* J[hXe[^Xۑp */
    u32           ulSave_SectorSize;      /* ZN^TCYۑp */

    SD_EnableClock();                        /* SD-CLK Enable */

    /*    ubNTCY̐ݒ */
    ulSave_SectorSize = SDCARD_SectorSize;   /* ZN^TCY̕ۑ */
    SDCARD_SectorSize = 8;                   /* SCR WX^ ]TCY 8bytes */
    SD_SetBlockLength(SDCARD_SectorSize);    /* SDJ[hf[^]TCY 8byte ݒ */
    if(SDCARD_ErrStatus){                    /* G[Xe[^X̊mFiG[LHj*/
        return SDCARD_ErrStatus;
    }

#if TIMEOUT
    SDCARD_TimerStart(SDCARD_RW_TIMEOUT);    /* ^CAEgp^C}X^[g */
#endif

    if(SD_AppCommand()){                  /* RCAݒ CMD55sIȂ? */
        SD_DisableClock();                /* SD-CLK Disable */
        return SDCARD_ErrStatus;          /* G[I */
    }

/*    func_SDCARD_CallBack = NULL;    */
    pSDCARD_info = NULL;
    ulSDCARD_RestSectorCount = ulSDCARD_SectorCount = 1; /* cZN^TCYAZN^JEgɂPݒ */
    pSDCARD_BufferAddr = SD_SCR;                         /* f[^i[obt@̃AhXݒ */

    /* ]Ȍ */
    SDCARD_EndFlag = FALSE;                  /* ]tONA */
    SDCARD_ErrStatus = SDMC_NORMAL;          /* G[Xe[^X̃NA */

#if (SCR_ON == 1)
    thread_flag = TRUE;
    SD_SendSCR();                            /*    SCR̎擾R}hs */
    PRINTDEBUG( "==Slp Tsk==\n");
#if (TARGET_OS_CTR == 1)
    //can_wup( 0);
    slp_tsk();
#else
    /*--------------------*/
//        OS_TPrintf( "sleep %d\n", __LINE__);
//        OS_SleepThread( NULL);
    /*--------------------*/
#endif
    PRINTDEBUG( "waked\n");
    thread_flag = FALSE;

    while(!SDCARD_EndFlag){                            /* J[hANZXI҂ */
      PRINTDEBUG( "k\n");
        if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){  /* ^CAEgG[mF */
            return SDCARD_ErrStatus;
        }
    }

    if(!(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS)){     /* R}hX|X(R1)̃J[hXe[^XG[łȂmF */
        SD_CheckStatus(FALSE);                        /* R}hX|X(R1) Card Status `FbN */
        if(!(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS)){ /* R}hX|X(R1)̃J[hXe[^XG[łȂmF */
            SD_SendStatus();                      /* J[hXe[^X̎擾R}hs */
            SD_CheckStatus(FALSE);                /* R}hX|X(R1) Card Status `FbN */
        }
    }
    SaveStatus = SDCARD_Status;                   /* J[hXe[^X̕ۑ */
    SaveErrStatus = SDCARD_ErrStatus;             /* G[Xe[^X̕ۑ */
    if(SDCARD_ErrStatus){                         /* G[Xe[^X̊mFiG[LHj*/
        i_sdmcErrProcess();                       /* G[̏ */
    }
    if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[mF */
        return SDCARD_ErrStatus;
    }
    SDCARD_Status = SaveStatus;                   /* J[hXe[^X̕A */
    SDCARD_ErrStatus = SaveErrStatus;             /* G[Xe[^X̕A */
#endif

    SDCARD_SectorSize = ulSave_SectorSize;        /* ۑĂZN^TCY߂idefault:512bytesj*/
    SD_SetBlockLength(SDCARD_SectorSize);         /* SDJ[hf[^]TCY 512bytes ݒ */

    SD_DisableClock();                            /* SD-CLK Disable */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         i_sdmcErrProcess

  Description:  when error is occured, get Card Status to check and stop the
                transfer.
                G[̏BJ[hXe[^X擾Af[^]
                ΃XgbvB

  Arguments:    None

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
static u16 i_sdmcErrProcess(void)
{
    u16 usRSP0;
    
    SDCARD_ErrStatus = SDMC_NORMAL;    /* G[Xe[^X̃NA */

    SDCARD_TimerStop();                /* ^CAEgp^C}Xgbv */

#if TIMEOUT
    SDCARD_TimerStart(SDCARD_ERRPROC_TIMEOUT); /* ^CAEgp^C}X^[g(2000msec) */
#endif

    SD_SendStatus();                   /* CMD13 addressed card sends its status register sAX|X҂ */
    
    if(!SDCARD_ErrStatus){             /* G[Xe[^X̊mFiG[LHj*/
        SD_GetFPGA( usRSP0, SD_RSP0);
        usRSP0 = (u16)(( usRSP0 & RSP_R1_CURRENT_STATE) >> 1);               /* JgXe[go */
        if((usRSP0 == CURRENT_STATE_DATA) || (usRSP0 == CURRENT_STATE_RCV)){ /* SDCARD Status  data rcv ̎ */
            SD_Command(SD_CMD_CMD | STOP_TRANSMISSION);                      /* CMD12iStopTransmissionjs */
        }
    }

    SDCARD_TimerStop();                /* ^CAEgp^C}Xgbv */

    return SDCARD_ErrStatus;
}


/*---------------------------------------------------------------------------*
  Name:         sdmcGetCardSize

  Description:  get card size (number of sectors).
                J[h̃ZN^擾

  Arguments:    None

  Returns:      number of sectors in the SD card which inserted.
 *---------------------------------------------------------------------------*/
u32 sdmcGetCardSize(void)
{
    /* J[hSZN^ (SDCARD_Layer_Init֐ŎZo) */
    return ulSDCARD_Size;
}


/*---------------------------------------------------------------------------*
  Name:         SDCARD_SD_Status

  Description:  get SD Status.
                SDXe[^X擾iDATCoR64oCgĂjB
                J[hXe[^Xł͂ȂƂ𗯈ӁB
                MultiBlock R/W ƈقȂADATCoRœ]ĂSDJ[h
                WX^́AMSBɑĂ邱ƂɒӁB
                (Physical Layer Specification 2.00 p12-13Q)

  Arguments:    None

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
static u16 SDCARD_SD_Status(void)
{
    SDMC_ERR_CODE SaveErrStatus;             /* G[Xe[^Xۑp */
    u32           SaveStatus;                /* J[hXe[^Xۑp */
    u32           ulSave_SectorSize;         /* ZN^TCYۑp */
    u32           ulSaveRestSectorCount;     /* cZN^TCYۑp */

    SD_EnableClock();                        /* SD-CLK Enable */

    /* ubNTCY̐ݒ */
    ulSave_SectorSize = SDCARD_SectorSize;   /* ZN^TCY̕ۑ */
    SDCARD_SectorSize = 64;                  /* SD_STATUS ]TCY 64bytes */
    SD_SetBlockLength(SDCARD_SectorSize);    /* SDJ[hf[^]TCY 64byte ݒ */
    if(SDCARD_ErrStatus){                    /* G[Xe[^X̊mFiG[LHj*/
        return SDCARD_ErrStatus;
    }

#if TIMEOUT
    SDCARD_TimerStart(SDCARD_RW_TIMEOUT);    /* ^CAEgp^C}X^[g(4000msec) */
#endif

    if(SD_AppCommand()){                     /* RCAݒ CMD55s IȂ?*/
        SD_DisableClock();                   /* SD-CLK Disable */
        return SDCARD_ErrStatus;             /* G[I */
    }

/*    func_SDCARD_CallBack = NULL;    */
    pSDCARD_info = NULL;
    ulSaveRestSectorCount = ulSDCARD_RestSectorCount;    /* cZN^TCYۑ(TODO:Ȃ?j */
    ulSDCARD_RestSectorCount = ulSDCARD_SectorCount = 1; /* cZN^TCYAZN^JEgPɐݒ */
    pSDCARD_BufferAddr = SD_SDSTATUS;                    /* f[^i[obt@̃AhXݒ */

    /* ]Ȍ */
    SDCARD_EndFlag   = FALSE;                /* ]tONA */
    SDCARD_ErrStatus = SDMC_NORMAL;          /* G[Xe[^X̃NA */

    thread_flag = TRUE;
    SD_SDStatus();                           /* ACMD13 SD_STATUS̎擾R}hs */
    PRINTDEBUG( "==Slp Tsk==\n");
#if (TARGET_OS_CTR == 1)
    //can_wup( 0);
    slp_tsk();
#else
    /*--------------------*/
//        OS_TPrintf( "sleep %d\n", __LINE__);
//        OS_SleepThread( NULL);
    /*--------------------*/
#endif
    PRINTDEBUG( "waked\n");
    thread_flag = FALSE;

    while(!SDCARD_EndFlag){                           /* J[hANZXI҂ */
        if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[mF */
            return SDCARD_ErrStatus;
        }
    }

    if(!(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS)){     /* R}hX|X(R1)̃J[hXe[^X炩̃G[łȂmF */
        SD_CheckStatus(FALSE);                        /* R}hX|X(R1) Card Status `FbN */
        if(!(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS)){ /* R}hX|X(R1)̃J[hXe[^XG[łȂmF */
            SD_SendStatus();                      /* J[hXe[^X擾R}hsAX|X(R1)҂ */
            SD_CheckStatus(FALSE);                /* R}hX|X(R1) Card Status `FbN */
        }
    }
    SaveStatus = SDCARD_Status;                   /* J[hXe[^X̕ۑ */
    SaveErrStatus = SDCARD_ErrStatus;             /* G[Xe[^X̕ۑ */
    if(SDCARD_ErrStatus){                         /* G[Xe[^X̊mFiG[LHj*/
        i_sdmcErrProcess();                       /* G[̏ */
    }
    if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[mF */
        return SDCARD_ErrStatus;
    }
    SDCARD_Status = SaveStatus;                   /* J[hXe[^X̕A */
    SDCARD_ErrStatus = SaveErrStatus;             /* G[Xe[^X̕A */

    SDCARD_SectorSize = ulSave_SectorSize;            /* ۑĂZN^TCY߂idefault:512bytesj*/
    SD_SetBlockLength(SDCARD_SectorSize);             /* SDJ[hf[^]TCY 512bytes ݒ */
    ulSDCARD_RestSectorCount = ulSaveRestSectorCount; /* ۑĂcZN^TCY߂(TODO:Ȃ?)*/

    SD_DisableClock();                            /* SD-CLK Disable */

    return SDCARD_ErrStatus;
}

/*******************************************************************************/
int MMCP_SetBusWidth( BOOL b4bit)
{
#if 0
    u32 ulSave_SectorSize;                          /* ZN^TCYۑp */
//    u16 TestData;
    u16 Resid;
#endif

    SD_EnableClock();                        /* SD-CLK Enable */
    
#if 0
    TestData = 0x5A;//0xA5;
    
    /* ubNTCY̐ݒ */
    ulSave_SectorSize = SDCARD_SectorSize;             /* ZN^TCY̕ۑ */
    SDCARD_SectorSize = 4;                             /* ]TCY 1oCg */
    SD_SetBlockLength( SDCARD_SectorSize);             /* SDJ[hf[^]TCY 1byte ݒ */
#endif
    /*R}h6s*/
    MMCP_WriteBusWidth( b4bit);
    SD_AndFPGA(SD_OPTION,(~SD_OPTION_WIDTH_1BIT)); /* IPɃrbg̐ݒ(4bit) */
    
#if 0
    /**/
    pSDCARD_info = NULL;
    ulSDCARD_RestSectorCount = ulSDCARD_SectorCount = 1;
    pSDCARD_BufferAddr = &TestData;            /* f[^i[obt@̃AhXݒ */
    SDCARD_EndFlag   = FALSE;                  /* ]tONA */
    SDCARD_ErrStatus = SDMC_NORMAL;            /* G[Xe[^X̃NA */
    
#if TIMEOUT
        SDCARD_TimerStart(SDCARD_RW_TIMEOUT);      /* ^CAEgp^C}X^[g(4000msec) */
#endif
    
    /* IPSD_SECCNTWX^LA]ZN^ݒ(CMD12ŝ) */
//    SD_EnableSeccnt( ulSDCARD_RestSectorCount);

    /*oXeXg*/
    MMCP_BusTest( FALSE);
    
    /**/
    while( !SDCARD_EndFlag) {                         /* J[hANZXI҂ */
        if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[? */
            return SDCARD_ErrStatus;                  /* G[I */
        }
    }
    

    /**/
    ulSDCARD_RestSectorCount = ulSDCARD_SectorCount = 1;/* cZN^TCYAZN^JEgɂPݒ */
    pSDCARD_BufferAddr = &Resid;                         /* f[^i[obt@̃AhXݒ */
    SDCARD_EndFlag   = FALSE;                          /* ]tONA */
    SDCARD_ErrStatus = SDMC_NORMAL;                    /* G[Xe[^X̃NA */

#if TIMEOUT
        SDCARD_TimerStart(SDCARD_RW_TIMEOUT);      /* ^CAEgp^C}X^[g(4000msec) */
#endif

    /**/
    MMCP_BusTest( TRUE);

    /**/
    while( !SDCARD_EndFlag) {                          /* J[hANZXI҂ */
        if( SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[? */
            return SDCARD_ErrStatus;                   /* G[I */
        }
    }

    /*oXeXgiȂoX4bitɊg*/
//    if( TestData == (~(Resid))) {
    if( Resid == 0xA5) {
        SD_AndFPGA(SD_OPTION,(~SD_OPTION_WIDTH_1BIT)); /* IPɃrbg̐ݒ(4bit) */
    }else{
        SD_OrFPGA(SD_OPTION,(SD_OPTION_WIDTH_1BIT)); /* IPɃrbg̐ݒ(1bit) */
    }
    
    SDCARD_SectorSize = ulSave_SectorSize;             /* ۑĂZN^TCYݒiftHg 512bytesɖ߂j*/
    SD_SetBlockLength( SDCARD_SectorSize);             /* SDJ[hf[^]TCY 512bytes ݒ */
#endif
    return( 0);
}
/*******************************************************************************/


/*---------------------------------------------------------------------------*
  Name:         i_sdmcCheckWP

  Description:  check the write protect bit in the SD_INFO1 register.
                SD_INFO1WX^̃CgveNgrbg𒲂ׂ
                i0:CgveNgĂA1:CgveNgĂȂjB
                CTRł̓|[g1͏1B

  Arguments:    None

  Returns:      number of sectors in the SD card which inserted.
 *---------------------------------------------------------------------------*/
static u16  i_sdmcCheckWP(void)
{
    if (SD_port_number == SDCARD_PORT0)            /* |[g0̂Ƃ */
    {
        if(!(SD_CheckFPGAReg(SD_INFO1,SD_INFO1_WRITEPROTECT))) { //WPtOĂȂ?
            SDCARD_ErrStatus |= SDMC_ERR_WP;    //G[tOWPG[rbg𗧂Ă
        }else{                                  //WPtOĂƂ
            SDCARD_ErrStatus &= ~SDMC_ERR_WP;   //G[tOWPG[rbg𗎂Ƃ
        }
        return    SDCARD_ErrStatus;
    }
    else if (SD_port_number == SDCARD_PORT1)       /* |[g1̂Ƃ */
    {  /*TWL̃|[g1̓CgveNgrbg0iveNgԁjȂ̂Ŕ]ĕ]*/
        if((SD_CheckFPGAReg(EXT_WP,EXT_WP_PORT1))) { //WPtOĂȂ?
            SDCARD_ErrStatus |= SDMC_ERR_WP;          //G[tOWPG[rbg𗧂Ă
        }else{                                        //WPtOĂƂ
            SDCARD_ErrStatus &= ~SDMC_ERR_WP;         //G[tOWPG[rbg𗎂Ƃ
        }
        return    SDCARD_ErrStatus;
    }
    return SDMC_ERR_END;    //ɂ͗Ȃ
}

/*---------------------------------------------------------------------------*
  Name:         sdmcWriteFifo

  Description:  write to card.
                bp[FIFOgpăJ[hւ̏݁B

  Arguments:    buf : ݃f[^i[Ăobt@̃AhX
                bufsize : ރTCYiZN^j
                offset : ݊JnItZbgiZN^ԍj
                info : sʂi[邽߂̍\̂ւ̃AhX

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
SDMC_ERR_CODE sdmcWriteFifo( const void* buf,u32 bufsize,u32 offset,void(*func)(),SdmcResultInfo *info)
{
    SDCARDMsg     SdMsg;
    OSMessage     recv_dat;
    SDMC_ERR_CODE api_result;

    if( bufsize == 0) {
       return( SDMC_ERR_PARAM);
    }
  
    SdMsg.buf       = (void*)buf;
    SdMsg.bufsize   = bufsize;
    SdMsg.offset    = offset;
    SdMsg.func      = func;
    SdMsg.info      = info;
    SdMsg.operation = SD_OPERATION_WRITE_WITH_FIFO;

    recv_dat = (OSMessage)&SdMsg;
    OS_SendMessage( &sdmc_dtq, recv_dat, OS_MESSAGE_BLOCK);

    /* Ԃl҂ */
    OS_ReceiveMessage( &sdmc_result_dtq, &recv_dat, OS_MESSAGE_BLOCK);
    api_result = (SDMC_ERR_CODE)recv_dat;
    
    return api_result;
}

/*---------------------------------------------------------------------------*
  Name:         SDCARDi_WriteFifo

  Description:  write to card.
                bp[FIFOgpăJ[hւ̏݁B

  Arguments:    buf : ݃f[^i[Ăobt@̃AhX
                bufsize : ރTCYiZN^j
                offset : ݊JnItZbgiZN^ԍj
                info : sʂi[邽߂̍\̂ւ̃AhX

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
static SDMC_ERR_CODE SDCARDi_WriteFifo( const void* buf,u32 bufsize,u32 offset,void(*func)(),SdmcResultInfo *info)
{
    u32* DmaReg;
    SDMC_ERR_CODE result;

#if (SD_FIFO_EMPTY_FLAG_NEW == 1)
  if( sdmc_dma_no == SDMC_NOUSE_DMA) {
        /* FIFO Full荞ݖAFIFO Empty荞ݗL */
        *(SDIF_CNT) = (*(SDIF_CNT) & (~SDIF_CNT_FFIE)) | SDIF_CNT_FEIE;
    }else{
        *(SDIF_CNT) = (*(SDIF_CNT) & (~(SDIF_CNT_FFIE | SDIF_CNT_FEIE)));
    }
#else
    /* FIFO荞݋֎~ */
    *(SDIF_CNT) = (*(SDIF_CNT) & (~(SDIF_CNT_FFIE | SDIF_CNT_FEIE)));
#endif
  
    *(SDIF_FDS) = (u16)SDCARD_SectorSize;   /* FIFÕf[^TCY */
    *(SDIF_FSC) = bufsize;
    *(SDIF_CNT) |= (SDIF_CNT_FCLR | SDIF_CNT_USEFIFO);        /* FIFOgptOON */
    CC_EXT_MODE = CC_EXT_MODE_DMA;          /* DMA[hON */

  if( sdmc_dma_no != SDMC_NOUSE_DMA) {
        DmaReg = (u32*)(REG_DMA4SAD_ADDR + (0x1C * sdmc_dma_no));
        *(u32*)(DmaReg+6) =  0;                              //reg_MI_DMA4CNT
        *(u32*)(DmaReg++) =  (u32)buf + SDCARD_SectorSize;   //reg_MI_DMA4SAD (new FIFO emptŷƂ1ZN^ォ)
        *(u32*)(DmaReg++) =  (u32)SDIF_FI; //reg_MI_DMA4DAD
        *(u32*)(DmaReg++) = ((SDCARD_SectorSize / sizeof(u32)) * (bufsize-1)); //][h(reg_MI_DMA4TCNT)
        *(u32*)(DmaReg++) = (SDCARD_SectorSize / sizeof(u32));             //][h(reg_MI_DMA4WCNT)
        *(u32*)(DmaReg++) = 0;                                             //C^[o(reg_MI_DMA4BCNT)
        *(u32*)(DmaReg+1) = REG_MI_DMA4CNT_E_MASK | 0x08000000 | 0x00070000 | 0x0800; //SDNA128WC, DEST-FIX(reg_MI_DMA4CNT)
    }

    result = SDCARDi_Write( buf, bufsize, offset, func, info);

    /* FIFO */
    *(SDIF_CNT) &= (~SDIF_CNT_USEFIFO);     /* FIFOgptOOFF */
    CC_EXT_MODE = CC_EXT_MODE_PIO;          /* PIO[h(DMA[hOFF) */

    return result;
}

/*---------------------------------------------------------------------------*
  Name:         sdmcWrite

  Description:  write to card.
                J[hւ̏݁B

  Arguments:    buf : ݃f[^i[Ăobt@̃AhX
                bufsize : ރTCYiZN^j
                offset : ݊JnItZbgiZN^ԍj
                info : sʂi[邽߂̍\̂ւ̃AhX

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
SDMC_ERR_CODE sdmcWrite( const void* buf,u32 bufsize,u32 offset,void(*func)(),SdmcResultInfo *info)
{
    SDCARDMsg     SdMsg;
    OSMessage     recv_dat;
    SDMC_ERR_CODE api_result;

    if( bufsize == 0) {
       return( SDMC_ERR_PARAM);
    }
  
    SdMsg.buf       = (void*)buf;
    SdMsg.bufsize   = bufsize;
    SdMsg.offset    = offset;
    SdMsg.func      = func;
    SdMsg.info      = info;
    SdMsg.operation = SD_OPERATION_WRITE;

    recv_dat = (OSMessage)&SdMsg;
    OS_SendMessage( &sdmc_dtq, recv_dat, OS_MESSAGE_BLOCK);
    
    /* Ԃl҂ */
    OS_ReceiveMessage( &sdmc_result_dtq, &recv_dat, OS_MESSAGE_BLOCK);
    api_result = (SDMC_ERR_CODE)recv_dat;
    
    return api_result;
}

/*---------------------------------------------------------------------------*
  Name:         SDCARDi_Write

  Description:  write to card.
                J[hւ̏݁B

  Arguments:    buf : ݃f[^i[Ăobt@̃AhX
                bufsize : ރTCYiZN^j
                offset : ݊JnItZbgiZN^ԍj
                info : sʂi[邽߂̍\̂ւ̃AhX

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
static SDMC_ERR_CODE SDCARDi_Write( const void* buf,u32 bufsize,u32 offset,void(*func)(),SdmcResultInfo *info)
{
    s16           nRetryCount;
    u32           ulResid;
    SDMC_ERR_CODE SaveErrStatus;  /* G[Xe[^Xۑp */
    u32           SaveStatus;     /* J[hXe[^Xۑp */

    if( func != NULL){            /* R[obN֐Null`FbN */
        return SDMC_ERR_PARAM;    /* R}hp[^G[ */
    }

#if (WP_ena == 1)
    if( i_sdmcCheckWP()) {
        return SDMC_ERR_WP;       /*** CgveNg̃`FbN    ***/
    }
#endif
    for(nRetryCount = 0;nRetryCount < SDCARD_RETRY_COUNT;nRetryCount++){

    SD_EnableClock();             /* SD-CLK Enable */

/*    func_SDCARD_CallBack = func;    */
        pSDCARD_info = info;
        ulSDCARD_RestSectorCount = ulSDCARD_SectorCount = bufsize;
        pSDCARD_BufferAddr = (void*)buf;                  /* f[^i[obt@̃AhXݒ */

        SDCARD_EndFlag = FALSE;                    /* ]tONA */
        SDCARD_ErrStatus = SDMC_NORMAL;            /* G[Xe[^X̃NA */

#if (TIMEOUT == 1)
        SDCARD_TimerStart(SDCARD_RW_TIMEOUT);      /* ^CAEgp^C}X^[g(4000msec) */
#endif

        /* IPSD_SECCNTWX^LA]ZN^ݒ(CMD12ŝ) */
        SD_EnableSeccnt( ulSDCARD_RestSectorCount);

      
#if (SD_FIFO_EMPTY_FLAG_NEW == 1)      
        if( SDCARD_UseFifoFlag) {                            /*--- FIFOgƂ ---*/
            ulSDCARD_RestSectorCount--;
            MI_CpuSend32( pSDCARD_BufferAddr, SDIF_FI, SDCARD_SectorSize);
            pSDCARD_BufferAddr += (512/2);
        }
#endif
      

        /*--- CgR}hs ---*/
        if( SDCARD_SDHCFlag) {
            SD_MultiWriteBlock( offset);           /* CgR}hs(FItZbg) */
        }else{
            SD_MultiWriteBlock( offset * SDCARD_SectorSize); /* CgR}hs(FItZbg*ZN^TCY) */
        }
        /*--------------------------*/

        /**/
        while( !SDCARD_EndFlag) {                         /* J[hANZXI҂ */
            if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[? */
                return SDCARD_ErrStatus;                  /* G[I */
            }
        }
        
        /* G[ĂȂA^CAEgȊÕG[̏ꍇ */
        if(!(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS)){     /* R}hX|X(R1)̃J[hXe[^XG[łȂmF */
            SD_CheckStatus(FALSE);                        /* R}hX|X(R1) Card Status `FbN */
            if(!(SDCARD_ErrStatus & SDMC_ERR_R1_STATUS)){ /* R}hX|X(R1)̃J[hXe[^XG[łȂmF */
                SD_SendStatus();                      /* J[hXe[^X̎擾R}hs */
                SD_CheckStatus(FALSE);                /* R}hX|X(R1) Card Status `FbN */
            }
        }
        SaveStatus = SDCARD_Status;                   /* J[hXe[^X̕ۑ */
        SaveErrStatus = SDCARD_ErrStatus;             /* G[Xe[^X̕ۑ */
        if( SDCARD_ErrStatus) {                       /* G[Xe[^X̊mF(G[L?)*/
            i_sdmcErrProcess();                       /* G[̏(status擾A~) */
        }
        if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[mF */
            return SDCARD_ErrStatus;                  /* G[I */
        }
#if RESID
        if( SDCARD_SDFlag) {                          /* SDJ[h(MMCłȂ)̏ꍇ */
            if( SDCARD_UseFifoFlag) {                 /*--- FIFOgpĂƂ ---*/
                *(SDIF_CNT) &= (~SDIF_CNT_USEFIFO);   /* ꎞIFIFOgp[hɂ */
                CC_EXT_MODE = CC_EXT_MODE_PIO;
                i_sdmcGetResid(&ulResid);             /* ݊ZN^̎擾 */
                *(SDIF_CNT) |= SDIF_CNT_USEFIFO;      /* FIFOgp[hɖ߂ */
                CC_EXT_MODE = CC_EXT_MODE_DMA;
            }else{                                    /*--- FIFOgpĂȂƂ ---*/
                i_sdmcGetResid(&ulResid);             /* ݊ZN^̎擾 */
            }
            if(info){
                info->resid = ulResid * SDCARD_SectorSize;/*** pSDCARD_info->resid info->resid ɏC@@2000.08.31. ***/
            }
            if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[mF */
                return SDCARD_ErrStatus;
            }
        }
#endif
        SDCARD_Status = SaveStatus;                   /* J[hXe[^X̕A */
        SDCARD_ErrStatus = SaveErrStatus;             /* G[Xe[^X̕A */
#if RESID
        if( SDCARD_SDFlag) {                          /* SDJ[h(MMCłȂ)̏ꍇ */
            if( bufsize != ulResid){                  /* Cgς݃ZN^Ȃ? */
                SD_SetErr( SDMC_ERR_NUM_WR_SECTORS);  /* G[tOZbg */
            }
        }
#endif
        if( SDCARD_ErrStatus == SDMC_NORMAL) {        /* G[Xe[^X̊mF(G[?)*/
            break;
        }
    }

    SD_DisableClock();                                /* NbN~ */

    return SDCARD_ErrStatus;
}


/*---------------------------------------------------------------------------*
  Name:         i_sdmcGetResid

  Description:  get the numbers of the well written(without errors) blocks.
                ݂ZN^̐擾B
                

  Arguments:    pResid : ݊ZN^Ԃϐւ̃|C^

  Returns:      0 : success
                > 0 : error
 *---------------------------------------------------------------------------*/
static u16 i_sdmcGetResid(u32 *pResid)
{
    u16    Resid[2];
    u32    ulSave_SectorSize;                          /* ZN^TCYۑp */

    SDCARD_ErrStatus = SDMC_NORMAL;                    /* G[Xe[^XNA */

    /* ubNTCY̐ݒ */
    ulSave_SectorSize = SDCARD_SectorSize;             /* ZN^TCY̕ۑ */
    SDCARD_SectorSize = 4;                             /* ]TCY 4oCg */
    SD_SetBlockLength( SDCARD_SectorSize);             /* SDJ[hf[^]TCY 4byte ݒ */
    if(SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){      /* ^CAEgG[? */
        return SDCARD_ErrStatus;
    }

#if (TIMEOUT == 1)
    SDCARD_TimerStart(SDCARD_RW_TIMEOUT);              /* ^CAEgp^C}X^[g(4000msec) */
#endif

    if( SD_AppCommand()) {                             /* RCAݒ CMD55s */
        return SDCARD_ErrStatus;                       /* G[(CMD55IȂ)Ȃ߂ */
    }

/*    func_SDCARD_CallBack = NULL;    */
    pSDCARD_info = NULL;
    ulSDCARD_RestSectorCount = ulSDCARD_SectorCount = 1;/* cZN^TCYAZN^JEgɂPݒ */
    pSDCARD_BufferAddr = Resid;                         /* f[^i[obt@̃AhXݒ */

    SDCARD_EndFlag   = FALSE;                          /* ]tONA */
    SDCARD_ErrStatus = SDMC_NORMAL;                    /* G[Xe[^X̃NA */

    thread_flag = TRUE;
    /*--- ACMD22 Cgς݃ZN^擾R}hs ---*/
    SD_SendNumWRSectors();                                
    /*-------------------------------------------------*/
    PRINTDEBUG( "==Slp Tsk==\n");
#if (TARGET_OS_CTR == 1)
//    can_wup( 0);
    slp_tsk();
#else
    /*--------------------*/
//        OS_TPrintf( "sleep %d\n", __LINE__);
//        OS_SleepThread( NULL);
    /*--------------------*/
#endif
    PRINTDEBUG( "waked\n");
    thread_flag = FALSE;

    while( !SDCARD_EndFlag) {                          /* J[hANZXI҂ */
        if( SDCARD_ErrStatus & SDMC_ERR_FPGA_TIMEOUT){ /* ^CAEgG[? */
            return SDCARD_ErrStatus;                   /* G[I */
        }
    }
    /* G[ĂȂA^CAEgȊÕG[̏ꍇ */
    if( SDCARD_ErrStatus) {                            /* (^CAEgȊO)G[? */
        i_sdmcErrProcess();                            /* G[̏ */
        *pResid = 0L;                                  /* Cgς݃ZN^ 0 ݒ */
    }else{                                             /* G[ĂȂꍇ */
        /* SDJ[h̃WX^MSB瑗Ă邽ߕבւs */
        Resid[1] = SD_SwapByte(&Resid[1]);             /*  1byte Ɖ 1byte ւ */
        (((LELONG *)pResid)->dt2word.low) = Resid[1];  /* Resid[1]̐ݒ */
        Resid[0] = SD_SwapByte(&Resid[0]);             /*  1byte Ɖ 1byte ւ */
        (((LELONG *)pResid)->dt2word.high) = Resid[0]; /* Resid[0]̐ݒ */
    }
    
    SDCARD_SectorSize = ulSave_SectorSize;             /* ۑĂZN^TCYݒiftHg 512bytesɖ߂j*/
    SD_SetBlockLength( SDCARD_SectorSize);             /* SDJ[hf[^]TCY 512bytes ݒ */
    /* SetBlockLengthŃG[oǂ? */

    return SDCARD_ErrStatus;
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_Restore_port0

  Description:  restore registers and variables of port0.
                |[g0̃WX^ϐ𕜋AB

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_Restore_port0(void)
{
    /* registers */
    SD_SetFPGA( SD_CLK_CTRL, SDPort0Context.SD_CLK_CTRL_VALUE);
    SD_SetFPGA( SD_OPTION,   SDPort0Context.SD_OPTION_VALUE);

    /* variables */
    SD_RCA           = SDPort0Context.SD_RCA;
    SDCARD_ErrStatus = SDPort0Context.ErrStatus;
    SDCARD_Status    = SDPort0Context.Status;
    SDCARD_MMCFlag   = SDPort0Context.MMCFlag;
    SDCARD_SDHCFlag  = SDPort0Context.SDHCFlag;
    SDCARD_SDFlag    = SDPort0Context.SDFlag;

    SDCARD_OutFlag = SDPort0Context.OutFlag;
    pSDCARD_info   = NULL;

    /*media registers*/
    MI_CpuCopy8( SDPort0Context.SD_CID, SD_CID, 16);
    MI_CpuCopy8( SDPort0Context.SD_CSD, SD_CSD, 16);
    MI_CpuCopy8( SDPort0Context.SD_OCR, SD_OCR,  4);
    MI_CpuCopy8( SDPort0Context.SD_SCR, SD_SCR,  8);
  
    /*recalc*/
    SDCARD_WP_PERMANENT = (u16)(SD_CSD[0] & (u16)(SDCARD_WP_PERMANENT_BIT));
    SDCARD_WP_TEMPORARY = (u16)(SD_CSD[0] & (u16)(SDCARD_WP_TEMPORARY_BIT));
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_Restore_port1

  Description:  restore registers and variables of port0.
                |[g1̃WX^ϐ𕜋AB

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_Restore_port1(void)
{
    /* registers */
    SD_SetFPGA( SD_CLK_CTRL, SDPort1Context.SD_CLK_CTRL_VALUE);
    SD_SetFPGA( SD_OPTION,   SDPort1Context.SD_OPTION_VALUE);

    /* variables */
    SD_RCA           = SDPort1Context.SD_RCA;
    SDCARD_ErrStatus = SDPort1Context.ErrStatus;
    SDCARD_Status    = SDPort1Context.Status;
    SDCARD_MMCFlag   = SDPort1Context.MMCFlag;
    SDCARD_SDHCFlag  = SDPort1Context.SDHCFlag;
    SDCARD_SDFlag    = SDPort1Context.SDFlag;

    SDCARD_OutFlag = SDPort1Context.OutFlag;
    pSDCARD_info   = NULL;

    /*media registers*/
    MI_CpuCopy8( SDPort1Context.SD_CID, SD_CID, 16);
    MI_CpuCopy8( SDPort1Context.SD_CSD, SD_CSD, 16);
    MI_CpuCopy8( SDPort1Context.SD_OCR, SD_OCR,  4);
    MI_CpuCopy8( SDPort1Context.SD_SCR, SD_SCR,  8);
  
    /*recalc*/
    SDCARD_WP_PERMANENT = (u16)(SD_CSD[0] & (u16)(SDCARD_WP_PERMANENT_BIT));
    SDCARD_WP_TEMPORARY = (u16)(SD_CSD[0] & (u16)(SDCARD_WP_TEMPORARY_BIT));
}


/*---------------------------------------------------------------------------*
  Name:         sdmcSelectedNo

  Description:  get selected port number.
                IĂ|[gԍ擾

  Arguments:    None

  Returns:      [15:8]port numbers which supportediT|[gĂ|[gj
                [7:0]port number which selected nowiIĂ|[gԍj
 *---------------------------------------------------------------------------*/
u16 sdmcSelectedNo(void)
{
    u16 i_sdmcSelect_Value;

    SD_GetFPGA(i_sdmcSelect_Value,SD_PORTSEL);    /* SD_PORTSELWX^l擾 */
    
    return i_sdmcSelect_Value;
}

/*---------------------------------------------------------------------------*
  Name:         sdmcSelect

  Description:  select port.
                |[gI

  Arguments:    select : [15:8]port numbers which supportediT|[gĂ|[gj
                         [7:0]port number which selected nowiI|[gԍj
  Returns:      0 : success
                >0 : error
 *---------------------------------------------------------------------------*/
SDMC_ERR_CODE sdmcSelect(u16 select)
{
    union
    {
        u16    Val;
        struct     
        {
            u8    L;        /* w肵|[g */
            u8    H;        /* T|[gĂ|[g */
        }    PSel;
    }    SDCARD_PSel;

    //TODO : transfer[hstand-by[hɂ?
/*    if( !SDCARD_EndFlag) {                      // ]cĂꍇ̓G[
        return SDMC_ERR_END;
    }
*/
    SDCARD_PSel.Val = select;
    if ((SDCARD_PSel.PSel.H > SDCARD_PORT_NO_MAX) 
        | (SDCARD_PSel.PSel.H < SDCARD_PORT_NO_MIN)
        | (SDCARD_PSel.PSel.L > SDCARD_PORT_SELECT_NO))
    {
        return SDMC_ERR_PARAM;                    /* R}hp[^G[        */
    }else{
        SD_port_en_numbers = SDCARD_PSel.PSel.H;  /* VT|[g|[gۑ */
        SD_port_number = SDCARD_PSel.PSel.L;      /* VI|[gԍۑ */
        
        SD_GetFPGA(SDCARD_PSel.Val,SD_PORTSEL);   /* ݂̃WX^擾 */
        
        if (SDCARD_PSel.PSel.L == 0)              /* ݑIĂ|[g 0 ? */
        {
            SDCARD_Backup_port0();
        }else{
            SDCARD_Backup_port1();
        }
        
        SDCARD_PSel.Val= select;
        
        if (SDCARD_PSel.PSel.L == 0)              /* VIꂽ|[g 0 ? */
        {
            SDCARD_Restore_port0();
        }else{
            SDCARD_Restore_port1();
        }
    
        SD_SetFPGA(SD_PORTSEL,select);            /* WX^փZbg    */
        return SDMC_NORMAL;                       /* 0@^[ */
    }
}


/*---------------------------------------------------------------------------*
  Name:         SDCARD_Thread

  Description:  SDJ[hC^XN

  Arguments:    

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_Thread( void* arg)
{
    SDCARDMsg*    SdMsg;
    OSMessage     current_dat;
    SDMC_ERR_CODE api_result;
#pragma unused( arg)

    while( TRUE) {
        /* bZ[W҂ */
        PRINTDEBUG( "rcv mes sdThread\n");
        OS_ReadMessage( &sdmc_dtq, &current_dat, OS_MESSAGE_BLOCK);
        SdMsg = (SDCARDMsg*)current_dat;
        PRINTDEBUG( "sd task : receive command : %d\n", SdMsg->operation);

        switch( SdMsg->operation) {
          case SD_OPERATION_INIT:
            api_result = i_sdmcInit();
            break;
          case SD_OPERATION_READ:
            PRINTDEBUG( "from:0x%x, sectors:0x%x\n", SdMsg->offset, SdMsg->bufsize);
            api_result = SDCARDi_Read( SdMsg->buf, SdMsg->bufsize,
                                       SdMsg->offset, SdMsg->func,
                                       SdMsg->info);
            break;
          case SD_OPERATION_READ_WITH_FIFO:
            PRINTDEBUG( "from:0x%x, sectors:0x%x\n", SdMsg->offset, SdMsg->bufsize);
            api_result = SDCARDi_ReadFifo( SdMsg->buf, SdMsg->bufsize,
                                           SdMsg->offset, SdMsg->func,
                                           SdMsg->info);
            break;
          case SD_OPERATION_WRITE:
            PRINTDEBUG( "from:0x%x, sectors:0x%x\n", SdMsg->offset, SdMsg->bufsize);
            api_result = SDCARDi_Write( SdMsg->buf, SdMsg->bufsize,
                                        SdMsg->offset, SdMsg->func,
                                        SdMsg->info);
            break;
          case SD_OPERATION_WRITE_WITH_FIFO:
            PRINTDEBUG( "from:0x%x, sectors:0x%x\n", SdMsg->offset, SdMsg->bufsize);
            api_result = SDCARDi_WriteFifo( SdMsg->buf, SdMsg->bufsize,
                                            SdMsg->offset, SdMsg->func,
                                            SdMsg->info);
            break;
          default:
            PRINTDEBUG( "sdmc-thread error : undefined command.\n");
            api_result = SDMC_ERR_COMMAND;
            break;
        }

        PRINTDEBUG( "sd task : operation ends(result : 0x%x), send message begin\n", api_result);

        /*bZ[Wԑ*/
        current_dat = (OSMessage)api_result;
        OS_SendMessage( &sdmc_result_dtq, current_dat, OS_MESSAGE_BLOCK);
        OS_ReceiveMessage( &sdmc_dtq, &current_dat, OS_MESSAGE_BLOCK);
    }
}

/*---------------------------------------------------------------------------*
  Name:         SDCARD_Intr_Thread

  Description:  SDJ[h荞ݏ^XN

  Arguments:    

  Returns:      None
 *---------------------------------------------------------------------------*/
static void SDCARD_Intr_Thread( void* arg)
{
   static i = 0;
    u16        sd_info1;//, sd_info2;
//    OSIntrMode enabled;
#pragma unused( arg)

    OS_EnableInterrupts();
    OS_EnableIrq();
  
    while( 1) {
      i++;
        PRINTDEBUG( "Slp sdIntr\n");

        OS_WaitIrq( FALSE, OS_IE_SD);
      
        (void)OS_ClearIrqCheckFlag( OS_IE_SD);
        *(vu16*)CTR_INT_IF = CTR_IE_SD_MASK;          /*SD荞݂IF*/
      
//        OS_TPrintf( "sdIntr waked\n");
      
      while( 1) {
        /*--- FIFOgƂ ---*/
        if( SDCARD_UseFifoFlag) {
            sd_info1 = SD_INFO1;
            if( (((*SDIF_CNT & SDIF_CNT_FULL)&&(*SDIF_CNT & SDIF_CNT_FFIE)) ||
                ((!(*SDIF_CNT & SDIF_CNT_NEMP))&&(*SDIF_CNT & SDIF_CNT_FEIE)))
                ){
            
//                OS_TPrintf( ">>>SD Intr(FIFO) %d\n", i);// Full or Empty)\n");
                OS_DisableIrqMask( OS_IE_SD);
                SDCARD_FPGA_irq();                        /*J[h̃[hCgv荞*/
                OS_EnableIrqMask( OS_IE_SD);
//                break;
              
#if (SD_FIFO_EMPTY_FLAG_NEW == 1)
#else
            }else{
                /*----- FIFO-EmptyFlagC̏ꍇAWriteBWE荞݂ɗKv -----*/
                if( SD_CheckFPGAReg( SD_INFO2, (SD_INFO2_MASK_BRE | SD_INFO2_MASK_BWE))) {
//                    OS_TPrintf( ">>>SD Intr(R/W Req)\n");
                    //ŎIɃbp[FIFO<->SD_BUF0ԂŒʐMs
                    OS_DisableIrqMask( OS_IE_SD);
                    SDCARD_FPGA_irq();
                    OS_EnableIrqMask( OS_IE_SD);
                    break;
                }
                /*----------*/
#endif              
            }
          
            /* ALL_END */
            sd_info1 = SD_INFO1;
            if( (SD_CheckFPGAReg( sd_info1, SD_INFO1_ALL_END))&&
                ((ulSDCARD_RestSectorCount<=0) || (sdmc_dma_no != SDMC_NOUSE_DMA))) {
//                    while( (*SDIF_CNT & SDIF_CNT_NEMP)) {};      /* FIFOɃf[^cĂ邤͏IȂ */
                ulSDCARD_RestSectorCount = 0; //DMAgp̂Ƃ̓fNgĂȂ̂
//                OS_TPrintf( ">>>SD Intr(End or Err) %d\n", i);
                (void)OS_ClearIrqCheckFlag( OS_IE_SD);
                *(vu16*)CTR_INT_IF = CTR_IE_SD_MASK;  /*SD荞݂IF*/
                OS_DisableIrqMask( OS_IE_SD);
                SYSFPGA_irq();
                OS_EnableIrqMask( OS_IE_SD);
                if( thread_flag) {
                    PRINTDEBUG( "--Wup sdThread!--\n");
                    sdmc_wakeup_count++;
                    PRINTDEBUG( "wakeup\n");
                    OS_WakeupThreadDirect( &sdmc_tsk);
                }
                break;
            }
          break;
        /*--- FIFOgȂƂ ---*/
        }else{
            if( SD_CheckFPGAReg( SD_INFO2, (SD_INFO2_MASK_BRE | SD_INFO2_MASK_BWE))) {
                PRINTDEBUG( ">>>SD Intr(R/W Req)\n");
                OS_DisableIrqMask( OS_IE_SD);
                SDCARD_FPGA_irq();                        /*J[h̃[hCgv荞*/
                OS_EnableIrqMask( OS_IE_SD);
            }else{
                PRINTDEBUG( ">>SD Intr(End or Err)\n");
                OS_DisableIrqMask( OS_IE_SD);
                SYSFPGA_irq();                            /*܂̓G[荞*/
                OS_EnableIrqMask( OS_IE_SD);
                /**/
                if( thread_flag) {
                    PRINTDEBUG( "--Wup sdThread!--\n");
                    sdmc_wakeup_count++;
                    OS_WakeupThreadDirect( &sdmc_tsk);
                }
            }
            break;
        }
      }
    }
}
