/*---------------------------------------------------------------------------*
  Project:  CTR - ELF Loader
  File:     elf_loader.h

  Copyright 2006,2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#ifndef _ELF_LOADER_H_
#define _ELF_LOADER_H_

#include "el_config.h"
#if (TARGET_OS_NITRO == 1)
#include <nitro/fs.h>
#else
#include <ctr.h>
#include <stdio.h>
#endif
#include "elf.h"

#ifdef __cplusplus
extern "C" {
#endif


/*------------------------------------------------------
  ZNVwb_g([hAhXɑΉ)
 -----------------------------------------------------*/
typedef struct {
  void*         next;
  u16           index;
  u16           debug_flag;    /*0:fobOłȂA1:fobO*/
  u32           loaded_adr;
  u32           alloc_adr;
  u32           loaded_size;
  Elf32_Shdr    Shdr;
}ElShdrEx;


/*------------------------------------------------------
  V{Ggg([hAhXɑΉ)
 -----------------------------------------------------*/
typedef struct {
  void*      next;
  u16        debug_flag;       /*0:fobOłȂA1:fobO*/
  u16        thumb_flag;
  u32        relocation_val;
  Elf32_Sym  Sym;
}ElSymEx;


/*------------------------------------------------------
  xj̃NXg(i_elDoRelocateŎgp)
 -----------------------------------------------------*/
typedef struct {
  void* next;
  u32   adr;     /* xjR[h̐擪AhX */
  u32   data;    /* xj̃ev[Ɋi[Ăѐ̃AhXl */
}ElVeneer;


/*------------------------------------------------------
  ELFIuWFNg̊Ǘ
 -----------------------------------------------------*/
typedef void (*i_elReadFunc)( void* buf, void* file_struct, u32 file_base, u32 file_offset, u32 size);
typedef struct {
  void*         ar_head;        /* AR܂ELFt@C̐擪AhX */
  void*         elf_offset;     /* ELFIuWFNg̐擪ւ̃ItZbg */
  void*         buf_current;    /* LoaderƗp */
  u16           shentsize;      /* 1ZNVwb_̃TCY */
  u16           process;        /* Ĕzu */

  ElShdrEx*     ShdrEx;         /* ShdrExXg̐擪 */

  Elf32_Ehdr    CurrentEhdr;    /* ELFwb_ */

  Elf32_Rel     Rel;            /* ĔzuGg */
  Elf32_Rela    Rela;
  Elf32_Sym     Sym;            /* V{Gg */
  Elf32_Shdr    SymShdr;

  ElSymEx*      SymEx;          /* SymExXg̐擪ifobOV{̂݌qj */
  ElSymEx**     SymExTbl;       /* SymExAhX̃e[uiSV{Ԃj*/
  u32           SymExTarget;    /* SymExXg\zV{ZNṼZNVԍ */

  ElVeneer*     VeneerEx;       /* xj̃NXg */
  
  i_elReadFunc  i_elReadStub;   /* [hX^u֐ */
  void*         FileStruct;     /* t@C\ */
    
  u32           entry_adr;      /* GgAhX */
}ElDesc;



/*------------------------------------------------------
  AhXe[u
 -----------------------------------------------------*/
typedef struct {
  void*      next;              /*̃AhXGg*/
  char*      name;              /**/
  void*      adr;               /*AhX*/
  u16        func_flag;         /*0:f[^A1:֐*/
  u16        thumb_flag;        /*0:armR[hA1:thumbR[h*/
}ElAdrEntry;


/*------------------------------------------------------
  ̍Ĕzue[u

  ォAhXe[uQƂΎ̂悤ɉB
  S_ = AdrEntry.adr;
  T_ = (u32)(AdrEntry.thumb_flag);
 -----------------------------------------------------*/
typedef struct {
  void*  next;                  /*̃Gg*/
  char*  sym_str;               /*̊OQƃV{*/
  u32    r_type;                /*P[V^CviELF32_R_TYPE( Rela.r_info)j*/
  u32    S_;                    /*̊OQƃV{AhX*/
  s32    A_;                    /*ς*/
  u32    P_;                    /*ς*/
  u32    T_;                    /*̊OQƃV{ARM/ThumbtO*/
  u32    sh_type;               /*SHT_REL or SHT_RELA*/
  u32    remove_flag;           /*ƂɃZbgiĂǂƂʂjtO*/
}ElUnresolvedEntry;



/* ElDesc  processl */
#define EL_FAILED           0x00
#define EL_INITIALIZED      0x5A
#define EL_COPIED           0xF0
#define EL_RELOCATED        0xF1



/* typedef */
//#if (TARGET_OS_NITRO == 1)
//#else
typedef void *(*ElAlloc)(size_t size);
typedef void (*ElFree)(void* ptr);
typedef u32 (*ElReadImage)( u32 offset, void* buf, u32 size);
//#endif


/*---------------------------------------------------------
 ELFIuWFNg̃TCY߂
 --------------------------------------------------------*/
u32 elGetElfSize( const void* buf);

/*------------------------------------------------------
  _Ci~bNNVXe
 -----------------------------------------------------*/
#if (TARGET_OS_NITRO == 1)
void elInit( void);
#else
void elInit( ElAlloc alloc, ElFree free);
#endif

/*------------------------------------------------------
  ElDesc\̂
 -----------------------------------------------------*/
BOOL elInitDesc( ElDesc* elElfDesc);

/*------------------------------------------------------
  ELFIuWFNg܂͂̃A[JCut@Cobt@ɍĔzu
 -----------------------------------------------------*/
#if (TARGET_OS_NITRO == 1)
u16 elLoadLibraryfromFile( ElDesc* elElfDesc, FSFile* ObjFile, void* buf);
#else
u16 elLoadLibraryfromFile( ElDesc* elElfDesc, int ObjFile, void* buf);
#endif

/*------------------------------------------------------
  ELFIuWFNg܂͂̃A[JCu[Ũ[hAPIʂčĔzu
 -----------------------------------------------------*/
//#if (TARGET_OS_NITRO == 1)
//#else
u16 elLoadLibrary( ElDesc* elElfDesc, ElReadImage readfunc, u32 len, void* buf);
//#endif

/*------------------------------------------------------
  ELFIuWFNg܂͂̃A[JCuobt@ɍĔzu
 -----------------------------------------------------*/
u16 elLoadLibraryfromMem( ElDesc* elElfDesc, void* obj_image, u32 obj_len, void* buf);

/*------------------------------------------------------
  AhXe[ugĖ̃V{
 -----------------------------------------------------*/
u16 elResolveAllLibrary( ElDesc* elElfDesc);


/*------------------------------------------------------
  AhXe[uGg폜
 -----------------------------------------------------*/
BOOL elRemoveAdrEntry( ElAdrEntry* AdrEnt);

/*------------------------------------------------------
  AhXe[uɃGgǉ
 -----------------------------------------------------*/
void elAddAdrEntry( ElAdrEntry* AdrEnt);

/*------------------------------------------------------
  AhXe[uɃX^eBbÑGgǉ
  iELCuWEAKV{ƂĒ`ĂA
    makelstt@C̒`ŏ㏑j
 -----------------------------------------------------*/
void elAddStaticSym( void);


/*------------------------------------------------------
  AhXe[uw蕶ɊYGgT
 -----------------------------------------------------*/
ElAdrEntry* elGetAdrEntry( const char* ent_name);

/*------------------------------------------------------
  AhXe[uw蕶ɊYAhXԂ
 -----------------------------------------------------*/
void* elGetGlobalAdr( const char* ent_name);



/*ɕKvȊ֐*/
//[hɕKvȃ̃oCgZo֐
//elFreeLibrary


#ifdef __cplusplus
}    /* extern "C" */
#endif


#endif    /*_ELF_LOADER_H_*/
