/*---------------------------------------------------------------------------*
  Project:  TWL - ELF Loader
  File:     loader_subset.c

  Copyright 2006,2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include "el_config.h"
#if (TARGET_OS_NITRO == 1)
  #include <twl.h>
#else
  #include <string.h>
#endif

#include "loader_subset.h"

extern ElUnresolvedEntry*    ELUnrEntStart;

static ElVeneer*             ELVenEntStart = NULL;
static ElVeneer*             ELV4tVenEntStart = NULL;


//ARM7ǂ𔻕ʂƂ́A
//#ifdef  SDK_ARM7
//#endif    /*SDK_ARM7*/

/*------------------------------------------------------
  externϐ
 -----------------------------------------------------*/
#if (TARGET_OS_NITRO == 1)
#else
extern ElAlloc   i_elAlloc;
extern ElFree    i_elFree;
#endif


/*------------------------------------------------------
  static֐
 -----------------------------------------------------*/
static u32 el_veneer[3] = { //ARMR[h
    0xE59FC000,    //(LDR r12,[PC])
    0xE12FFF1C,    //(BX  r12)
    0x00000000,    //(data)
};

static u16 el_veneer_t[10] = { //v4tpThumbR[h
    0xB580,  //(PUSH {R7,LR})
    0x46FE,  //(NOP)
    0x46FE,  //(MOV LR, PC)
    0x2707,  //(MOV R7, 7)
    0x44BE,  //(ADD LR,R7)
    0x4F01,  //(LDR R7,[PC+4])
    0x4738,  //(BX R7) ---> ARM routine
    0xBD80,  //(POP {R7,PC} <---LR(v4tł͖Thumb)
    0x0000,  //(data) ARM routine address
    0x0000,  //(data)
};

static void     i_elBuildSymList( ElDesc* elElfDesc, u32 symsh_index);
static ElSymEx* i_elGetSymExfromList( ElSymEx* SymExStart, u32 index);
static ElSymEx* i_elGetSymExfromTbl( ElSymEx** SymExTbl, u32 index);
static void     i_elUnresolvedInfoInit( ElUnresolvedEntry* UnresolvedInfo);
static void     i_elAddUnresolvedEntry( ElUnresolvedEntry* UnrEnt);

static void      i_elAddVeneerEntry( ElVeneer** start, ElVeneer* VenEnt);
static BOOL      i_elRemoveVenEntry( ElVeneer** start, ElVeneer* VenEnt);
static ElVeneer* i_elGetVenEntry( ElVeneer** start, u32 data);
static BOOL      i_elIsFar( u32 P, u32 S, s32 threshold);


/*------------------------------------------------------
  AhXACg
 -----------------------------------------------------*/
#define ALIGN( addr, align_size ) (((addr)  & ~((align_size) - 1)) + (align_size))

static u32 i_elALIGN( u32 addr, u32 align_size);
u32 i_elALIGN( u32 addr, u32 align_size)
{
    u32 aligned_addr;
    
    if( (addr % align_size) == 0) {
        aligned_addr = addr;
    }else{
        aligned_addr = (((addr) & ~((align_size) - 1)) + (align_size));
    }
    
    return aligned_addr;
}


/*------------------------------------------------------
  xjobt@ɃRs[
    start : Ăяo
    data : ѐ
    threshold : ͈͓̔ɊɃxjΎg܂킷
 -----------------------------------------------------*/
void* i_elCopyVeneerToBuffer( ElDesc* elElfDesc, u32 start, u32 data, s32 threshold)
{
    u32*        el_veneer_dest;
    u32         load_start;
    Elf32_Addr  sh_size;
    ElVeneer*   elVenEnt;

    /*--- łɃxj΂g ---*/
    elVenEnt = i_elGetVenEntry( &ELVenEntStart, data);
    if( elVenEnt != NULL) {
        if( i_elIsFar( start, elVenEnt->adr, threshold) == FALSE) {
            PRINTDEBUG( "Veneer Hit!\n");
            return( (void*)elVenEnt->adr);
        }else{
            /*xjĎgȂ̂ŃNXg폜*/
            i_elRemoveVenEntry( &ELVenEntStart, elVenEnt);
        }
    }
    /*--------------------------------------*/

    /*ACgƂ*/
    load_start = i_elALIGN( ((u32)(elElfDesc->buf_current)), 4);
    /*TCYݒ*/
    sh_size = 12;//el_veneer

    /*Rs[*/
    OSAPI_CPUCOPY8( el_veneer, (u32*)load_start, sh_size);

    /*obt@|C^ړ*/
    elElfDesc->buf_current = (void*)(load_start + sh_size);
    el_veneer_dest = (u32*)load_start;
    el_veneer_dest[2] = data;

    /*--- xjXgɒǉ ---*/
    elVenEnt = OSAPI_MALLOC( sizeof( ElVeneer));
    elVenEnt->adr = load_start;
    elVenEnt->data = data;
    i_elAddVeneerEntry( &ELVenEntStart, elVenEnt);
    /*--------------------------*/

    /*[h擪AhXԂ*/
    return (void*)load_start;
}


/*------------------------------------------------------
  xjobt@ɃRs[
    start : Ăяo
    data : ѐ
    threshold : ͈͓̔ɊɃxjΎg܂킷
 -----------------------------------------------------*/
void* i_elCopyV4tVeneerToBuffer( ElDesc* elElfDesc, u32 start, u32 data, s32 threshold)
{
    u32*        el_veneer_dest;
    u32         load_start;
    Elf32_Addr  sh_size;
    ElVeneer*   elVenEnt;

    /*--- łɃxj΂g ---*/
    elVenEnt = i_elGetVenEntry( &ELV4tVenEntStart, data);
    if( elVenEnt != NULL) {
        if( i_elIsFar( start, elVenEnt->adr, threshold) == FALSE) {
            PRINTDEBUG( "Veneer Hit!\n");
            return( (void*)elVenEnt->adr);
        }else{
            /*xjĎgȂ̂ŃNXg폜*/
            i_elRemoveVenEntry( &ELV4tVenEntStart, elVenEnt);
        }
    }
    /*--------------------------------------*/

    /*ACgƂ*/
    load_start = i_elALIGN( ((u32)(elElfDesc->buf_current)), 4);
    /*TCYݒ*/
    sh_size = 20;//el_veneer_t

    /*Rs[*/
    OSAPI_CPUCOPY8( el_veneer_t, (u32*)load_start, sh_size);

    /*obt@|C^ړ*/
    elElfDesc->buf_current = (void*)(load_start + sh_size);
    el_veneer_dest = (u32*)load_start;
    el_veneer_dest[4] = data;

    /*--- xjXgɒǉ ---*/
    elVenEnt = OSAPI_MALLOC( sizeof( ElVeneer));
    elVenEnt->adr = load_start;
    elVenEnt->data = data;
    i_elAddVeneerEntry( &ELV4tVenEntStart, elVenEnt);
    /*--------------------------*/

    /*[h擪AhXԂ*/
    return (void*)load_start;
}


/*------------------------------------------------------
  ZOgobt@ɃRs[
 -----------------------------------------------------*/
void* i_elCopySegmentToBuffer( ElDesc* elElfDesc, Elf32_Phdr* Phdr)
{
    u32 load_start;
    
    load_start = i_elALIGN( Phdr->p_vaddr, Phdr->p_align);
    
    elElfDesc->i_elReadStub( (void*)load_start,
                             elElfDesc->FileStruct,
                             (u32)(elElfDesc->ar_head),
                             (u32)(elElfDesc->elf_offset)+(u32)(Phdr->p_offset),
                             Phdr->p_filesz);
    
    return( (void*)load_start);
}

/*------------------------------------------------------
  ZNVobt@ɃRs[
 -----------------------------------------------------*/
void* i_elCopySectionToBuffer( ElDesc* elElfDesc, Elf32_Shdr* Shdr)
{
    u32         load_start;
    Elf32_Addr  sh_size;

    /*ACgƂ*/
    load_start = i_elALIGN( ((u32)(elElfDesc->buf_current)), (Shdr->sh_addralign));
    /*TCYݒ*/
    sh_size = Shdr->sh_size;

    /*Rs[*/
    elElfDesc->i_elReadStub( (void*)load_start,
                             elElfDesc->FileStruct,
                             (u32)(elElfDesc->ar_head),
                             (u32)(elElfDesc->elf_offset)+(u32)(Shdr->sh_offset),
                             sh_size);

    /*obt@|C^ړ*/
    elElfDesc->buf_current = (void*)(load_start + sh_size);

    /*[h擪AhXԂ*/
    return (void*)load_start;
}


/*------------------------------------------------------
  ZNVobt@ɊmہiRs[ȂjA
  ̗̈0Ŗ߂
 -----------------------------------------------------*/
void* i_elAllocSectionToBuffer( ElDesc* elElfDesc, Elf32_Shdr* Shdr)
{
    u32         load_start;
    Elf32_Addr  sh_size;

    /*ACgƂ*/
    load_start = i_elALIGN( ((u32)(elElfDesc->buf_current)), (Shdr->sh_addralign));
    /*TCYݒ*/
    sh_size = Shdr->sh_size;

    /*obt@|C^ړ*/
    elElfDesc->buf_current = (void*)(load_start + sh_size);

    /*0Ŗ߂i.bssZNVz肵Ă邽߁j*/
    OSAPI_CPUFILL8( (void*)load_start, 0, sh_size);
    
    /*mۂ擪AhXԂ*/
    return (void*)load_start;
}


/*------------------------------------------------------
  wCfbNX̃vOwb_obt@Ɏ擾
 -----------------------------------------------------*/
void i_elGetPhdr( ElDesc* elElfDesc, u32 index, Elf32_Phdr* Phdr)
{
    u32 offset;
    
    offset = (elElfDesc->CurrentEhdr.e_phoff) +
             ((u32)(elElfDesc->CurrentEhdr.e_phentsize) * index);
    
    elElfDesc->i_elReadStub( Phdr,
                             elElfDesc->FileStruct,
                             (u32)(elElfDesc->ar_head),
                             (u32)(elElfDesc->elf_offset) + offset,
                             sizeof( Elf32_Shdr));
}

/*------------------------------------------------------
  wCfbNX̃ZNVwb_obt@Ɏ擾
 -----------------------------------------------------*/
void i_elGetShdr( ElDesc* elElfDesc, u32 index, Elf32_Shdr* Shdr)
{
    u32 offset;
    
    offset = (elElfDesc->CurrentEhdr.e_shoff) + ((u32)(elElfDesc->shentsize) * index);
    
    elElfDesc->i_elReadStub( Shdr,
                             elElfDesc->FileStruct,
                             (u32)(elElfDesc->ar_head),
                             (u32)(elElfDesc->elf_offset) + offset,
                             sizeof( Elf32_Shdr));
}

/*------------------------------------------------------
  wCfbNX̃ZNṼGgobt@Ɏ擾
 -----------------------------------------------------*/
void i_elGetSent( ElDesc* elElfDesc, u32 index, void* entry_buf, u32 offset, u32 size)
{
    Elf32_Shdr  Shdr;

    i_elGetShdr( elElfDesc, index, &Shdr);
    
    elElfDesc->i_elReadStub( entry_buf,
                             elElfDesc->FileStruct,
                             (u32)(elElfDesc->ar_head),
                             (u32)(elElfDesc->elf_offset) + (u32)(Shdr.sh_offset) + offset,
                             size);
}


/*------------------------------------------------------
  wZNVwb_̎wCfbNX̃Ggobt@Ɏ擾
    iRel,Rela,SymȂǃGgTCYŒ̃ZNV̂݁j
    Shdr : wZNVwb_
    index : Ggԍ(0`)
 -----------------------------------------------------*/
void i_elGetEntry( ElDesc* elElfDesc, Elf32_Shdr* Shdr, u32 index, void* entry_buf)
{
    u32 offset;

    offset = (u32)(Shdr->sh_offset) + ((Shdr->sh_entsize) * index);
    
    elElfDesc->i_elReadStub( entry_buf,
                             elElfDesc->FileStruct,
                             (u32)(elElfDesc->ar_head),
                             (u32)(elElfDesc->elf_offset) + offset,
                             Shdr->sh_entsize);
}

/*------------------------------------------------------
  STRZNVwb_̎wCfbNX̕擾

    Shdr : wZNVwb_
    index : GgCfbNX(0`)
 -----------------------------------------------------*/
void i_elGetStrAdr( ElDesc* elElfDesc, u32 strsh_index, u32 ent_index, char* str, u32 len)
{
    /*Gg̐擪AhX*/
    i_elGetSent( elElfDesc, strsh_index, str, ent_index, len);
}

/*------------------------------------------------------
  V{Ē`

    <RelZNVwb_>
    RelShdr->sh_link : V{ZNVԍ
    RelShdr->sh_info : ^[QbgZNVԍ(Frel.text.text\)
    
    <RelGg>
    Rel->r_offset : ^[QbgZNṼItZbgAhX
    ELF32_R_SYM( Rel->r_info) : V{Ggԍ
    ELF32_R_TYPE( Rel->r_info) : P[g^Cv

    <SymZNVwb_>
    SymShdr->sh_link : V{̕e[uZNVԍ
    
    <SymGg>
    Sym->st_name : V{̕Ggԍ
    Sym->st_value : V{̏ZNṼItZbgAhX
    Sym->st_size : V{̏ZNVł̃TCY
    Sym->st_shndx : V{̏ZNV̔ԍ
    ELF32_ST_BIND( Sym->st_info) : oCh(LOCAL or GLOBAL)
    ELF32_ST_TYPE( Sym->st_info) : ^Cv(V{֘AtĂΏ)
 -----------------------------------------------------*/
void i_elRelocateSym( ElDesc* elElfDesc, u32 relsh_index)
{
    u32                 i;
    u32                 num_of_rel;     //Ē`ׂV{̐
    Elf32_Shdr          RelOrRelaShdr;  //REL܂RELAZNVwb_
    Elf32_Rela          CurrentRela;    //REL܂RELAGg̃Rs[
    Elf32_Shdr*         SymShdr;
    ElSymEx*            CurrentSymEx;
    ElShdrEx*           TargetShdrEx;
    ElShdrEx*           CurrentShdrEx;
    u32                 relocation_adr;
    char                sym_str[128];
    u32                 copy_size;
    ElAdrEntry*         CurrentAdrEntry;
    u32                 sym_loaded_adr = 0;
    u32                 thumb_func_flag = 0;
    ElUnresolvedEntry   UnresolvedInfo;
    ElUnresolvedEntry*  UnrEnt;
    u32                 unresolved_num = 0;

    /*REL or RELAZNVwb_擾*/
    i_elGetShdr( elElfDesc, relsh_index, &RelOrRelaShdr);

    /*^[QbgZNVEXwb_*/
    TargetShdrEx = i_elGetShdrExfromList( elElfDesc->ShdrEx, RelOrRelaShdr.sh_info);
    
    num_of_rel = (RelOrRelaShdr.sh_size) / (RelOrRelaShdr.sh_entsize);    //Ē`ׂV{̐

    /*V{Xg쐬*/
    i_elBuildSymList( elElfDesc, RelOrRelaShdr.sh_link);
    /*i_elBuildSymListĂԂelElfDesc->SymShdrZbg*/
    SymShdr = &(elElfDesc->SymShdr);
    PRINTDEBUG( "SymShdr->link:%02x, SymShdr->info:%02x\n", SymShdr->sh_link, SymShdr->sh_info);


    /*--- Ē`KvȃV{̍Ē` ---*/
    for( i=0; i<num_of_rel; i++) {
        
        /*- Rel܂RelaGg擾 -*/
        i_elGetEntry( elElfDesc, &RelOrRelaShdr, i, &CurrentRela);
        
        if( RelOrRelaShdr.sh_type == SHT_REL) {
            CurrentRela.r_addend = 0;
        }
        /*-----------------------------*/

        /*V{ExGg擾(i_elGetSymExfromTblł悢)*/
        CurrentSymEx = (ElSymEx*)(elElfDesc->SymExTbl[ELF32_R_SYM( CurrentRela.r_info)]);

//        if( CurrentSymEx->debug_flag == 1) {            /*fobȌꍇ*/
        if( CurrentSymEx == NULL) {
        }else{                                            /*fobOłȂꍇ*/
            /**/
            i_elUnresolvedInfoInit( &UnresolvedInfo);
            /*AhXidlłuPvj*/
            relocation_adr = (TargetShdrEx->loaded_adr) + (CurrentRela.r_offset);
            UnresolvedInfo.r_type = ELF32_R_TYPE( CurrentRela.r_info);
            UnresolvedInfo.A_ = (CurrentRela.r_addend);
            UnresolvedInfo.P_ = (relocation_adr);
            UnresolvedInfo.sh_type = (RelOrRelaShdr.sh_type);
            
            /*V{̃AhX˂~߂*/
            if( CurrentSymEx->Sym.st_shndx == SHN_UNDEF) {
                /*AhXe[u猟*/
                i_elGetStrAdr( elElfDesc, SymShdr->sh_link, CurrentSymEx->Sym.st_name, sym_str, 128);
                CurrentAdrEntry = elGetAdrEntry( sym_str);
                if( CurrentAdrEntry) {
                    sym_loaded_adr = (u32)(CurrentAdrEntry->adr);
                    /*THUMB֐tOidlłuTvjTHUMB or ARM ̔*/
                    thumb_func_flag = CurrentAdrEntry->thumb_flag;
                    PRINTDEBUG( "\n symbol found %s : %8x\n", sym_str, sym_loaded_adr);
                }else{
                    /*Ȃ̓G[(S_T_łȂ)*/
                    copy_size = (u32)OSAPI_STRLEN( sym_str) + 1;
                    UnresolvedInfo.sym_str = OSAPI_MALLOC( copy_size);
                    OSAPI_CPUCOPY8( sym_str, UnresolvedInfo.sym_str, copy_size);

                    /*O[oȖe[uɒǉ*/
                    copy_size = sizeof( ElUnresolvedEntry);
                    UnrEnt = OSAPI_MALLOC( copy_size);
                    OSAPI_CPUCOPY8( &UnresolvedInfo, UnrEnt, copy_size);
                    i_elAddUnresolvedEntry( UnrEnt);

                    unresolved_num++;    /*V{JEg*/
                    PRINTDEBUG( "\n WARNING! cannot find symbol : %s\n", sym_str);
                }
            }else{
                /*V{̏ZNVExwb_擾*/
                CurrentShdrEx = i_elGetShdrExfromList( elElfDesc->ShdrEx, CurrentSymEx->Sym.st_shndx);
                sym_loaded_adr = CurrentShdrEx->loaded_adr;
                sym_loaded_adr += CurrentSymEx->Sym.st_value;    //sym_loaded_adr͎dlłuSv
                /*THUMB֐tOidlłuTvjTHUMB or ARM ̔*/
                thumb_func_flag = CurrentSymEx->thumb_flag;
            }

            if( !UnresolvedInfo.sym_str) {        /*sym_strZbgĂƂ̓V{s\*/
                /*dlƓϐɂ*/
                UnresolvedInfo.S_ = (sym_loaded_adr);
                UnresolvedInfo.T_ = (thumb_func_flag);

                /*--------------- V{̉(Ĕzu̎s) ---------------*/
                /*CurrentSymEx->relocation_val =*/ i_elDoRelocate( elElfDesc,
                                                                   &UnresolvedInfo);
                /*------------------------------------------------------------*/
            }
        }
    }
    /*-----------------------------------*/


    /*V{XgJ*/
//    i_elFreeSymList( elElfDesc);

    
    /* Ĕzu */
    if( unresolved_num == 0) {
        elElfDesc->process = EL_RELOCATED;
    }
}


/*------------------------------------------------------
  O[oV{AhXe[uɓ

    <SymZNVwb_>
    SymShdr->sh_link : V{̕e[uZNVԍ
    
    <SymGg>
    Sym->st_name : V{̕Ggԍ
    Sym->st_value : V{̏ZNṼItZbgAhX
    Sym->st_size : V{̏ZNVł̃TCY
    Sym->st_shndx : V{̏ZNV̔ԍ
    ELF32_ST_BIND( Sym->st_info) : oCh(LOCAL or GLOBAL)
    ELF32_ST_TYPE( Sym->st_info) : ^Cv(V{֘AtĂΏ)
 -----------------------------------------------------*/
void i_elGoPublicGlobalSym( ElDesc* elElfDesc, u32 symtblsh_index)
{
    Elf32_Shdr   SymShdr;
    ElSymEx*     CurrentSymEx;
    ElShdrEx*    CurrentShdrEx;
    ElAdrEntry*  ExportAdrEntry;
    char         sym_str[128];
    u32          copy_size;
    
    /*SYMTABZNVwb_擾*/
    i_elGetShdr( elElfDesc, symtblsh_index, &SymShdr);
    
    /*V{Xg쐬*/
    i_elBuildSymList( elElfDesc, symtblsh_index);
    
    /*--- CuGLOBALV{AhXe[uɌJ ---*/
    CurrentSymEx = elElfDesc->SymEx; //TblłȂA炽ǂdebug͊܂܂Ȃ
    while( CurrentSymEx != NULL) {
//        CurrentSymEx = i_elGetSymExfromList( elElfDesc->SymExTbl, i);
//        if( CurrentSymEx != NULL) {
        /*GLOBALŁA֘AZNVCuɑ݂ꍇ*/
        if( (ELF32_ST_BIND( CurrentSymEx->Sym.st_info) == STB_GLOBAL) &&
            (CurrentSymEx->Sym.st_shndx != SHN_UNDEF)) {
            
            ExportAdrEntry = OSAPI_MALLOC( sizeof(ElAdrEntry));    /*m*/
            
            ExportAdrEntry->next = NULL;
            
            i_elGetStrAdr( elElfDesc, SymShdr.sh_link, CurrentSymEx->Sym.st_name, sym_str, 128);
            copy_size = (u32)OSAPI_STRLEN( sym_str) + 1;
            ExportAdrEntry->name = OSAPI_MALLOC( copy_size);
            OSAPI_CPUCOPY8( sym_str, ExportAdrEntry->name, copy_size);

            CurrentShdrEx = i_elGetShdrExfromList( elElfDesc->ShdrEx, CurrentSymEx->Sym.st_shndx);
            //Sym.st_value͋/ARM/Thumb𔻕ʂł悤ɒĂꍇ̂ŁA̒폜Đ̒lo
            ExportAdrEntry->adr = (void*)(CurrentShdrEx->loaded_adr + ((CurrentSymEx->Sym.st_value)&0xFFFFFFFE));
            ExportAdrEntry->func_flag = (u16)(ELF32_ST_TYPE( CurrentSymEx->Sym.st_info));
            ExportAdrEntry->thumb_flag = CurrentSymEx->thumb_flag;

            if( elGetAdrEntry( ExportAdrEntry->name) == NULL) {    //ĂȂ
                elAddAdrEntry( ExportAdrEntry);    /*o^*/
            }
        }
        CurrentSymEx = CurrentSymEx->next;
//        }
    }
    /*----------------------------------------------------------------*/

    /*V{XgJ*/
//    i_elFreeSymList( elElfDesc);
}


/*------------------------------------------------------
  V{Xg쐬

 -----------------------------------------------------*/
static void i_elBuildSymList( ElDesc* elElfDesc, u32 symsh_index)
{
    u32         i;
    u32         num_of_sym;        //V{̑S̐
    u16         debug_flag;
    Elf32_Sym   TestSym;
    ElSymEx*    CurrentSymEx;
    ElShdrEx*   CurrentShdrEx;
    Elf32_Shdr* SymShdr;
    ElSymEx     DmySymEx;

    if( elElfDesc->SymExTarget == symsh_index) {
        PRINTDEBUG( "%s skip.\n", __FUNCTION__);
        return;                              //ɃXg쐬ς
    }else{
        i_elFreeSymList( elElfDesc); /*V{XgJ*/
    }
    
    PRINTDEBUG( "%s build\n", __FUNCTION__);
    
    /*SYMTABZNVwb_擾*/
    i_elGetShdr( elElfDesc, symsh_index, &(elElfDesc->SymShdr));
    SymShdr = &(elElfDesc->SymShdr);
    
    num_of_sym = (SymShdr->sh_size) / (SymShdr->sh_entsize);    //V{̑S̐
    elElfDesc->SymExTbl = OSAPI_MALLOC( num_of_sym * 4);
    
    /*---------- ElSymEx̃Xg ----------*/
    CurrentSymEx = &DmySymEx;
    for( i=0; i<num_of_sym; i++) {
        
        /*V{GgRs[*/
        i_elGetEntry( elElfDesc, (Elf32_Shdr*)SymShdr, i, &TestSym);
        /*-- fobOtOZbg --*/
        CurrentShdrEx = i_elGetShdrExfromList( elElfDesc->ShdrEx, TestSym.st_shndx);
        if( CurrentShdrEx) {
            debug_flag = CurrentShdrEx->debug_flag;
        }else{
            debug_flag = 0;
        }/*-------------------------------*/

        if( debug_flag == 1) {
            elElfDesc->SymExTbl[i] = NULL;
        }else{
            CurrentSymEx->next = OSAPI_MALLOC( sizeof(ElSymEx));
            CurrentSymEx = (ElSymEx*)(CurrentSymEx->next);
            
            OSAPI_CPUCOPY8( &TestSym, &(CurrentSymEx->Sym), sizeof(TestSym));
            
            elElfDesc->SymExTbl[i] = CurrentSymEx;
            
            PRINTDEBUG( "sym_no: %02x ... st_shndx: %04x\n", i, CurrentSymEx->Sym.st_shndx);
        }
    }
    
    CurrentSymEx->next = NULL;
    elElfDesc->SymEx = DmySymEx.next;
    /*-------------------------------------------*/
    

    /*-------- ARM or Thumb (i_elCodeIsThumbSymExoĂȂƓȂ) --------*/
    CurrentSymEx = elElfDesc->SymEx;
    while( CurrentSymEx != NULL) { //炽ǂdebug͖
        /*-- ElSymExThumbtOZbgi֐V{Kvj--*/
        if( ELF32_ST_TYPE( CurrentSymEx->Sym.st_info) == STT_FUNC) {
            CurrentSymEx->thumb_flag = (u16)(i_elCodeIsThumb( elElfDesc,
                                                              CurrentSymEx->Sym.st_shndx,
                                                              CurrentSymEx->Sym.st_value));
        }else{
            CurrentSymEx->thumb_flag = 0;
        }/*--------------------------------------------------------*/
        
        CurrentSymEx = CurrentSymEx->next;
    }
    /*-------------------------------------------------------------------------------------*/


    elElfDesc->SymExTarget = symsh_index;
}


/*------------------------------------------------------
  V{XgJ

 -----------------------------------------------------*/
void i_elFreeSymList( ElDesc* elElfDesc)
{
    ElSymEx*  CurrentSymEx;
    ElSymEx*  FwdSymEx;

    if( elElfDesc->SymExTbl != NULL) {
        OSAPI_FREE( elElfDesc->SymExTbl);
        elElfDesc->SymExTbl = NULL;
    }
    
    /*------- ElSymEx̃Xg -------*/
    CurrentSymEx = elElfDesc->SymEx;
    if( CurrentSymEx) {
        while( CurrentSymEx->next != NULL) {
            FwdSymEx = CurrentSymEx;
            CurrentSymEx = CurrentSymEx->next;
            OSAPI_FREE( FwdSymEx);
        }
        elElfDesc->SymEx = NULL;
    }
    /*-----------------------------------------*/


    elElfDesc->SymExTarget = 0xFFFFFFFF;
}


/*------------------------------------------------------
  ƂɃV{

    r_type,S_,A_,P_,T_SĕĂKvB
 -----------------------------------------------------*/
#define _S_    (UnresolvedInfo->S_)
#define _A_    (UnresolvedInfo->A_)
#define _P_    (UnresolvedInfo->P_)
#define _T_    (UnresolvedInfo->T_)
BOOL    i_elDoRelocate( ElDesc* elElfDesc, ElUnresolvedEntry* UnresolvedInfo)
{
    s32    signed_val;
    u32    relocation_val = 0;
    u32    relocation_adr;
    BOOL   ret_val;
    BOOL   veneer_flag = TRUE;

    ret_val = TRUE;
    relocation_adr = _P_;

    switch( (UnresolvedInfo->r_type)) {
      case R_ARM_PC24:
      case R_ARM_PLT32:
      case R_ARM_CALL:
      case R_ARM_JUMP24:
        if( UnresolvedInfo->sh_type == SHT_REL) {
            _A_ = (s32)(((*(vu32*)relocation_adr)|0xFF800000) << 2);    //ʓIɂ-8ɂȂĂ͂
        }
        /*----------------- veneer -----------------*/
        veneer_flag = i_elIsFar( _P_, _S_|_T_, 0x2000000); //+-32MBytes?
#if (TARGET_ARM_V5 == 1)
        if( veneer_flag == TRUE) {
#else //(TARGET_ARM_V4 == 1)
        //ARM->ThumbARM->ARMŃOu`̂Ƃ veneer gp
        if( (_T_) || (veneer_flag == TRUE)) {
#endif
            //_A_PCΕ̂Ƃ̒lȂ̂ŁAΕ̏ꍇ͊֌WȂ
            _S_ = (u32)i_elCopyVeneerToBuffer( elElfDesc, _P_,
                                               (u32)(( (s32)(_S_) /*+ _A_*/) | (s32)(_T_)),
                                               0x2000000);
            _T_ = 0; //veneerARMR[ĥ
        }/*-----------------------------------------*/
        signed_val = (( (s32)(_S_) + _A_) | (s32)(_T_)) - (s32)(_P_);
        if( _T_) {        /*BLX߂ARMThumbɔ(v5BLxjABXƂdg݂Kv)*/
            relocation_val = (0xFA000000) | ((signed_val>>2) & 0x00FFFFFF) | (((signed_val>>1) & 0x1)<<24);
        }else{            /*BL߂ARMARMɔ*/
            signed_val >>= 2;
            relocation_val = (*(vu32*)relocation_adr & 0xFF000000) | (signed_val & 0x00FFFFFF);
        }
        *(vu32*)relocation_adr = relocation_val;
        break;
      case R_ARM_ABS32:
        relocation_val = (( _S_ + _A_) | _T_);
        *(vu32*)relocation_adr = relocation_val;
        break;
      case R_ARM_REL32:
      case R_ARM_PREL31:
        relocation_val = (( _S_ + _A_) | _T_) - _P_;
        *(vu32*)relocation_adr = relocation_val;
        break;
      case R_ARM_LDR_PC_G0:
        /*----------------- veneer -----------------*/
        veneer_flag = i_elIsFar( _P_, _S_|_T_, 0x2000000); //+-32MBytes?
#if (TARGET_ARM_V5 == 1)
        if( veneer_flag == TRUE) {
#else //(TARGET_ARM_V4 == 1)
        //ARM->ThumbARM->ARMŃOu`̂Ƃ veneer gp
        if( (_T_) || (veneer_flag == TRUE)) {
#endif
            //_A_PCΕ̂Ƃ̒lȂ̂ŁAΕ̏ꍇ͊֌WȂ
            _S_ = (u32)i_elCopyVeneerToBuffer( elElfDesc, _P_,
                                               (u32)(( (s32)(_S_) /*+ _A_*/) | (s32)(_T_)),
                                               0x2000000);
            _T_ = 0; //veneerARMR[ĥ
        }/*-----------------------------------------*/
        signed_val = ( (s32)(_S_) + _A_) - (s32)(_P_);
        signed_val >>= 2;
        relocation_val = (*(vu32*)relocation_adr & 0xFF000000) | (signed_val & 0x00FFFFFF);
        *(vu32*)relocation_adr = relocation_val;
        break;
      case R_ARM_ABS16:
      case R_ARM_ABS12:
      case R_ARM_THM_ABS5:
      case R_ARM_ABS8:
        relocation_val = _S_ + _A_;
        *(vu32*)relocation_adr = relocation_val;
        break;
      case R_ARM_THM_PC22:/*ʖFR_ARM_THM_CALL*/
        if( UnresolvedInfo->sh_type == SHT_REL) {
            _A_ = (s32)(((*(vu16*)relocation_adr & 0x07FF)<<11) + ((*((vu16*)(relocation_adr)+1)) & 0x07FF));
            _A_ = (s32)((_A_ | 0xFFC00000) << 1);    //ʓIɂ-4ɂȂĂ͂(PC͌߃AhX+4Ȃ̂)
        }
        /*----------------- veneer -----------------*/
        veneer_flag = i_elIsFar( _P_, _S_|_T_, 0x400000); //+-4MBytes?
#if (TARGET_ARM_V5 == 1)
        if( veneer_flag == TRUE) {
            //_A_PCΕ̂Ƃ̒lȂ̂ŁAΕ̏ꍇ͊֌WȂ
            _S_ = (u32)i_elCopyVeneerToBuffer( elElfDesc, _P_,
                                               (u32)(( (s32)(_S_) /*+ _A_*/) | (s32)(_T_)),
                                               0x400000);
            _T_ = 0; //veneerARMR[ĥ
        }/*-----------------------------------------*/
#else //(TARGET_ARM_V4 == 1)
        /*----------------- v4t veneer -----------------*/
        //Thumb->ARMThumb->ThumbŃOu`̂Ƃ v4T veneer gp
        if( (_T_ == 0) || (veneer_flag == TRUE)) {
            _S_ = (u32)i_elCopyV4tVeneerToBuffer( elElfDesc, _P_,
                                                  (u32)(( (s32)(_S_)) | (s32)(_T_)),
                                                  0x400000);
            _T_ = 1; //v4t veneerThumbR[ĥ
        }
        /*---------------------------------------------*/
#endif
        signed_val = (( (s32)(_S_) + _A_) | (s32)(_T_)) - (s32)(_P_);
        signed_val >>= 1;
        if( _T_) {    /*BL߂ThumbThumbɔ*/
            relocation_val = (u32)(*(vu16*)relocation_adr & 0xF800) | ((signed_val>>11) & 0x07FF) +
                                   ((((*((vu16*)(relocation_adr)+1)) & 0xF800) | (signed_val & 0x07FF)) << 16);
        }else{        /*BLX߂ThumbARMɔ(v5BLxjABXƂdg݂Kv)*/
            if( (signed_val & 0x1)) {    //_P_4oCgACĂȂƂɗ
                signed_val += 1;
            }
            relocation_val = (u32)(*(vu16*)relocation_adr & 0xF800) | ((signed_val>>11) & 0x07FF) +
                                   ((((*((vu16*)(relocation_adr)+1)) & 0xE800) | (signed_val & 0x07FF)) << 16);
        }
        *(vu16*)relocation_adr = (vu16)relocation_val;
        *((vu16*)relocation_adr+1) = (vu16)((u32)(relocation_val) >> 16);
        break;
      case R_ARM_NONE:
        /*R_ARM_NONE̓^[Qbg̃V{ێAJɃfbhXgbv
          Ȃ悤ɂ܂*/
        break;
      case R_ARM_THM_JUMP24:
      default:
        ret_val = FALSE;
        PRINTDEBUG( "ERROR! : unsupported relocation type (0x%x)!\n", (UnresolvedInfo->r_type));
        PRINTDEBUG( "S = 0x%x\n", _S_);
        PRINTDEBUG( "A = 0x%x\n", _A_);
        PRINTDEBUG( "P = 0x%x\n", _P_);
        PRINTDEBUG( "T = 0x%x\n", _T_);
        break;
    }
    
    return ret_val;//relocation_val;
}
#undef _S_
#undef _A_
#undef _P_
#undef _T_

/*------------------------------------------------------
  XgwCfbNXElSymExo
 -----------------------------------------------------*/
static ElSymEx* i_elGetSymExfromList( ElSymEx* SymExStart, u32 index)
{
    u32         i;
    ElSymEx*    SymEx;

    SymEx = SymExStart;
    for( i=0; i<index; i++) {
        SymEx = (ElSymEx*)(SymEx->next);
        if( SymEx == NULL) {
            break;
        }
    }
    return SymEx;
}

/*------------------------------------------------------
  e[uwCfbNXElSymExo
 -----------------------------------------------------*/
static ElSymEx* i_elGetSymExfromTbl( ElSymEx** SymExTbl, u32 index)
{
    return( (ElSymEx*)(SymExTbl[index]));
}

/*------------------------------------------------------
  XgwCfbNXElShdrExo
 -----------------------------------------------------*/
ElShdrEx* i_elGetShdrExfromList( ElShdrEx* ShdrExStart, u32 index)
{
    u32         i;
    ElShdrEx*   ShdrEx;

    ShdrEx = ShdrExStart;
    for( i=0; i<index; i++) {
        ShdrEx = (ElShdrEx*)(ShdrEx->next);
        if( ShdrEx == NULL) {
            break;
        }
    }
    return ShdrEx;
}



/*------------------------------------------------------
  wCfbNX̃ZNVfobO񂩂ǂ肷

fobO̒`
EZNV".debug"n܂ZNV

E.rel.debug` ZNVȂǁAsh_info fobOZNVԍ
@ĂZNV
 -----------------------------------------------------*/
BOOL i_elShdrIsDebug( ElDesc* elElfDesc, u32 index)
{
    Elf32_Shdr  TmpShdr;
    char        shstr[6];

    /*-- ZNV̕6擾 --*/
    i_elGetShdr( elElfDesc, index, &TmpShdr);
    i_elGetStrAdr( elElfDesc, elElfDesc->CurrentEhdr.e_shstrndx,
                   TmpShdr.sh_name, shstr, 6);
    /*-------------------------------------*/
    
    if( OSAPI_STRNCMP( shstr, ".debug", 6) == 0) {    /*fobOZNV̏ꍇ*/
        return TRUE;
    }else{                        /*fobOZNVɊւĔzuZNV̏ꍇ*/
        if( (TmpShdr.sh_type == SHT_REL) || (TmpShdr.sh_type == SHT_RELA)) {
            if( i_elShdrIsDebug( elElfDesc, TmpShdr.sh_info) == TRUE) {
                return TRUE;
            }
        }
    }

    return FALSE;
}



/*------------------------------------------------------
  elElfDescSymExe[u𒲂ׁAwCfbNX
@wItZbgɂR[hARMTHUMB𔻒肷B
  i\ elElfDesc->SymShdr  elElfDesc->SymEx ݒ肵ĂƁj
    
    sh_index : ׂZNVCfbNX
    offset : ׂZNṼItZbg
 -----------------------------------------------------*/
u32 i_elCodeIsThumb( ElDesc* elElfDesc, u16 sh_index, u32 offset)
{
    u32            i;
    u32            thumb_flag;
    Elf32_Shdr*    SymShdr;
    char           str_adr[3];
    ElSymEx*       CurrentSymEx;

    /*V{̃ZNVwb_SymExXg擾*/
    SymShdr = &(elElfDesc->SymShdr);
    CurrentSymEx = elElfDesc->SymEx;

    i = 0;
    thumb_flag = 0;
    while( CurrentSymEx != NULL) {
        
        if( CurrentSymEx->Sym.st_shndx == sh_index) {
            i_elGetStrAdr( elElfDesc, SymShdr->sh_link, CurrentSymEx->Sym.st_name, str_adr, 3);
            if( OSAPI_STRNCMP( str_adr, "$a\0", OSAPI_STRLEN("$a\0")) == 0) {
                thumb_flag = 0;
            }else if( OSAPI_STRNCMP( str_adr, "$t\0", OSAPI_STRLEN("$t\0")) == 0) {
                thumb_flag = 1;
            }
            if( CurrentSymEx->Sym.st_value > offset) {
                break;
            }
        }
        
        CurrentSymEx = CurrentSymEx->next;
        i++;
    }

    return thumb_flag;
}


/*---------------------------------------------------------
 Gg
 --------------------------------------------------------*/
static void i_elUnresolvedInfoInit( ElUnresolvedEntry* UnresolvedInfo)
{
    UnresolvedInfo->sym_str = NULL;
    UnresolvedInfo->r_type = 0;
    UnresolvedInfo->S_ = 0;
    UnresolvedInfo->A_ = 0;
    UnresolvedInfo->P_ = 0;
    UnresolvedInfo->T_ = 0;
    UnresolvedInfo->remove_flag = 0;
}

/*---------------------------------------------------------
 e[uɃGgǉ
 --------------------------------------------------------*/
static void i_elAddUnresolvedEntry( ElUnresolvedEntry* UnrEnt)
{
    ElUnresolvedEntry    DmyUnrEnt;
    ElUnresolvedEntry*   CurrentUnrEnt;

    if( !ELUnrEntStart) {
        ELUnrEntStart = UnrEnt;
    }else{
        DmyUnrEnt.next = ELUnrEntStart;
        CurrentUnrEnt = &DmyUnrEnt;

        while( CurrentUnrEnt->next != NULL) {
            CurrentUnrEnt = CurrentUnrEnt->next;
        }
        CurrentUnrEnt->next = (void*)UnrEnt;
    }
    UnrEnt->next = NULL;
}


/*---------------------------------------------------------
 xje[uɃGgǉ
 --------------------------------------------------------*/
static void i_elAddVeneerEntry( ElVeneer** start, ElVeneer* VenEnt)
{
    ElVeneer    DmyVenEnt;
    ElVeneer*   CurrentVenEnt;

    if( !(*start)) {
        (*start) = VenEnt;
    }else{
        DmyVenEnt.next = (*start);
        CurrentVenEnt = &DmyVenEnt;

        while( CurrentVenEnt->next != NULL) {
            CurrentVenEnt = CurrentVenEnt->next;
        }
        CurrentVenEnt->next = (void*)VenEnt;
    }
    VenEnt->next = NULL;
}

/*------------------------------------------------------
  xje[uGg폜
 -----------------------------------------------------*/
static BOOL i_elRemoveVenEntry( ElVeneer** start, ElVeneer* VenEnt)
{
    ElVeneer  DmyVenEnt;
    ElVeneer* CurrentVenEnt;

    DmyVenEnt.next = (*start);
    CurrentVenEnt = &DmyVenEnt;

    while( CurrentVenEnt->next != VenEnt) {
        if( CurrentVenEnt->next == NULL) {
            return FALSE;
        }else{
            CurrentVenEnt = (ElVeneer*)CurrentVenEnt->next;
        }
    }

    /*NXǧq*/
    CurrentVenEnt->next = VenEnt->next;
    (*start) = DmyVenEnt.next;

    /*J*/
    OSAPI_FREE( VenEnt);
    
     return TRUE;
}

/*------------------------------------------------------
  xje[uwf[^ɊYGgT
 -----------------------------------------------------*/
static ElVeneer* i_elGetVenEntry( ElVeneer** start, u32 data)
{
    ElVeneer* CurrentVenEnt;

    CurrentVenEnt = (*start);
    if( CurrentVenEnt == NULL) {
        return NULL;
    }
    while( CurrentVenEnt->data != data) {
        CurrentVenEnt = (ElVeneer*)CurrentVenEnt->next;
        if( CurrentVenEnt == NULL) {
            break;
        }
    }
    return CurrentVenEnt;
}

/*------------------------------------------------------
  xje[u
 -----------------------------------------------------*/
void* i_elFreeVenTbl( void)
{
    ElVeneer*    FwdVenEnt;
    ElVeneer*    CurrentVenEnt;
    
    /*--- ElVenEntrỹXg ---*/
    CurrentVenEnt = ELVenEntStart;
    while( CurrentVenEnt != NULL) {
        FwdVenEnt = CurrentVenEnt;
        CurrentVenEnt = CurrentVenEnt->next;
        OSAPI_FREE( FwdVenEnt);              //\̎g
    }
    ELVenEntStart = NULL;
    /*------------------------------------*/

    /*--- ElVenEntrỹXg ---*/
    CurrentVenEnt = ELV4tVenEntStart;
    while( CurrentVenEnt != NULL) {
        FwdVenEnt = CurrentVenEnt;
        CurrentVenEnt = CurrentVenEnt->next;
        OSAPI_FREE( FwdVenEnt);              //\̎g
    }
    ELV4tVenEntStart = NULL;
    /*------------------------------------*/
    
    return NULL;
}

/*------------------------------------------------------
  2̃AhXl̍ (+threshold)`(-threshold)
  ܂Ă邩𒲂ׂ
 -----------------------------------------------------*/
static BOOL i_elIsFar( u32 P, u32 S, s32 threshold)
{
    s32 diff;

    diff = (s32)S - (s32)(P);
    if( (diff < threshold)&&( diff > -threshold)) {
        return( FALSE); //Near
    }else{
        return( TRUE);  //Far
    }
}

