/*---------------------------------------------------------------------------*
  Project:  TwlSDK - library - camera
  File:     camera.c

  Copyright 2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Log: $
  $NoKeywords: $
 *---------------------------------------------------------------------------*/
#include <twl.h>
#include <twl/mic.h>

/*---------------------------------------------------------------------------*
    萔`
 *---------------------------------------------------------------------------*/
// l߂ăRs[
#define MIC_PACK_U16(d, s)                   \
    ((d)[0] = (u8)((*((u16*)s) >>  0) & 0xFF),  \
     (d)[1] = (u8)((*((u16*)s) >>  8) & 0xFF))

#define MIC_PACK_U32(d, s)                   \
    ((d)[0] = (u8)((*((u32*)s) >>  0) & 0xFF),  \
     (d)[1] = (u8)((*((u32*)s) >>  8) & 0xFF),  \
     (d)[2] = (u8)((*((u32*)s) >> 16) & 0xFF),  \
     (d)[3] = (u8)((*((u32*)s) >> 24) & 0xFF))

/*---------------------------------------------------------------------------*
    ^`
 *---------------------------------------------------------------------------*/
typedef struct _TWLMICWork
{
    BOOL            lock;
    TwlMicCallback  callback;					// 񓯊֐pR[obN
    void*			callbackArg;				// L֐p
    TwlMicCallback  full_callback;				// 񓯊֐pR[obN
    void*			full_arg;					// L֐p
    TWLMICResult    result;             		// 擪f[^ʘg
    TWLMICPxiCommand   command;        			// R}h
    TWLMICPxiResult    pxiResult;      			// 擪f[^ʘg
	u16*               pOneBuffer;				// PTvOobt@ۑp
	void**             pLastSamplingAddress;	// ŐVTvOf[^i[AhX
	u8*                pAmpGain;				// AvQCi[AhX
    u8      current;                    		// Mς݃f[^ (oCgP) (擪!!)
    u8      total;                      		// ŏIf[^ (1 + 㑱R}h*3)
    u8      data[TWL_MIC_PXI_DATA_SIZE_MAX];    // ARM7̃f[^ۑp
}
TWLMICWork;

/*---------------------------------------------------------------------------*
    ÓIϐ`
 *---------------------------------------------------------------------------*/
static BOOL micInitialized;
static TWLMICWork micWork;

/*---------------------------------------------------------------------------*
    ֐`
 *---------------------------------------------------------------------------*/
static BOOL TWLMICi_SendPxiCommand(TWLMICPxiCommand command, u8 size, u8 data);
static void TWLMICi_SendPxiData(u8 *pData);
static void TWLMICi_PxiCallback(PXIFifoTag tag, u32 data, BOOL err);
static void TWLMICi_CallbackAndUnlock(TWLMICResult result);
static void TWLMICi_GetResultCallback(TWLMICResult result, void *arg);
static void TWLMICi_WaitBusy(void);

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_Init

  Description:  MICCuB

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void TWL_MIC_Init(void)
{
    // ς݂mF
    if (micInitialized)
    {
        return;
    }
    micInitialized = 1;

    // ϐ
    micWork.lock = FALSE;
    micWork.callback = NULL;

    // PXI֘A
    PXI_Init();
    while (!PXI_IsCallbackReady(PXI_FIFO_TAG_TWL_MIC, PXI_PROC_ARM7))
    {
    }
    PXI_SetFifoRecvCallback(PXI_FIFO_TAG_TWL_MIC, TWLMICi_PxiCallback);
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_End

  Description:  MICCuIB

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void TWL_MIC_End(void)
{
    // ς݂mF
    if (micInitialized == 0)
    {
        return;
    }
    micInitialized = 0;

    // PXI֘A~
    PXI_SetFifoRecvCallback(PXI_FIFO_TAG_TWL_MIC, NULL);
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_DoSamplingAsync

  Description:  }CNPTvOJni񓯊Łj

  Arguments:    

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_DoSamplingAsync(u16* buf,  TwlMicCallback callback, void* callbackArg)
{
    const TWLMICPxiCommand  command = TWL_MIC_PXI_COMMAND_ONE_SAMPLING;
    const u8                size = TWL_MIC_PXI_SIZE_ONE_SAMPLING;	// oCg
    OSIntrMode enabled;

	// buffer NULL`FbN
	// ToDo: K؂ȃAhXǂ`FbN
	if (buf == NULL)
	{
        return TWL_MIC_RESULT_ILLEGAL_PARAMETER;
	}

	// bN
    enabled = OS_DisableInterrupts();
    if (micWork.lock)
    {
        (void)OS_RestoreInterrupts(enabled);
        return TWL_MIC_RESULT_BUSY;
    }
    micWork.lock = TRUE;
    (void)OS_RestoreInterrupts(enabled);

    // 񓯊֐pR[obNݒ
    micWork.callback = callback;
    micWork.callbackArg = callbackArg;

	// obt@AhXۑ
	micWork.pOneBuffer = buf;

    // R}hM
    if (TWLMICi_SendPxiCommand(command, size, 0) == FALSE)
    {
        return TWL_MIC_RESULT_SEND_ERROR;
    }

    return TWL_MIC_RESULT_SUCCESS;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_DoSampling

  Description:  }CNPTvOJniŁj

  Arguments:    

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_DoSampling(u16* buf)
{
    micWork.result = TWL_MIC_DoSamplingAsync(buf, TWLMICi_GetResultCallback, 0);
    if (micWork.result == TWL_MIC_RESULT_SUCCESS)
    {
        TWLMICi_WaitBusy();
    }
    return micWork.result;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_StartAutoSamplingAsync

  Description:  }CNTvOJni񓯊Łj

  Arguments:    mic      - one of MicSelect

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_StartAutoSamplingAsync(TwlMicAutoParam* param, TwlMicCallback callback, void* callbackArg)
{
    const TWLMICPxiCommand  command = TWL_MIC_PXI_COMMAND_AUTO_START;
    const u8                size = TWL_MIC_PXI_SIZE_AUTO_START;	// oCg
    OSIntrMode enabled;
    u8  data[size+2];
    int i;

	// DMA-No`FbN
    if ( param->dmaNo < MI_EXDMA_CH_MIN || MI_EXDMA_CH_MAX < param->dmaNo )
	{
        return TWL_MIC_RESULT_ILLEGAL_PARAMETER;
	}

	// buffer NULL`FbN & 4oCgACg`FbN
	// ToDo: K؂ȃAhXǂ`FbN
	if (param->buffer == NULL || (u32)param->buffer & 0x03)
	{
        return TWL_MIC_RESULT_ILLEGAL_PARAMETER;
	}

	// size 4oCg̔{`FbN
	// ToDo: buffer + size K؂ȃ͈͂
	if (param->size & 0x03)
	{
        return TWL_MIC_RESULT_ILLEGAL_PARAMETER;
	}

	// frequency`FbN
	if ( param->frequency > TWL_MIC_FREQUENCY_1_4 )
	{
        return TWL_MIC_RESULT_ILLEGAL_PARAMETER;
	}

	// bN
    enabled = OS_DisableInterrupts();
    if (micWork.lock)
    {
        (void)OS_RestoreInterrupts(enabled);
        return TWL_MIC_RESULT_BUSY;
    }
    micWork.lock = TRUE;
    (void)OS_RestoreInterrupts(enabled);

    // 񓯊֐pR[obNݒ
    micWork.callback = callback;
    micWork.callbackArg = callbackArg;

	// FULLR[obNݒ
	if (param->full_callback)
	{
		micWork.full_callback = param->full_callback;
		micWork.full_arg = param->full_arg;
	}

    // f[^쐬
    data[0] = (u8)param->dmaNo;
    MIC_PACK_U32(&data[1], &param->buffer);
    MIC_PACK_U32(&data[5], &param->size);
    data[9]  = (u8)param->frequency;
    data[10] = (u8)param->loop_enable;

    // R}hM
    if (TWLMICi_SendPxiCommand(command, size, data[0]) == FALSE)
    {
        return TWL_MIC_RESULT_SEND_ERROR;
    }
    for (i = 1; i < size; i+=3) {
        TWLMICi_SendPxiData(&data[i]);
    }

    return TWL_MIC_RESULT_SUCCESS;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_StartAutoSampling

  Description:  }CNTvOJniŁj

  Arguments:    param

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_StartAutoSampling(TwlMicAutoParam* param)
{
    micWork.result = TWL_MIC_StartAutoSamplingAsync(param, TWLMICi_GetResultCallback, 0);
    if (micWork.result == TWL_MIC_RESULT_SUCCESS)
    {
        TWLMICi_WaitBusy();
    }
    return micWork.result;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_StopAutoSamplingAsync

  Description:  }CNTvO~i񓯊Łj

  Arguments:    none

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_StopAutoSamplingAsync( TwlMicCallback callback, void* callbackArg )
{
    const TWLMICPxiCommand  command = TWL_MIC_PXI_COMMAND_AUTO_STOP;
    const u8                size = TWL_MIC_PXI_SIZE_AUTO_STOP;	// oCg
    OSIntrMode enabled;

	// bN
    enabled = OS_DisableInterrupts();
    if (micWork.lock)
    {
        (void)OS_RestoreInterrupts(enabled);
        return TWL_MIC_RESULT_BUSY;
    }
    micWork.lock = TRUE;
    (void)OS_RestoreInterrupts(enabled);

    // R[obNݒ
    micWork.callback = callback;
    micWork.callbackArg = callbackArg;

    // R}hM
    if (TWLMICi_SendPxiCommand(command, size, 0) == FALSE)
    {
        return TWL_MIC_RESULT_SEND_ERROR;
    }

    return TWL_MIC_RESULT_SUCCESS;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_StopAutoSampling

  Description:  }CNTvO~iŁj

  Arguments:    param

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_StopAutoSampling( void )
{
    micWork.result = TWL_MIC_StopAutoSamplingAsync(TWLMICi_GetResultCallback, 0);
    if (micWork.result == TWL_MIC_RESULT_SUCCESS)
    {
        TWLMICi_WaitBusy();
    }
    return micWork.result;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_GetLastSamplingAddressAsync

  Description:  ŐṼTvOf[^̊i[AhXԂ܂B
				AAAhX̓TvOԂɗ_IɌvZꂽ
				̂ł邽ߌ덷܂ł܂B
				i񓯊Łj

  Arguments:    adress      : AhXi[|C^̃AhX
				callback    :
				callbackArg :  

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_GetLastSamplingAddressAsync( void** adress, TwlMicCallback callback, void* callbackArg )
{
    const TWLMICPxiCommand  command = TWL_MIC_PXI_COMMAND_GET_LAST_SAMPLING_ADDRESS;
    const u8                size = TWL_MIC_PXI_SIZE_GET_LAST_SAMPLING_ADDRESS;	// oCg
    OSIntrMode enabled;

	// bN
    enabled = OS_DisableInterrupts();
    if (micWork.lock)
    {
        (void)OS_RestoreInterrupts(enabled);
        return TWL_MIC_RESULT_BUSY;
    }
    micWork.lock = TRUE;
    (void)OS_RestoreInterrupts(enabled);

    // R[obNݒ
    micWork.callback = callback;
    micWork.callbackArg = callbackArg;

	// AhXi[AhXۑ
	micWork.pLastSamplingAddress = adress;

    // R}hM
    if (TWLMICi_SendPxiCommand(command, size, 0) == FALSE)
    {
        return TWL_MIC_RESULT_SEND_ERROR;
    }

    return TWL_MIC_RESULT_SUCCESS;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_GetLastSamplingAddress

  Description:  ŐṼTvOf[^̊i[AhXԂ܂B
				AAAhX̓TvOԂɗ_IɌvZꂽ
				̂ł邽ߌ덷܂ł܂B
				iŁj

  Arguments:    adress      : AhXi[|C^̃AhX

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_GetLastSamplingAddress( void** adress )
{
    micWork.result = TWL_MIC_GetLastSamplingAddressAsync( adress, TWLMICi_GetResultCallback, 0);
    if (micWork.result == TWL_MIC_RESULT_SUCCESS)
    {
        TWLMICi_WaitBusy();
    }
    return micWork.result;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_SetAmpGainAsync

  Description:  vO}uQCAv̐ݒs܂Bi񓯊Łj
				̊֐Őݒ肵QC̓I[gQCRg[
@@@@@@@@ɂȂĂƂ̂ݗLƂȂ邱ƂɒӂĂB

  Arguments:    gain 		: ݒQCi0`119 = 0`59.5dBj
				callback    :
				callbackArg :  

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_SetAmpGainAsync( u8 gain, TwlMicCallback callback, void* callbackArg )
{
    const TWLMICPxiCommand  command = TWL_MIC_PXI_COMMAND_SET_AMP_GAIN;
    const u8                size = TWL_MIC_PXI_SIZE_SET_AMP_GAIN;	// oCg
    u8  data[size+2];
    OSIntrMode enabled;

	// ݒQC͈̓`FbN
	if ( gain > 119)
	{
        return TWL_MIC_RESULT_ILLEGAL_PARAMETER;
	}

	// bN
    enabled = OS_DisableInterrupts();
    if (micWork.lock)
    {
        (void)OS_RestoreInterrupts(enabled);
        return TWL_MIC_RESULT_BUSY;
    }
    micWork.lock = TRUE;
    (void)OS_RestoreInterrupts(enabled);

    // R[obNݒ
    micWork.callback = callback;
    micWork.callbackArg = callbackArg;

    // f[^쐬
    data[0] = gain;

    // R}hM
    if (TWLMICi_SendPxiCommand(command, size, data[0]) == FALSE)
    {
        return TWL_MIC_RESULT_SEND_ERROR;
    }

    return TWL_MIC_RESULT_SUCCESS;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_SetAmpGain

  Description:  vO}uQCAv̐ݒs܂BiŁj
				̊֐Őݒ肵QC̓I[gQCRg[
				ɂȂĂƂ̂ݗLƂȂ邱ƂɒӂĂB

  Arguments:    gain 		: ݒQCi0`119 = 0`59.5dBj

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_SetAmpGain( u8 gain )
{
    micWork.result = TWL_MIC_SetAmpGainAsync( gain, TWLMICi_GetResultCallback, 0);
    if (micWork.result == TWL_MIC_RESULT_SUCCESS)
    {
        TWLMICi_WaitBusy();
    }
    return micWork.result;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_GetAmpGainAsync

  Description:  TWL[h̃}CNQC(PGAB)擾܂B
				i񓯊Łj

  Arguments:    gain        : }CNQCl̊i[AhX
				callback    :
				callbackArg :  

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_GetAmpGainAsync( u8* gain, TwlMicCallback callback, void* callbackArg )
{
    const TWLMICPxiCommand  command = TWL_MIC_PXI_COMMAND_GET_AMP_GAIN;
    const u8                size = TWL_MIC_PXI_SIZE_GET_AMP_GAIN;	// oCg
    OSIntrMode enabled;

	// NULL`FbN
	if ( gain == NULL)
	{
        return TWL_MIC_RESULT_ILLEGAL_PARAMETER;
	}

	// bN
    enabled = OS_DisableInterrupts();
    if (micWork.lock)
    {
        (void)OS_RestoreInterrupts(enabled);
        return TWL_MIC_RESULT_BUSY;
    }
    micWork.lock = TRUE;
    (void)OS_RestoreInterrupts(enabled);

    // R[obNݒ
    micWork.callback = callback;
    micWork.callbackArg = callbackArg;

	// QCi[AhXۑ
	micWork.pAmpGain = gain;

    // R}hM
    if (TWLMICi_SendPxiCommand(command, size, 0) == FALSE)
    {
        return TWL_MIC_RESULT_SEND_ERROR;
    }

    return TWL_MIC_RESULT_SUCCESS;
}

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_GetAmpGain

  Description:  TWL[h̃}CNQC(PGAB)擾܂B
				iŁj

  Arguments:    gain        : }CNQCl̊i[AhX

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_GetAmpGain( u8* gain )
{
    micWork.result = TWL_MIC_GetAmpGainAsync( gain, TWLMICi_GetResultCallback, 0);
    if (micWork.result == TWL_MIC_RESULT_SUCCESS)
    {
        TWLMICi_WaitBusy();
    }
    return micWork.result;
}

/*---------------------------------------------------------------------------*
  Name:         TWLMICi_SendPxiCommand

  Description:  w擪R}hPXIoRARM7ɑMB

  Arguments:    command     - ΏۃR}h
                size        - Mf[^TCY (oCgP)
                data        - 擪f[^ (1oCĝ)

  Returns:      BOOL     - PXIɑ΂đMꍇTRUEA
                           PXIɂ鑗MɎsꍇFALSEԂB
 *---------------------------------------------------------------------------*/
static BOOL TWLMICi_SendPxiCommand(TWLMICPxiCommand command, u8 size, u8 data)
{
    u32 pxiData = (u32)(TWL_MIC_PXI_START_BIT |
            ((command << TWL_MIC_PXI_COMMAND_SHIFT) & TWL_MIC_PXI_COMMAND_MASK) |
            ((size << TWL_MIC_PXI_DATA_NUMS_SHIFT) & TWL_MIC_PXI_DATA_NUMS_MASK) |
            ((data << TWL_MIC_PXI_1ST_DATA_SHIFT) & TWL_MIC_PXI_1ST_DATA_MASK));
    if (0 > PXI_SendWordByFifo(PXI_FIFO_TAG_TWL_MIC, pxiData, 0))
    {
        return FALSE;
    }
    return TRUE;
}

/*---------------------------------------------------------------------------*
  Name:         TWLMICi_SendPxiData

  Description:  w㑱f[^PXIoRARM7ɑMB

  Arguments:    pData   - 3oCgf[^̐擪ւ̃|C^

  Returns:      None
 *---------------------------------------------------------------------------*/
static void TWLMICi_SendPxiData(u8 *pData)
{
    u32 pxiData = (u32)((pData[0] << 16) | (pData[1] << 8) | pData[2]);
    while (0 > PXI_SendWordByFifo(PXI_FIFO_TAG_TWL_MIC, pxiData, 0))
    {
    }
}

/*---------------------------------------------------------------------------*
  Name:         TWLMICi_PxiCallback

  Description:  񓯊֐p̋ʃR[obN֐B

  Arguments:    tag -  PXI tag which show message type.
                data - message from ARM7.
                err -  PXI transfer error flag.

  Returns:      None.
 *---------------------------------------------------------------------------*/
static void TWLMICi_PxiCallback(PXIFifoTag tag, u32 data, BOOL err)
{
#pragma unused( tag )
    TWLMICResult   result;

    // PXIʐMG[mF
    if (err)
    {
        // V[PXI
        TWLMICi_CallbackAndUnlock(TWL_MIC_RESULT_FATAL_ERROR);
        return;
    }
    // 擪f[^
    if (data & TWL_MIC_PXI_START_BIT)
    {
        // Mf[^
        SDK_ASSERT((data & TWL_MIC_PXI_RESULT_BIT) == TWL_MIC_PXI_RESULT_BIT);
        micWork.total = (u8)((data & TWL_MIC_PXI_DATA_NUMS_MASK) >> TWL_MIC_PXI_DATA_NUMS_SHIFT);
        micWork.current = 0;
        micWork.command = (TWLMICPxiCommand)((data & TWL_MIC_PXI_COMMAND_MASK) >> TWL_MIC_PXI_COMMAND_SHIFT);
        micWork.pxiResult = (TWLMICPxiResult)((data & TWL_MIC_PXI_1ST_DATA_MASK) >> TWL_MIC_PXI_1ST_DATA_SHIFT);
    }
    // 㑱f[^
    else
    {
        if (micWork.current < TWL_MIC_PXI_DATA_SIZE_MAX)
        {
            micWork.data[micWork.current++] = (u8)((data & 0xFF0000) >> 16);
        }
        if (micWork.current < TWL_MIC_PXI_DATA_SIZE_MAX)
        {
            micWork.data[micWork.current++] = (u8)((data & 0x00FF00) >> 8);
        }
        if (micWork.current < TWL_MIC_PXI_DATA_SIZE_MAX)
        {
            micWork.data[micWork.current++] = (u8)((data & 0x0000FF) >> 0);
        }
    }

    if (micWork.current >= micWork.total-1)   // > ͖͂
    {
        // ʂmF
        switch (micWork.pxiResult)
        {
	        case TWL_MIC_PXI_RESULT_SUCCESS:
	            result = TWL_MIC_RESULT_SUCCESS;
	            break;
	        case TWL_MIC_PXI_RESULT_INVALID_COMMAND:
	            result = TWL_MIC_RESULT_INVALID_COMMAND;
	            break;
	        case TWL_MIC_PXI_RESULT_INVALID_PARAMETER:
	            result = TWL_MIC_RESULT_ILLEGAL_PARAMETER;
	            break;
	        case TWL_MIC_PXI_RESULT_ILLEGAL_STATUS:
	            result = TWL_MIC_RESULT_ILLEGAL_STATUS;
	            break;
	        case TWL_MIC_PXI_RESULT_BUSY:
	            result = TWL_MIC_RESULT_BUSY;
	            break;
	        default:
	            result = TWL_MIC_RESULT_FATAL_ERROR;
        }

		switch (micWork.command)
		{
			case TWL_MIC_PXI_COMMAND_MIC_BUFFER_FULL:
				// FULLR[obŇĂяo
				if (micWork.full_callback)
				{
					micWork.full_callback( result, micWork.full_arg );
				}
				break;

			case TWL_MIC_PXI_COMMAND_ONE_SAMPLING:
				// PTvOʊi[
				*micWork.pOneBuffer = (u16)(micWork.data[0] | (micWork.data[1] << 8));
		        // 񓯊֐pR[obŇĂяobN
		        TWLMICi_CallbackAndUnlock(result);
				break;

			case TWL_MIC_PXI_COMMAND_GET_LAST_SAMPLING_ADDRESS:
				// ŐVTvOf[^AhXi[
				*micWork.pLastSamplingAddress = (void *)(micWork.data[0] | (micWork.data[1] << 8) | (micWork.data[2] << 16) | (micWork.data[3] << 24));
		        // 񓯊֐pR[obŇĂяobN
		        TWLMICi_CallbackAndUnlock(result);
				break;

			case TWL_MIC_PXI_COMMAND_GET_AMP_GAIN:
				// AvQCi[
				*micWork.pAmpGain = micWork.data[0];
		        // 񓯊֐pR[obŇĂяobN
		        TWLMICi_CallbackAndUnlock(result);
				break;

			default:
		        // 񓯊֐pR[obŇĂяobN
		        TWLMICi_CallbackAndUnlock(result);
		}
    }
}

/*---------------------------------------------------------------------------*
  Name:         TWLMICi_CallbackAndUnlock

  Description:  R[obŇĂяoƃbN̉s

  Arguments:    result  - ARM7瑗ꂽ

  Returns:      None.
 *---------------------------------------------------------------------------*/
static void TWLMICi_CallbackAndUnlock(TWLMICResult result)
{
    TwlMicCallback cb;

	// bN
    if (micWork.lock)
    {
        micWork.lock = FALSE;
    }

	// 񓯊֐pR[obNĂяo
    if (micWork.callback)
    {
        cb = micWork.callback;
        micWork.callback = NULL;
        cb(result, micWork.callbackArg);
    }
}

/*---------------------------------------------------------------------------*
  Name:         TWLMICi_GetResultCallback

  Description:  ֐Ŏs񓯊֐̌ʂ擾邽߂
				gpB

  Arguments:    result - 񓯊֐̏ʁB
                arg    - gpȂB

  Returns:      None.
 *---------------------------------------------------------------------------*/
static void TWLMICi_GetResultCallback(TWLMICResult result, void *arg)
{
#pragma unused( arg )

    micWork.result = result;
}

/*---------------------------------------------------------------------------*
  Name:         TWLMICi_WaitBusy

  Description:  MIC̔񓯊bNĂԑ҂B

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
#if 0
#include    <nitro/code32.h>
static asm void TWLMICi_WaitBusy(void)
{
    ldr     r12,    =micWork.lock
loop:
    ldr     r0,     [ r12,  #0 ]
    cmp     r0,     #TRUE
    beq     loop
    bx      lr
}
#include    <nitro/codereset.h>
#else
extern void PXIi_HandlerRecvFifoNotEmpty(void);
static void TWLMICi_WaitBusy(void)
{
    volatile BOOL *p = &micWork.lock;

    while (*p)
    {
        if (OS_GetCpsrIrq() == OS_INTRMODE_IRQ_DISABLE)
        {
            PXIi_HandlerRecvFifoNotEmpty();
        }
    }
}
#endif
