/*---------------------------------------------------------------------------*
  Project:  TwlSDK - tests - camera
  File:     main.c

  Copyright 2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Log: main.c,v $
  $NoKeywords: $
 *---------------------------------------------------------------------------*/

#include <twl.h>
#include <twl/camera.h>

#define DMA_NO  5           // gpDMAԍ(4-7)
#define DMA_IE  OS_IE_DMA5  // Ή銄荞
#define WIDTH   256         // C[W̕
#define HEIGHT  192         // C[W̍

#define LINES_AT_ONCE   CAMERA_GET_MAX_LINES(WIDTH)     // ̓]C
#define BYTES_PER_LINE  CAMERA_GET_LINE_BYTES(WIDTH)    // ꃉC̓]oCg

#if 0
    J瑗Ăf[^At[Pʂ菬PʂŏꍇA
    CAMERA_DmaRecvInfinity() płBƂp邵ȂB
    (DMAȂňSɃJ̃f[^擾iȂƂɒӁIII)
#endif

static void VBlankIntr(void);
static void CameraDmaIntr(void);
static void CameraErrIntr(void);

static int lineNumber = 0;
static BOOL effectFlag = FALSE;
static u16 pipeBuffer[BYTES_PER_LINE * LINES_AT_ONCE / sizeof(u16)] ATTRIBUTE_ALIGN(32);
/*---------------------------------------------------------------------------*
  Name:         TwlMain

  Description:  main

  Arguments:    None

  Returns:      None
 *---------------------------------------------------------------------------*/
void TwlMain()
{
    CAMERAResult result;

    // 
    OS_Init();
    OS_InitThread();
    GX_Init();
    OS_InitTick();

    // VuN荞ݐݒ
    OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrq();
    (void)GX_VBlankIntr(TRUE);
    (void)OS_EnableInterrupts();

    // VRAM\[h
    GX_SetBankForLCDC(GX_VRAM_LCDC_A);
    MI_CpuClearFast((void *)HW_LCDC_VRAM_A, WIDTH*HEIGHT*2);
    GX_SetGraphicsMode(GX_DISPMODE_VRAM_A, GX_BGMODE_0, GX_BG0_AS_2D);
    //GX_SetDispSelect(GX_DISP_SELECT_SUB_MAIN);
    OS_WaitVBlankIntr();
    GX_DispOn();

    // J
    CAMERA_Init();      // wakeup camera module

    result = CAMERA_I2CActivate(CAMERA_SELECT_IN);
    if (result != CAMERA_RESULT_SUCCESS)
    {
        OS_TPrintf("CAMERA_I2CActivate was failed. (%d)\n", result);
    }
#if 0
    result = CAMERA_I2CResize(CAMERA_SELECT_IN, 320, 240);
    if (result != CAMERA_RESULT_SUCCESS)
    {
        OS_TPrintf("CAMERA_I2CResize was failed. (%d)\n", result);
    }
#endif
    CAMERA_SetTrimmingParamsCenter(WIDTH, HEIGHT, 320, 240);    // clipped by camera i/f
    CAMERA_SetTrimming(TRUE);
    CAMERA_SetOutputFormat(CAMERA_OUTPUT_RGB);
    CAMERA_SetTransferLines(CAMERA_GET_MAX_LINES(WIDTH));

    // DMA荞ݐݒ
    OS_SetIrqFunction(DMA_IE, CameraDmaIntr);
    (void)OS_EnableIrqMask(DMA_IE);

    // DMAX^[g (IɎ~߂ȂiɗL)
    MIi_StopExDma(DMA_NO);
    CAMERA_DmaPipeInfinity(DMA_NO, pipeBuffer, BYTES_PER_LINE * LINES_AT_ONCE);

    // J荞ݐݒ
    CAMERA_SetBufferErrorIntrrupt(TRUE);
    CAMERA_SetMasterIntrrupt(TRUE);
    OS_SetIrqFunction(OS_IE_CAM, CameraErrIntr);
    (void)OS_EnableIrqMask(OS_IE_CAM);

    // JX^[g
    lineNumber = 0;
    CAMERA_ClearBuffer();
    CAMERA_StartCapture();
    OS_TPrintf("Camera is shooting a movie...\n");

    while (1)
    {
        u16 pad;
        u16 trg;
        static u16 old = 0xffff;    // ignore the trigger by first data

        OS_WaitVBlankIntr();

        pad = PAD_Read();
        trg = (u16)(pad & ~old);
        old = pad;

        if (trg & PAD_BUTTON_A)
        {
            effectFlag ^= 1;
            OS_TPrintf("Effect %s\n", effectFlag ? "ON" : "OFF");
        }
    }
}

//--------------------------------------------------------------------------------
//    uuN荞ݏ
//
void VBlankIntr(void)
{
    OS_SetIrqCheckFlag(OS_IE_V_BLANK); // checking VBlank interrupt
}

//--------------------------------------------------------------------------------
//    DMA荞ݏ (dH)
//
#define PRINT_RATE  32
void CameraDmaIntr(void)
{
    static BOOL effect = FALSE;
    OS_SetIrqCheckFlag(DMA_IE); // checking dma interrupt

    // Kvȏăt[obt@ɃRs[
    if (effect)
    {
        int i;
        u32 *src = (u32*)pipeBuffer;
        u32 *dest = (u32*)HW_LCDC_VRAM_A + lineNumber * WIDTH / 2;
        for (i = 0; i < WIDTH * LINES_AT_ONCE / 2; i++) // pack 2 pixel data
        {
            dest[i] = ~src[i];  // reverse (if using RGBA format, should bitORed with 0x10001000)
        }
    }
    else
    {
        MI_CpuCopy8(pipeBuffer, (u16*)HW_LCDC_VRAM_A + lineNumber * WIDTH, sizeof(pipeBuffer));
    }
    DC_InvalidateRange(pipeBuffer, sizeof(pipeBuffer));

    lineNumber += LINES_AT_ONCE;
    if (lineNumber >= HEIGHT)
    {
        static OSTick begin = 0;
        static int count = 0;
        // KvȂt[obt@̃XbvȂ

        effect = effectFlag;    // ̃^C~OŔf

        // debug print
        OS_TPrintf(".");
        if (begin == 0) // first time
        {
            begin = OS_GetTick();
        }
        else if (++count == PRINT_RATE)
        {
            OSTick uspf = OS_TicksToMicroSeconds(OS_GetTick() - begin) / count;
            int mfps = (int)(1000000000LL / uspf);
            OS_TPrintf("%2d.%03d fps\n", mfps / 1000, mfps % 1000);
            count = 0;
            begin = OS_GetTick();
        }

        lineNumber = 0;
    }
}

//--------------------------------------------------------------------------------
//    J荞ݏ (G[Ȃ͂)
//
#if 0
    DMA͕uvCŗǂH
#endif
void CameraErrIntr(void)
{
    OS_SetIrqCheckFlag(OS_IE_CAM);  // checking camera interrupt

    OS_TPrintf("Error was occurred.\n");

    // J~
    CAMERA_StopCapture();
    lineNumber = 0;

    // JĊJ
    CAMERA_ClearBuffer();
    CAMERA_StartCapture();
}
