/*---------------------------------------------------------------------------*
  Project:  TwlSDK - library - twl-mic
  File:     twl_mic_api.h

  Copyright 2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Log: $
  $NoKeywords: $
 *---------------------------------------------------------------------------*/

#ifndef TWL_MIC_MIC_API_H_
#define TWL_MIC_MIC_API_H_

#include <twl/types.h>

#ifdef __cplusplus
extern "C" {
#endif

/*===========================================================================*/

// ʒ`
typedef enum _TWLMICResult
{
    TWL_MIC_RESULT_SUCCESS = 0,
    TWL_MIC_RESULT_ILLEGAL_PARAMETER,
    TWL_MIC_RESULT_INVALID_COMMAND,
    TWL_MIC_RESULT_ILLEGAL_STATUS,
    TWL_MIC_RESULT_BUSY,
    TWL_MIC_RESULT_SEND_ERROR,
    TWL_MIC_RESULT_FATAL_ERROR,
    TWL_MIC_RESULT_MAX
}
TWLMICResult;

typedef enum
{
    TWL_MIC_FREQUENCY_ALL             =  0x0,		// 47.61 / 32.73 kHz
    TWL_MIC_FREQUENCY_1_2             =  0x1,		// 23.81 / 16.36 kHz
    TWL_MIC_FREQUENCY_1_3             =  0x2,		// 15.87 / 10.91 kHz
    TWL_MIC_FREQUENCY_1_4             =  0x3,		// 11.90 /  8.18 kHz
    TWL_MIC_FREQUENCY_NUM			  =  0x4
}
TwlMicFrequency;

// R[obN
typedef void (*TwlMicCallback)(TWLMICResult result, void* arg);

// I[gTvOpݒ`
typedef struct TwlMicAutoParam
{
	u32    			dmaNo;				// DMA No
    void   			*buffer;            // ʊi[obt@ւ̃|C^(DMAg4byteACgj
    u32     		size;               // obt@TCY(DMAg4bytePʁj
    TwlMicFrequency frequency;          // TvOpx
    BOOL    		loop_enable;        // obt@t̃[v
    TwlMicCallback 	full_callback;     	// obt@t̃R[obN
    void*			full_arg;   		// LR[obNɎw肷
}
TwlMicAutoParam;


/*---------------------------------------------------------------------------*
  Name:         MIC_Init

  Description:  MICCuB

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void 
TWL_MIC_Init(void);

/*---------------------------------------------------------------------------*
  Name:         MIC_End

  Description:  MICCuIB

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
TWL_MIC_End(void);

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_DoSamplingAsync

  Description:  }CNPTvOJni񓯊Łj

  Arguments:    

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_DoSamplingAsync(u16* buf,  TwlMicCallback callback, void* callbackArg);

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_DoSampling

  Description:  }CNPTvOJniŁj

  Arguments:    

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_DoSampling(u16* buf);

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_StartAutoSamplingAsync

  Description:  }CNTvOJni񓯊Łj

  Arguments:    param

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_StartAutoSamplingAsync(TwlMicAutoParam* param, TwlMicCallback callback, void* callbackArg);

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_StartAutoSampling

  Description:  }CNTvOJniŁj

  Arguments:    param

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_StartAutoSampling(TwlMicAutoParam* param);

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_StopAutoSampling

  Description:  }CNTvO~iŁj

  Arguments:    param

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_StopAutoSampling( void );

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_StopAutoSamplingAsync

  Description:  }CNTvO~i񓯊Łj

  Arguments:    none

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult 
TWL_MIC_StopAutoSamplingAsync(  TwlMicCallback callback, void* callbackArg );

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_GetLastSamplingAddressAsync

  Description:  ŐṼTvOf[^̊i[AhXԂ܂B
				AAAhX̓TvOԂɎZôł
				ߌ덷܂ł܂BARM7ARM9ւƒʒmĂ
				ԂɃTvO͐ił\܂B
				i񓯊Łj

  Arguments:    adress      : AhXi[|C^̃AhX
				callback    :
				callbackArg :  

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_GetLastSamplingAddressAsync( void** adress, TwlMicCallback callback, void* callbackArg );

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_GetLastSamplingAddress

  Description:  ŐṼTvOf[^̊i[AhXԂ܂B
				AAAhX̓TvOԂɎZôł
				ߌ덷܂ł܂BARM7ARM9ւƒʒmĂ
				ԂɃTvO͐ił\܂B
				iŁj

  Arguments:    adress      : AhXi[|C^̃AhX

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_GetLastSamplingAddress( void** adress );

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_SetAmpGainAsync

  Description:  vO}uQCAv̐ݒs܂Bi񓯊Łj
				̊֐Őݒ肵QC̓I[gQCRg[
				ɂȂĂƂ̂ݗLƂȂ邱ƂɒӂĂB

  Arguments:    gain 		: ݒQCi0`119 = 0`59.5dBj
				callback    :
				callbackArg :  

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_SetAmpGainAsync( u8 gain, TwlMicCallback callback, void* callbackArg );

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_SetAmpGain

  Description:  vO}uQCAv̐ݒs܂BiŁj
				̊֐Őݒ肵QC̓I[gQCRg[
				ɂȂĂƂ̂ݗLƂȂ邱ƂɒӂĂB

  Arguments:    gain 		: ݒQCi0`119 = 0`59.5dBj

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_SetAmpGain( u8 gain );

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_GetAmpGainAsync

  Description:  TWL[h̃}CNQC(PGAB)擾܂B
				i񓯊Łj

  Arguments:    gain        : }CNQCl̊i[AhX
				callback    :
				callbackArg :  

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_GetAmpGainAsync( u8* gain, TwlMicCallback callback, void* callbackArg );

/*---------------------------------------------------------------------------*
  Name:         TWL_MIC_GetAmpGain

  Description:  TWL[h̃}CNQC(PGAB)擾܂B
				iŁj

  Arguments:    gain        : }CNQCl̊i[AhX

  Returns:      TWLMICResult
 *---------------------------------------------------------------------------*/
TWLMICResult
TWL_MIC_GetAmpGain( u8* gain );

/*===========================================================================*/

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* TWL_MIC_MIC_API_H_ */
