/*---------------------------------------------------------------------------*
  Project:  TWL - ELF Loader
  File:     elf_loader.h

  Copyright 2006,2007 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#ifndef _ELF_LOADER_H_
#define _ELF_LOADER_H_

#include "types.h"
#include "elf.h"
#include "stdio.h"


/*------------------------------------------------------
  ZNVwb_g([hAhXɑΉ)
 -----------------------------------------------------*/
typedef struct {
  void*			next;
  u16			index;
  u16			debug_flag;		/*0:fobOłȂA1:fobO*/
  u32			loaded_adr;
  u32			alloc_adr;
  u32			loaded_size;
  Elf32_Shdr	Shdr;
}ELShdrEx;


/*------------------------------------------------------
  V{Ggg([hAhXɑΉ)
 -----------------------------------------------------*/
typedef struct {
  void*		next;
  u16		debug_flag;			/*0:fobOłȂA1:fobO*/
  u16		thumb_flag;
  u32		relocation_val;
  Elf32_Sym	Sym;
}ELSymEx;



/*------------------------------------------------------
  ELFIuWFNg̊Ǘ
 -----------------------------------------------------*/
typedef void (*ELi_ReadFunc)( void* buf, void* file_struct, u32 file_base, u32 file_offset, u32 size);
typedef struct {
  void*			ar_head;		/* AR܂ELFt@C̐擪AhX */
  void*			elf_offset;		/* ELFIuWFNg̐擪ւ̃ItZbg */
  void*			buf_current;	/* LoaderƗp */
  u16			shentsize;		/* 1ZNVwb_̃TCY */
  u16			process;		/* Ĕzu */

  ELShdrEx*		ShdrEx;			/* ShdrExXg̐擪 */

  Elf32_Ehdr	CurrentEhdr;	/* ELFwb_ */

  Elf32_Rel		Rel;			/* ĔzuGg */
  Elf32_Rela	Rela;
  Elf32_Sym		Sym;			/* V{Gg */
  Elf32_Shdr	SymShdr;

  ELSymEx*		SymEx;			/* SymExXg̐擪 */

  ELi_ReadFunc	ELi_ReadStub;	/* [hX^u֐ */
  void*			FileStruct;		/* t@C\ */

  u32			mem_adr;		/*ŏɃ[hꂽZNVsh_addr(DSROMwb_pp[^)*/
}ELHandle;



/*------------------------------------------------------
  AhXe[u
 -----------------------------------------------------*/
typedef struct {
  void*		next;				/*̃AhXGg*/
  char*		name;				/**/
  void*		adr;				/*AhX*/
  u16		func_flag;			/*0:f[^A1:֐*/
  u16		thumb_flag;			/*0:armR[hA1:thumbR[h*/
}ELAdrEntry;


/*------------------------------------------------------
  ̍Ĕzue[u

  ォAhXe[uQƂΎ̂悤ɉB
  S_ = AdrEntry.adr;
  T_ = (u32)(AdrEntry.thumb_flag);
 -----------------------------------------------------*/
typedef struct {
  void* next;					/*̃Gg*/
  char*	sym_str;				/*̊OQƃV{*/
  u32	r_type;					/*P[V^CviELF32_R_TYPE( Rela.r_info)j*/
  u32	S_;						/*̊OQƃV{AhX*/
  s32	A_;						/*ς*/
  u32	P_;						/*ς*/
  u32	T_;						/*̊OQƃV{ARM/ThumbtO*/
  u32	sh_type;				/*SHT_REL or SHT_RELA*/
  u32	remove_flag;			/*ƂɃZbgiĂǂƂʂjtO*/
  ELAdrEntry*	AdrEnt;			/*AhXe[uToGg̏ꏊ*/
}ELUnresolvedEntry;



/* ELHandle  processl */
#define EL_FAILED			0x00
#define EL_INITIALIZED		0x5A
#define EL_COPIED			0xF0
#define EL_RELOCATED		0xF1



/*---------------------------------------------------------
 ELFIuWFNg̃TCY߂
 --------------------------------------------------------*/
u32 EL_GetElfSize( const void* buf);

/*------------------------------------------------------
  _Ci~bNNVXe
 -----------------------------------------------------*/
void EL_Init( void);

/*------------------------------------------------------
  ELHandle\̂
 -----------------------------------------------------*/
BOOL EL_InitHandle( ELHandle* ElfHandle);

/*------------------------------------------------------
  ELFIuWFNg܂͂̃A[JCut@Cobt@ɍĔzu
 -----------------------------------------------------*/
u16 EL_LoadLibraryfromFile( ELHandle* ElfHandle, FILE* ObjFile, void* buf);

/*------------------------------------------------------
  ELFIuWFNg܂͂̃A[JCuobt@ɍĔzu
 -----------------------------------------------------*/
u16 EL_LoadLibraryfromMem( ELHandle* ElfHandle, void* obj_image, u32 obj_len, void* buf);

/*------------------------------------------------------
  AhXe[ugĖ̃V{
 -----------------------------------------------------*/
u16 EL_ResolveAllLibrary( void);


/*------------------------------------------------------
  }[LOꂽV{Jpt@Cɍ\̂Ƃďo
 -----------------------------------------------------*/
void EL_ExtractStaticSym1( void);
/*------------------------------------------------------
  }[LOꂽV{Jpt@CAPIƂďo
 -----------------------------------------------------*/
void EL_ExtractStaticSym2( void);


/*------------------------------------------------------
  AhXe[uGg폜
 -----------------------------------------------------*/
BOOL EL_RemoveAdrEntry( ELAdrEntry* AdrEnt);

/*------------------------------------------------------
  AhXe[uɃGgǉ
 -----------------------------------------------------*/
void EL_AddAdrEntry( ELAdrEntry* AdrEnt);

/*------------------------------------------------------
  AhXe[uw蕶ɊYGgT
 -----------------------------------------------------*/
ELAdrEntry* EL_GetAdrEntry( char* ent_name);

/*------------------------------------------------------
  AhXe[uw蕶ɊYAhXԂ
 -----------------------------------------------------*/
void* EL_GetGlobalAdr( char* ent_name);





/*ɕKvȊ֐*/
//[hɕKvȃ̃oCgZo֐
//EL_FreeLibrary

#endif	/*_ELF_LOADER_H_*/



